@@ -1,5 +1,5 @@
 ;;;;;
-;;;;; $Id: async.el,v 44.48 2002/10/20 13:20:00 byers Exp $
+;;;;; $Id: async.el,v 44.65 2004/11/12 10:54:56 _cvs_pont_lyskomelisp Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -37,7 +37,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: async.el,v 44.48 2002/10/20 13:20:00 byers Exp $\n"))
+	      "$Id: async.el,v 44.65 2004/11/12 10:54:56 _cvs_pont_lyskomelisp Exp $\n"))
 
 
 (defun lyskom-is-ignoring-async (buffer message &rest args)
@@ -97,7 +97,6 @@ this function shall be with current-buffer the BUFFER."
 	      'you-changed-name-to
 	      new-name
 	      (lyskom-default-button 'conf conf-no)))
-	 ;; (cache-del-conf-stat conf-no) ;+++Borde {ndra i cachen i st{llet.
 	 (let ((cached-stat (cache-get-conf-stat conf-no))
                (cached-ustat (cache-get-uconf-stat conf-no)))
 	   (when cached-stat
@@ -120,21 +119,16 @@ this function shall be with current-buffer the BUFFER."
          ;; Update in the mship-edit buffer
 
          (lp--maybe-update-entry-for-conf conf-no))))
-     
+
      ((eq msg-no 6)			;i_am_on - something is moving
-      (let ((info (lyskom-parse-who-info)))
-	(lyskom-save-excursion
-	 (set-buffer buffer)
-	 (if (or (null lyskom-pers-no)
-		 (zerop lyskom-pers-no))
-	     nil
-	   (cache-add-who-info info)))))
-     
+      (lyskom-parse-who-info))
+
      ((eq msg-no 7)			; Database is syncing.
       (lyskom-save-excursion
        (set-buffer buffer)
        ;; I removed the test for kom-presence-messages /david
-       (if (not (lyskom-is-in-minibuffer))
+       (if (and (not (lyskom-is-in-minibuffer))
+                kom-show-sync-messages)
 	   (lyskom-message "%s" (lyskom-get-string 'database-sync)))
        (setq mode-line-process (lyskom-get-string 'mode-line-saving))
        (setq lyskom-is-saving t)
@@ -157,7 +151,9 @@ this function shall be with current-buffer the BUFFER."
      
      ((eq msg-no 9)			; A person has logged in
       (let ((pers-no (lyskom-parse-num))
-	    (session-no (lyskom-parse-num)))
+	    ;;;(session-no (lyskom-parse-num))
+            )
+        (lyskom-parse-num)
 	(lyskom-save-excursion
 	 (set-buffer buffer)
 	 (if (and lyskom-pers-no
@@ -166,11 +162,6 @@ this function shall be with current-buffer the BUFFER."
 	     (initiate-get-conf-stat 'follow
 				     'lyskom-show-logged-in-person
 				     pers-no))
-	 (if (and lyskom-pers-no
-		  (not (zerop lyskom-pers-no))
-		  lyskom-who-info-buffer-is-on)
-	     (initiate-get-session-info 'who-buffer 'cache-add-session-info
-					session-no))
 	 )))
 
      ;; msg-no 10 is the old broadcast message. No longer used.
@@ -179,13 +170,6 @@ this function shall be with current-buffer the BUFFER."
       (lyskom-save-excursion
        (set-buffer buffer)
        (lyskom-insert-before-prompt (lyskom-get-string-sol 'lyskom-is-full))
-;;;       (if (and (eq major-mode 'lyskom-mode)
-;;;		(not (listp lyskom-time-last-command))
-;;;		kom-auto-quit-when-idle)
-;;;	   (progn
-;;;	     (lyskom-insert-before-prompt
-;;;	      (lyskom-get-string-sol 'session-auto-ended))
-;;;	      (kom-quit 1)
        ))
      
      
@@ -218,9 +202,7 @@ this function shall be with current-buffer the BUFFER."
 		      (lyskom-show-presence pers-no kom-presence-messages-in-buffer)))
 	     (initiate-get-conf-stat 'follow
 				     'lyskom-show-logged-out-person
-				     pers-no session-no))
-	 (if (and lyskom-pers-no (not (zerop lyskom-pers-no)))
-	     (lyskom-run 'who-buffer 'cache-del-who-info session-no)))))
+				     pers-no session-no)))))
 
      ((eq msg-no 14)                    ; Deleted text
       (let* ((text-no (lyskom-parse-num))
@@ -284,13 +266,81 @@ this function shall be with current-buffer the BUFFER."
               (when (eq pers-no lyskom-pers-no)
                 (lyskom-collect 'follow)
                 (initiate-get-conf-stat 'follow nil conf-no)
-                (initiate-query-read-texts 'follow nil pers-no conf-no)
+                (initiate-query-read-texts 'follow nil pers-no conf-no t 0)
                 (lyskom-use 'follow 'lyskom-async-new-membership pers-no conf-no))
             ))))
 
+     ((eq msg-no 19)                    ; async-new-user-area
+      (let* ((pers-no (lyskom-parse-num))
+             (old-user-area (lyskom-parse-num))
+             (new-user-area (lyskom-parse-num)))
+        (lyskom-save-excursion
+          (set-buffer buffer)
+          (when (and (eq pers-no lyskom-pers-no)
+                     (not (eq lyskom-current-user-area new-user-area)))
+            (initiate-get-pers-stat 'follow 
+                                    'lyskom-async-new-user-area
+                                    pers-no
+                                    old-user-area
+                                    new-user-area)))))
+
+     ((eq msg-no 20)                    ; async-new-presentation
+      (let* ((conf-no (lyskom-parse-num))
+             (old-pres (lyskom-parse-num))
+             (new-pres (lyskom-parse-num)))
+        (lyskom-ignore old-pres)
+        (lyskom-save-excursion
+          (set-buffer buffer)
+          (when (cache-get-conf-stat conf-no)
+            (set-conf-stat->presentation (cache-get-conf-stat conf-no)
+                                         new-pres)
+            (lp--maybe-update-entry-for-conf conf-no)))))
+
+     ((eq msg-no 21)                    ; async-new-motd
+      (let* ((conf-no (lyskom-parse-num))
+             (old-motd (lyskom-parse-num))
+             (new-motd (lyskom-parse-num)))
+        (lyskom-save-excursion
+          (set-buffer buffer)
+          (lyskom-ignore old-motd)
+          (when (cache-get-conf-stat conf-no)
+            (set-conf-stat->msg-of-day (cache-get-conf-stat conf-no)
+                                       new-motd)))))
+
+     ((eq msg-no 22)                    ; async-text-aux-changes
+      (let* ((text-no (lyskom-parse-num))
+             (deleted-aux (lyskom-parse-aux-item-list))
+             (added-aux (lyskom-parse-aux-item-list))
+             (text-stat nil))
+        (lyskom-save-excursion
+          (set-buffer buffer)
+          (setq text-stat (cache-get-text-stat text-no))
+          (when text-stat
+            (set-text-stat->aux-items text-stat
+                                      (lyskom-aux-item-modify-list
+                                       (text-stat->aux-items text-stat)
+                                       deleted-aux
+                                       added-aux))))))
      (t
       (lyskom-skip-tokens tokens)))))
 
+(defun lyskom-async-new-user-area (pers-stat old-user-area new-user-area)
+  (when pers-stat
+    (let ((need-reread (and (eq lyskom-pers-no (pers-stat->pers-no pers-stat))
+                            (not (eq lyskom-current-user-area new-user-area)))))
+  ;; Update the cache
+  (set-pers-stat->user-area pers-stat new-user-area)
+  ;; Re-read
+  (when (and need-reread
+             new-user-area
+             (not (zerop new-user-area)))
+    (initiate-get-text 'follow (lambda (text)
+                                 (when text
+                                   (lyskom-format-insert-before-prompt 'reading-settings-from-server)
+                                   (setq lyskom-saved-unknown-variables
+                                    (lyskom-read-options-eval text))))
+                       new-user-area)))))
+
 
 (defun lyskom-async-forced-leave-conf (conf-stat conf-no)
   (if conf-stat
@@ -330,10 +380,10 @@ this function shall be with current-buffer the BUFFER."
           ;; Already a member. Perhaps the priority changed.
           ;; Update the cache. The reading list is probably also
           ;; not quite correct since the priority might have changed
-          ;; FIXME: Maybe fix this.
 
           (cur-mship
-           (lyskom-replace-membership membership))
+           (lyskom-replace-membership membership)
+           (lyskom-sort-to-do-list))
 
           ;; Not a member. Completely new. Deal with it.
           (t (lyskom-add-membership membership conf-no))))))
@@ -353,9 +403,7 @@ according to the value of FLAG."
 
 (defun lyskom-show-logged-in-person (conf-stat)
   "Visa p} kommandoraden vem som loggat in."
-  (let ((server (or (cdr (lyskom-string-assoc lyskom-server-name
-                                              kom-server-aliases))
-                    lyskom-server-name)))
+  (let ((server (lyskom-session-nickname)))
     (cond
      ((lyskom-is-in-minibuffer))
      ((lyskom-show-presence (conf-stat->conf-no conf-stat)
@@ -374,20 +422,18 @@ according to the value of FLAG."
       (if conf-stat
           (lyskom-format-insert-before-prompt 'has-entered-r conf-stat
                                               (and kom-text-properties
-                                                   '(face kom-presence-face))
+                                                   `(face ,kom-presence-face))
                                               server)
         (lyskom-format-insert-before-prompt 'has-entered-r
                                             (lyskom-get-string 'unknown-person)
                                             (and kom-text-properties
-                                                 '(face kom-presence-face))
+                                                 `(face ,kom-presence-face))
                                             server))))))
 
 
 (defun lyskom-show-logged-out-person (conf-stat session-no)
   "Visa p} kommandoraden vem som loggat ut."
-  (let ((server (or (cdr (lyskom-string-assoc lyskom-server-name
-                                              kom-server-aliases))
-                    lyskom-server-name)))
+  (let ((server (lyskom-session-nickname)))
   (cond
    ((lyskom-is-in-minibuffer))
    ((lyskom-show-presence (conf-stat->conf-no conf-stat) 
@@ -403,45 +449,16 @@ according to the value of FLAG."
     (if conf-stat
 	(lyskom-format-insert-before-prompt 'has-left-r conf-stat
 					    (and kom-text-properties
-						 '(face kom-presence-face))
+						 `(face ,kom-presence-face))
                                             server)
       (lyskom-format-insert-before-prompt 'has-left-r
 					  (lyskom-get-string 'unknown-person)
 					  (and kom-text-properties
-					       '(face kom-presence-face))
+					       `(face ,kom-presence-face))
                                           server))))))
 
 
 
-(defun lyskom-show-changed-person (personconfstat conf-num doing)
-  "Tells the user what another person is doing."
-  (if personconfstat			;+++ Annan felhantering
-      (progn
-	(cond
-	 ((and (lyskom-show-presence (conf-stat->conf-no personconfstat)
-                                     kom-presence-messages-in-echo-area)
-	       (or (= 0 conf-num)
-		   (eq conf-num lyskom-current-conf))
-	       (/= 0 (length doing)))
-	  (lyskom-message "%s %s" (conf-stat->name personconfstat) 
-		   (let ((string
-			  (concat (char-to-string (downcase
-						   (string-to-char doing)))
-				  (substring doing 1))))
-		     string))))
-	(cond
-	 ((and (lyskom-show-presence (conf-stat->conf-no personconfstat)
-                                     kom-presence-messages-in-buffer)
-	       (or (= 0 conf-num)
-		   (eq conf-num lyskom-current-conf))
-	       (/= 0 (length doing)))
-	  (lyskom-format-insert-before-prompt
-	   "%#1M %#2s\n"
-	   personconfstat
-	   (concat (char-to-string (downcase (string-to-char doing)))
-		   (substring doing 1))))))))
-
-
 (defun lyskom-is-in-minibuffer ()
   "Returns non-nil if I am using the minibuffer for some reading."
   (or lyskom-inhibit-minibuffer-messages
@@ -449,7 +466,7 @@ according to the value of FLAG."
       (not (zerop (minibuffer-depth)))))
 
 
-(defun lyskom-show-personal-message (sender recipient message 
+ (defun lyskom-show-personal-message (sender recipient message 
                                             &optional when nobeep)
   "Insert a personal message into the lyskom buffer.
 Args: SENDER: conf-stat for the person sending the message.
@@ -460,13 +477,12 @@ Args: SENDER: conf-stat for the person sending the message.
       NOBEEP: True means don't beep. No matter what."
   (lyskom-insert-personal-message sender recipient message when nobeep)
   (setq lyskom-last-personal-message-sender 
-        (if (stringp sender) sender (conf-stat->name sender)))
+        (if (stringp sender) sender (conf-stat->conf-no sender)))
   (setq lyskom-last-group-message-recipient 
         (if (and recipient
                  (not (eq 0 recipient))
-                 (not (eq (conf-stat->conf-no recipient)
-                          lyskom-pers-no)))
-            (conf-stat->name recipient)
+                 (not (eq (conf-stat->conf-no recipient) lyskom-pers-no)))
+            (conf-stat->conf-no recipient)
           nil))
   (run-hooks 'lyskom-personal-message-hook)
   (run-hooks 'kom-personal-message-hook))
@@ -529,9 +545,11 @@ Non-nil NOBEEP means don't beep."
                           message
                           when
                           (when kom-async-highlight-dashed-lines
-                            '(face kom-async-dashed-lines-face))
+                            `(face ,(or kom-async-dashed-lines-face
+                                        lyskom-default-async-dashed-lines-face)))
                           (when kom-async-highlight-text-body
-                            '(face kom-async-text-body-face))))
+                            `(face ,(or kom-async-text-body-face
+                                        lyskom-default-async-text-body-face)))))
           ((= (conf-stat->conf-no recipient) lyskom-pers-no) ; Private
            (if (not nobeep) (lyskom-beep kom-ding-on-personal-messages sender))
            (lyskom-format (lyskom-get-string-sol 'message-from)
@@ -542,9 +560,11 @@ Non-nil NOBEEP means don't beep."
                           message
                           when
                           (when kom-async-highlight-dashed-lines
-                            '(face kom-async-dashed-lines-face))
+                            `(face ,(or kom-async-dashed-lines-face
+                                        lyskom-default-async-dashed-lines-face)))
                           (when kom-async-highlight-text-body
-                            '(face kom-async-text-body-face))))
+                            `(face ,(or kom-async-text-body-face
+                                        lyskom-default-async-text-body-face)))))
           (t                            ; Group message
            (if (not nobeep) (lyskom-beep kom-ding-on-group-messages recipient))
            (lyskom-format (lyskom-get-string-sol 'message-from-to)
@@ -559,9 +579,11 @@ Non-nil NOBEEP means don't beep."
                            (t (lyskom-get-string 'unknown)))
                           when
                           (when kom-async-highlight-dashed-lines
-                            '(face kom-async-dashed-lines-face))
+                            `(face ,(or kom-async-dashed-lines-face
+                                        lyskom-default-async-dashed-lines-face)))
                           (when kom-async-highlight-text-body
-                            '(face kom-async-text-body-face)))))))
+                            `(face ,(or kom-async-text-body-face
+                                        lyskom-default-async-text-body-face))))))))
 
 
   
@@ -597,7 +619,8 @@ converted, before insertion."
            (lyskom-insert string )
            (if pop
                (save-selected-window
-                 (select-window (lyskom-display-buffer (current-buffer)))
+                 (select-window (lyskom-display-buffer (current-buffer)
+                                                       (not (eq pop t))))
                  (goto-char (point-max))
                  (recenter -1))))))))))
   
@@ -623,10 +646,13 @@ converted, before insertion."
   (if (and kom-mark-read-texts-as-read-in-new-recipient
            (lyskom-text-read-at-least-once-p text-stat t)
            (not (eq conf-no lyskom-pers-no)))
-      (initiate-mark-as-read 'follow
-                             nil
-                             conf-no 
-                             (list text-no))
+      (lyskom-traverse misc-info (text-stat->misc-info-list text-stat)
+        (when (and (memq (misc-info->type misc-info) lyskom-recpt-types-list)
+                   (eq conf-no (misc-info->recipient-no misc-info)))
+          (initiate-mark-as-read 'follow
+                                 nil
+                                 conf-no 
+                                 (list (misc-info->local-no misc-info)))))
 
     ;; Text is previously unread or in the mailbox
     (let ((local-no nil))
