@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;;; $Id: aux-items.el,v 44.35 2002/06/12 18:29:32 byers Exp $
+;;;;; $Id: aux-items.el,v 44.42 2004/02/24 12:23:16 byers Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -34,13 +34,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: aux-items.el,v 44.35 2002/06/12 18:29:32 byers Exp $\n"))
-
-;;; (eval-when-compile
-;;;   (require 'lyskom-defvar "defvar.el")
-;;;   (require 'lyskom-vars "vars.el")
-;;;   (require 'lyskom-types "komtypes.el"))
-
+	      "$Id: aux-items.el,v 44.42 2004/02/24 12:23:16 byers Exp $\n"))
 
 (def-kom-var lyskom-aux-item-definitions nil
   "List of aux item definitions.")
@@ -58,16 +52,6 @@
 (defun lyskom-aux-item-definition->number (def)
   (car def))
 
-(defun lyskom-map-aux-items (fn text-stat)
-  "Apply FN to every aux item in TEXT-STAT."
-  (let ((items (text-stat->aux-items text-stat)))
-    (while items
-      (when (and (car items) 
-                 (not (aux-item-flags->deleted (aux-item->flags items))))
-        (funcall fn (car items)))
-      (setq items (cdr items)))))
-
-
 
 (defun lyskom-aux-item-try-call (item method default &rest args)
   (when (lyskom-aux-item-p item)
@@ -119,7 +103,6 @@ return non-nil if the item is to be included in the list."
       (setq item-list (cdr item-list)))
     (nreverse result)))
 
-                                        
 (defun lyskom-get-aux-item (item-list tag)
   "Return all aux-items in ITEM-LIST with tag TAG."
   (lyskom-match-aux-items item-list
@@ -152,6 +135,28 @@ return non-nil if the item is to be included in the list."
                             (aux-item->data item)
 			    item))
 
+(defun lyskom-aux-item-modify-list (item-list deleted added)
+  "Return a copy of ITEM-LIST with items in DELETED removed and items
+in ADDED added."
+  (let ((new-nos (nconc (mapcar 'aux-item->aux-no deleted)
+                        (mapcar 'aux-item->aux-no added))))
+    (nconc (filter-list (lambda (el) (not (memq (aux-item->aux-no el) new-nos)))
+                        item-list)
+           added)))
+
+(defun lyskom-aux-item-validate (data &rest tests)
+  "Validata aux-item data.
+DATA is data to validate. TESTS are the tests to use.
+A test can be a function or a regular expression to match.
+Invalid tests are silently ignored."
+  (not (lyskom-traverse test tests
+         (condition-case nil
+             (unless
+                 (cond ((stringp test) (string-match test data))
+                       ((functionp test) (funcall test data)))
+               (lyskom-traverse-break t))
+           (error (lyskom-traverse-break t))))))
+
 
 ;;; ======================================================================
 
@@ -330,7 +335,7 @@ return non-nil if the item is to be included in the list."
 
 (def-aux-item send-comments-to 33
   (text-name aux-send-comments-to-name)
-  (info . lyskom-print-aux-item-info)
+  (info . lyskom-aux-item-info)
   (status-print . lyskom-print-send-comments-to))
 
 (def-aux-item world-readable 34
@@ -341,6 +346,11 @@ return non-nil if the item is to be included in the list."
   (edit-insert . lyskom-edit-insert-world-readable)
   (text-print-when . header))
 
+(def-aux-item mx-refuse-import 35
+  (text-name aux-mx-refuse-import)
+  (info . lyskom-aux-item-info)
+  (status-print . lyskom-print-mx-refuse-import))
+
 (def-aux-item elisp-client-read-faq 10000
   (text-name aux-elisp-client-read-faq-name)
   (info . lyskom-aux-item-info)
@@ -359,10 +369,10 @@ return non-nil if the item is to be included in the list."
 (defun lyskom-aux-item-decode-data (item)
   (set-aux-item->data 
    item
-   (decode-coding-string (aux-item->data item) lyskom-server-coding-system)))
+   (lyskom-decode-coding-string (aux-item->data item) lyskom-server-coding-system)))
 
 (defun lyskom-aux-item-encode-data (item)
-  (encode-coding-string (aux-item->data item) lyskom-server-coding-system)
+  (lyskom-encode-coding-string (aux-item->data item) lyskom-server-coding-system)
   )
 
 
@@ -433,6 +443,10 @@ return non-nil if the item is to be included in the list."
                        pers)
                   pers
                 (aux-item->creator item))))
+
+    ;; We use string-to-int here since we handle floats in 
+    ;; lyskom-format.
+
     (concat
      (cond ((string-match "^P\\([0-9]+\\)" (aux-item->data item))
             (lyskom-format 'cross-reference-pers-aux 
@@ -460,6 +474,8 @@ return non-nil if the item is to be included in the list."
      )))
 
 (defun lyskom-status-print-cross-reference (item &optional obj pers)
+  ;; We use string-to-int here since we handle floats in 
+  ;; lyskom-format.
   (lyskom-insert 
    (concat
     (cond ((string-match "^P\\([0-9]+\\)" (aux-item->data item))
@@ -530,8 +546,11 @@ return non-nil if the item is to be included in the list."
 (defun lyskom-faq-for-conf-action (text-stat)
   (let ((faqs (text-stat-find-aux text-stat 28)))
     (lyskom-traverse aux faqs
-      (lyskom-register-read-faq (string-to-int (aux-item->data aux))
-                                (text-stat->text-no text-stat)))))
+      (condition-case nil
+          (lyskom-register-read-faq (lyskom-string-to-int
+                                     (aux-item->data aux) t)
+                                    (text-stat->text-no text-stat))
+       (lyskom-integer-conversion-error nil)))))
 
 (defun lyskom-request-confirmation-action (text-stat)
   (let ((confirmations (text-stat-find-aux text-stat 7))
@@ -568,6 +587,8 @@ return non-nil if the item is to be included in the list."
 
 
 (defun lyskom-print-redirect (item &optional obj)
+  ;; We use string-to-int here since we handle floats in 
+  ;; lyskom-format.
   (concat
    (cond ((string-match "^E-mail:\\(.*\\)$" (aux-item->data item))
           (lyskom-format 'redirect-email-aux 
@@ -590,7 +611,7 @@ return non-nil if the item is to be included in the list."
    (if (null item)
        string
      (setq item (car item))
-     (unless (find-face 'kom-xface)
+     (unless (lyskom-find-face 'kom-xface)
        (make-face 'kom-xface))
      (let* ((h (concat "X-Face: " (aux-item->data item)))
             (g (intern h lyskom-xface-cache))
@@ -623,7 +644,7 @@ return non-nil if the item is to be included in the list."
 		     (substring (text->decoded-text-mass text text-stat) 0 (match-beginning 0)))
 		    (t ""))
 	      "\"")
-    (lyskom-format 'no-such-text-no text-no)))
+    (lyskom-format 'no-such-text-m text-no)))
 
 (defun lyskom-deferred-print-faq (text-stat defer-info)
   (if text-stat
@@ -642,28 +663,38 @@ return non-nil if the item is to be included in the list."
                              (lyskom-print-faq-format-subject nil nil (defer-info->data defer-info)))))
 
 (defun lyskom-status-print-faq-text (item &optional obj)
-  (let* ((text-no (string-to-int (aux-item->data item)))
-         (subject (if kom-deferred-printing
-                      (lyskom-create-defer-info 'get-text-stat
-                                                text-no
-                                                'lyskom-deferred-print-faq
-                                                nil nil nil
-                                                text-no)
-                    (blocking-do-multiple ((text (get-text text-no))
-                                           (text-stat (get-text-stat text-no)))
-                      (lyskom-print-faq-format-subject text text-stat text-no)))))
-    (lyskom-format-insert 'faq-in-text-aux 
-                          text-no
-                          subject)
-    (lyskom-insert (lyskom-aux-item-terminating-button item obj))
-    (lyskom-insert "\n")))
+  (if (lyskom-aux-item-validate (aux-item->data item) 'lyskom-string-to-int)
+      (let* ((text-no (string-to-int (aux-item->data item)))
+             (subject (if kom-deferred-printing
+                          (lyskom-create-defer-info 'get-text-stat
+                                                    text-no
+                                                    'lyskom-deferred-print-faq
+                                                    nil nil nil
+                                                    text-no)
+                        (blocking-do-multiple ((text (get-text text-no))
+                                               (text-stat (get-text-stat text-no)))
+                          (lyskom-print-faq-format-subject text text-stat text-no)))))
+        (lyskom-format-insert 'faq-in-text-aux 
+                              text-no
+                              subject)
+        (lyskom-insert (lyskom-aux-item-terminating-button item obj)))
+    (lyskom-format-insert 'bad-faq-in-text-aux
+                          (aux-item->data item)
+                          `(face ,kom-warning-face)
+                          (lyskom-aux-item-terminating-button item obj)))
+  (lyskom-insert "\n"))
 
 (defun lyskom-print-faq-for-conf (item &optional obj)
-  (let ((conf-no (string-to-int (aux-item->data item))))
-    (concat 
-     (cond ((zerop conf-no) (lyskom-get-string 'faq-for-server-aux))
-          (t (lyskom-format 'faq-for-conf-aux conf-no)))
-     (lyskom-aux-item-terminating-button item obj))))
+  (if (lyskom-aux-item-validate (aux-item->data item) 'lyskom-string-to-int)
+      (let ((conf-no (string-to-int (aux-item->data item))))
+        (concat 
+         (cond ((zerop conf-no) (lyskom-get-string 'faq-for-server-aux))
+               (t (lyskom-format 'faq-for-conf-aux conf-no)))
+         (lyskom-aux-item-terminating-button item obj)))
+    (lyskom-format 'bad-faq-for-conf-aux 
+                   (aux-item->data item)
+                   `(face ,kom-warning-face)
+                   (lyskom-aux-item-terminating-button item obj))))
 
 (defun lyskom-print-creating-software (item &optional obj)
   (when (or kom-show-creating-software
@@ -673,11 +704,15 @@ return non-nil if the item is to be included in the list."
      (lyskom-aux-item-terminating-button item obj))))
 
 (defun lyskom-print-send-comments-to (item &optional obj)
-  (when (string-match "^\\([0-9]+\\)" (aux-item->data item))
-    (let ((conf-no (string-to-int (match-string 1 (aux-item->data item)))))
-      (lyskom-format-insert 'status-send-comments-to
-                            conf-no 
-                            (lyskom-aux-item-terminating-button item obj)))))
+  (if (lyskom-aux-item-validate (aux-item->data item) 'lyskom-string-to-int)
+      (let ((conf-no (string-to-int (aux-item->data item))))
+        (lyskom-format-insert 'status-send-comments-to
+                              conf-no 
+                              (lyskom-aux-item-terminating-button item obj)))
+    (lyskom-format-insert 'bad-status-send-comments-to
+                          (aux-item->data item)
+                          `(face ,kom-warning-face)
+                          (lyskom-aux-item-terminating-button item obj))))
 
 
 
@@ -700,29 +735,49 @@ return non-nil if the item is to be included in the list."
                         (aux-item->data item)
                         (lyskom-aux-item-terminating-button item obj)))
 
+(defun lyskom-print-mx-refuse-import (item &optional obj)
+  (let* ((sym (intern (format "mx-refuse-import-%s" (aux-item->data item))))
+         (s (or (lyskom-try-get-string sym 'lyskom-message)
+                (format "'%s'" (aux-item->data item)))))
+    (lyskom-format-insert 'conf-mx-refuse-import s
+                          (lyskom-aux-item-terminating-button item obj))))
+
 (defun lyskom-print-recommended-conf (item &optional obj)
-  (let ((conf-no (string-to-int (if (string-match " " (aux-item->data item))
-                                    (substring (aux-item->data item) 0 (match-beginning 0))
-                                  (aux-item->data item)))))
-    (lyskom-format-insert 'recommended-conf-aux
-                          conf-no
+  (if (lyskom-aux-item-validate (aux-item->data item) 'lyskom-string-to-int)
+      (let ((conf-no (string-to-int (aux-item->data item))))
+        (lyskom-format-insert 'recommended-conf-aux
+                              conf-no
+                              (lyskom-aux-item-terminating-button item obj)))
+    (lyskom-format-insert 'bad-recommended-conf-aux
+                          (aux-item->data item)
+                          `(face ,kom-warning-face)
                           (lyskom-aux-item-terminating-button item obj))))
 
 (defun lyskom-print-elisp-client-read-faq (item &optional obj)
-  (when kom-extended-status-information
+  (when (lyskom-extended-status-information 'read-faq)
     (when (string-match "^\\([0-9]+\\) \\([0-9]+\\)" (aux-item->data item))
-      (let ((conf-no (string-to-int (match-string 1 (aux-item->data item))))
-            (text-no (string-to-int (match-string 2 (aux-item->data item)))))
-        (lyskom-format-insert 'status-read-faq-aux-item 
-                              conf-no 
-                              text-no
-                              (lyskom-aux-item-terminating-button item obj))))))
+      (condition-case nil
+          (let ((conf-no (lyskom-string-to-int (match-string 1 (aux-item->data item)) t))
+                (text-no (lyskom-string-to-int (match-string 2 (aux-item->data item)) t)))
+            (lyskom-format-insert 'status-read-faq-aux-item 
+                                  conf-no 
+                                  text-no
+                                  (lyskom-aux-item-terminating-button item obj)))
+        (lyskom-integer-conversion-error
+         (lyskom-format-insert 'bad-status-read-faq-aux-item
+                               (aux-item->data item)
+                               `(face ,kom-warning-face)
+                               (lyskom-aux-item-terminating-button item obj)))))))
 
 (defun lyskom-print-elisp-client-rejected-invitation (item &optional obj)
-  (when (string-match "^\\([0-9]+\\)" (aux-item->data item))
-    (let ((conf-no (string-to-int (match-string 1 (aux-item->data item)))))
-      (lyskom-format-insert 'status-rejected-recommendation-aux-item
-                            conf-no 
-                            (lyskom-aux-item-terminating-button item obj)))))
+  (if (lyskom-aux-item-validate (aux-item->data item) 'lyskom-string-to-int)
+      (let ((conf-no (string-to-int (aux-item->data item))))
+        (lyskom-format-insert 'status-rejected-recommendation-aux-item
+                              conf-no 
+                              (lyskom-aux-item-terminating-button item obj)))
+    (lyskom-format-insert 'bad-status-rejected-recommendation-aux-item
+                          (aux-item->data item)
+                          `(face ,kom-warning-face)
+                          (lyskom-aux-item-terminating-button item obj))))
 
 (provide 'lyskom-aux-items)
