@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;;; $Id: buffers.el,v 44.23 2002/02/24 20:23:25 joel Exp $
+;;;;; $Id: buffers.el,v 44.28 2004/11/15 17:27:17 _cvs_pont_lyskomelisp Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -35,7 +35,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: buffers.el,v 44.23 2002/02/24 20:23:25 joel Exp $\n"))
+	      "$Id: buffers.el,v 44.28 2004/11/15 17:27:17 _cvs_pont_lyskomelisp Exp $\n"))
 
 
 ;;;;
@@ -140,7 +140,7 @@ a child of some buffer, reparent it."
         (setq done (cons current done))
         (setq queue (append queue (lyskom-get-buffer-children current)))
         (funcall function current)))))
-        
+
 
 (defun lyskom-map-buffer-children (function buffer)
   "Apply FUNCTION to each child of BUFFER and make a list of the results."
@@ -192,7 +192,11 @@ the children object"
 (add-hook 'kill-buffer-query-functions
           'lyskom-buffer-hierarchy-query-kill-function)
 
+;;;; ======================================================================
 
+(defun lyskom-update-unread-mode-line ()
+  (setq lyskom-unread-mode-line 
+	(lyskom-make-lyskom-unread-mode-line)))
 
 ;;;; ======================================================================
 ;;;; ======================================================================
@@ -205,6 +209,7 @@ the children object"
         (lyskom-clean-buffer-list lyskom-sessions-with-unread-letters)
         lyskom-buffer-list
         (lyskom-clean-buffer-list lyskom-buffer-list))
+  (lyskom-update-unread-mode-line)
   (lyskom-set-default 'lyskom-need-prompt-update t))
 
 (defun lyskom-clean-buffer-list (buffers)
@@ -226,6 +231,7 @@ If BUFFER is not specified, assume the current buffer"
   (lyskom-remove-unread-buffer buffer)  
   (setq lyskom-buffer-list
         (delq buffer lyskom-buffer-list))
+  (lyskom-update-unread-mode-line)
   (lyskom-set-default 'lyskom-need-prompt-update t))
 
 (defun lyskom-remove-unread-buffer (buffer &optional letters-only)
@@ -244,6 +250,7 @@ If BUFFER is not specified, assume the current buffer"
    buffer)
   (setq lyskom-sessions-with-unread-letters
         (delq buffer lyskom-sessions-with-unread-letters))
+  (lyskom-update-unread-mode-line)
   (lyskom-set-default 'lyskom-need-prompt-update t))
   
 
@@ -265,6 +272,7 @@ If BUFFER is not specified, assume the current buffer"
      buffer)
     (setq lyskom-sessions-with-unread-letters
           (cons buffer lyskom-sessions-with-unread-letters)))
+  (lyskom-update-unread-mode-line) 
   (lyskom-set-default 'lyskom-need-prompt-update t))
 
 (defvar lyskom-associated-buffer-list nil
@@ -351,18 +359,20 @@ categories")
 (add-hook 'kill-emacs-query-functions 'lyskom-quit-query)
 
 (defun lyskom-recode-buffer-name (name)
-  (cond ((and (multibyte-string-p name) (not enable-multibyte-characters))
-	 (encode-coding-string name 
-			       (or (and lyskom-language
-					(lyskom-language-coding
-					 lyskom-language))
-				   'raw-text)))
-	((and (not (multibyte-string-p name)) enable-multibyte-characters)
-	 (decode-coding-string name 
+  (cond ((and (lyskom-multibyte-string-p name)
+              (not enable-multibyte-characters))
+	 (lyskom-encode-coding-string name 
 			       (or (and lyskom-language
 					(lyskom-language-coding
 					 lyskom-language))
 				   'raw-text)))
+	((and (not (lyskom-multibyte-string-p name)) 
+              enable-multibyte-characters)
+	 (lyskom-decode-coding-string name 
+                                      (or (and lyskom-language
+                                               (lyskom-language-coding
+                                                lyskom-language))
+                                          'raw-text)))
 	(t name)))
 
 (defun lyskom-generate-new-buffer (name)
@@ -426,13 +436,16 @@ to all children"
 
 
 
-(defun lyskom-display-buffer (buffer)
+(defun lyskom-display-buffer (buffer &optional keep-frame-iconified)
   "Display the buffer BUFFER and select the window displaying it.
 
 If BUFFER is already visible in some window in any frame, iconified or
 otherwise, make that window visible. Otherwise display buffer as per 
 which category it is in. Selects the window.
 
+If optional argument KEEP-FRAME-ICONIFIED is non-nil, then don't
+deiconify any frames.
+
 Returns the window displaying BUFFER."
 
 
@@ -461,7 +474,8 @@ Returns the window displaying BUFFER."
     (cond
      (visible-frame (select-window visible-frame)
                     visible-frame)
-     (iconified-frame (make-frame-visible (window-frame iconified-frame))
+     (iconified-frame (unless keep-frame-iconified 
+                        (make-frame-visible (window-frame iconified-frame)))
                       (select-window iconified-frame)
                       iconified-frame)
 
@@ -570,7 +584,7 @@ Returns the window displaying BUFFER."
             (make-local-variable 'lyskom-saved-window-configuration)
             (setq lyskom-saved-window-configuration
                   saved-window-configuration)
-            (make-local-hook 'kill-buffer-hook)
+            (lyskom-make-local-hook 'kill-buffer-hook)
             (add-hook 'kill-buffer-hook
                       'lyskom-undisplay-buffer-hook nil t)
             (put 'kill-buffer-hook 'permanent-local t)
