@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;;; $Id: clienttypes.el,v 44.17 2002/05/25 18:22:39 byers Exp $
+;;;;; $Id: clienttypes.el,v 44.27 2004/07/21 11:14:38 byers Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -37,7 +37,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: clienttypes.el,v 44.17 2002/05/25 18:22:39 byers Exp $\n"))
+	      "$Id: clienttypes.el,v 44.27 2004/07/21 11:14:38 byers Exp $\n"))
 
 
 ;;; ================================================================
@@ -55,6 +55,7 @@
 ;;;		    commands: kom-find-root-review, kom-review-tree
 ;;;   REVIEW-MARK - List of texts created by the review-mark command
 ;;;   REVIEW-FAQ  - List of texts created by kom-review-faq or similar
+;;;   REVIEW-FAQ-TREE  - Like REVIEW-TREE but for FAQs
 ;;;   COMM-IN     - Type containing the list of comments to a text
 ;;;   FOOTN-IN    - Type containing the list of footnotes to a text
 ;;;   CONF        - Basic type of unread in a conf.
@@ -72,88 +73,17 @@
 
 ;;; read-info
 
-(defsubst lyskom-create-read-info (type
-				conf-stat
-				priority
-				text-list
-				&optional comm-to forward misc)
-  "Create a read-info from all parameters."
-  ;; The last nil is for the unfetched-texts pair (first . last)
-  ;; This field is only applicable in read-infos of type CONF where
-  ;; it shows which part of the map for this conference that has not
-  ;; yet been fetched.
-  (cons
-   'READ-INFO
-   (vector type conf-stat priority text-list comm-to forward nil misc)))
-
-(defsubst read-info->type (read-info)
-  "Get type from read-info."
-  (elt (cdr read-info) 0))
-
-(defsubst read-info->conf-stat (read-info)
-  "Get conf-stat from read-info."
-  (elt (cdr read-info) 1))
-
-(defsubst read-info->priority (read-info)
-  "Get priority from read-info."
-  (elt (cdr read-info) 2))
-
-(defsubst read-info->text-list (read-info)
-  "Get text-list from read-info."
-  (elt (cdr read-info) 3))
-
-(defsubst read-info->comm-to (read-info)
-  "Get comm-to from read-info."
-  (elt (cdr read-info) 4))
-
-(defsubst read-info->forward (read-info)
-  "Get forward from read-info."
-  (elt (cdr read-info) 5))
-
-(defsubst read-info->unfetched-texts (read-info)
-  "Get forward from read-info."
-  (elt (cdr read-info) 6))
-
-(defsubst read-info->misc (read-info)
-  "Get forward from read-info."
-  (elt (cdr read-info) 7))
-
-(defsubst set-read-info->type (read-info newval)
-  "Set type in read-info to NEWVAL."
-  (aset (cdr read-info) 0 newval))
-
-(defsubst set-read-info->conf-stat (read-info newval)
-  "Set conf-stat in read-info to NEWVAL."
-  (aset (cdr read-info) 1 newval))
-
-(defsubst set-read-info->priority (read-info newval)
-  "Set priority in read-info to NEWVAL."
-  (aset (cdr read-info) 2 newval))
-
-(defsubst set-read-info->text-list (read-info newval)
-  "Set text-list in read-info to NEWVAL."
-  (aset (cdr read-info) 3 newval))
-
-(defsubst set-read-info->comm-to (read-info newval)
-  "Set comm-to in read-info to NEWVAL."
-  (aset (cdr read-info) 4 newval))
-
-(defsubst set-read-info->forward (read-info newval)
-  "Set forward in read-info to NEWVAL."
-  (aset (cdr read-info) 5 newval))
-
-(defsubst set-read-info->unfetched-texts (read-info newval)
-  "Set forward in read-info to NEWVAL."
-  (aset (cdr read-info) 6 newval))
-
-(defsubst set-read-info->misc (read-info newval)
-  "Set forward in read-info to NEWVAL."
-  (aset (cdr read-info) 7 newval))
-
-
-(defsubst lyskom-read-info-p (object)
-  "Return t if OBJECT is a read-info."
-  (eq (car-safe object) 'READ-INFO))
+(def-komtype read-info 
+  ((type                :read-only t)
+   (conf-stat           :read-only t)
+   priority 
+   (text-list           :read-only t)
+   &optional 
+   (comm-to             :read-only t)
+   forward 
+   (unfetched-texts     :read-only t :automatic nil)
+   (misc                :read-only t))
+  :nil-safe)
 
 
 (defsubst read-info-append-text-list (read-info texts)
@@ -162,10 +92,13 @@
 (defsubst read-info-enter-text-last (read-info text-no)
   (read-info-append-text-list read-info (list text-no)))
 
-				   
+
 ;;; ================================================================
 ;;;                          read-list
-
+;;;
+;;; We don't use def-komtype here because the code below relies on 
+;;; read-lists being a cons of READ-LIST and the data.
+;;;
 
 ;;; Constructor:
 
@@ -204,25 +137,37 @@ The range of valid values for N is [0, num-entries - 1]."
 
 (defsubst read-list-length (read-list)
   "Return the number of entries in READ-LIST."
-  (1- (length read-list)))
+  (length (read-list->all-entries read-list)))
 
 
 ;;; Modifiers:
 
 (defsubst set-read-list-empty (read-list)
   "Empty READ-LIST destructively."
-  (setcdr read-list nil))
-
+  (let ((tmp (cdr read-list)))
+    (setcdr read-list nil)
+    (lyskom-traverse read-info tmp
+      (when (eq 'CONF (read-info->type read-info))
+        (lp--maybe-update-unreads (conf-stat->conf-no
+                                   (read-info->conf-stat read-info)))))))
 
 (defsubst set-read-list-del-first (read-list)
   "Delete the first entry of READ-LIST if there is one."
-  (if (cdr read-list)
-      (setcdr read-list (cdr (cdr read-list)))))
+  (let ((el (car (cdr read-list))))
+    (if (cdr read-list)
+        (setcdr read-list (cdr (cdr read-list))))
+    (when (and el (eq (read-info->type el) 'CONF))
+      (lp--maybe-update-unreads (conf-stat->conf-no
+                                 (read-info->conf-stat el))))))
 
 
 (defsubst read-list-enter-first (read-info read-list)
   "Enter READ-INFO first into READ-LIST."
-  (setcdr read-list (cons read-info (cdr read-list))))
+  (setcdr read-list (cons read-info (cdr read-list)))
+  (when (eq 'CONF (read-info->type read-info))
+    (lp--maybe-update-unreads (conf-stat->conf-no
+                               (read-info->conf-stat read-info))))
+  )
 
 
 (defun read-list-enter-text (text-no recipient rlist)
@@ -239,6 +184,7 @@ Returns t if there was a conference to insert this text into."
 	       (conf-stat->conf-no (read-info->conf-stat read-info))))
        (read-info-enter-text-last read-info text-no)
        (setq inserted t))))
+    (when inserted (lp--maybe-update-unreads (conf-stat->conf-no recipient)))
     inserted))
 
 
@@ -252,14 +198,17 @@ TEXT-NO may be nil, in which case only empty read-infos on RLIST are removed."
   (let* ((prev rlist)			;"Previous" cons-celll
 	 (curr (cdr rlist)))		;Current cons-cell
     (while curr
-      (if text-no
-	  (cond
-	   ((let ((type (read-info->type (car curr)))) 
+      (when text-no
+        (cond
+         ((let ((type (read-info->type (car curr)))) 
 					; Don't change REVIEW et c.
-	      (memq type lyskom-review-types-list)))
-	   (t				; Do change all other entries.
-	    (let ((tl  (read-info->text-list (car curr))))
-	      (text-list->delq tl text-no)))))
+            (memq type lyskom-review-types-list)))
+         (t				; Do change all other entries.
+          (let ((tl  (read-info->text-list (car curr))))
+            (when (text-list->memq tl text-no)
+              (text-list->delq tl text-no)
+              (lp--maybe-update-unreads
+               (conf-stat->conf-no (read-info->conf-stat (car curr)))))))))
       
       ;; Delete this element from RLIST if the text-list became or was empty.
       
@@ -284,17 +233,17 @@ will be inserted before it."
 	(type (read-info->type read-info)))
     (while continue
       (cond
-       ;; This case was added by davidk 960925. It is not used from
-       ;; everywhere, but at least lyskom-enter-map-in-to-do-list
-       ;; should become more efficient.
        ((and (eq type 'CONF)
 	     (eq (read-info->type (car (cdr rlist))) 'CONF)
-	     (eq conf-stat (read-info->conf-stat (car (cdr rlist)))))
+	     (eq (conf-stat->conf-no conf-stat)
+                 (conf-stat->conf-no (read-info->conf-stat
+                                      (car (cdr rlist))))))
 	(read-info-append-text-list
 	 (car (cdr rlist))
 	 (text-list->texts (read-info->text-list read-info)))
+        (lp--maybe-update-unreads (conf-stat->conf-no conf-stat))
 	(setq continue nil)) 
-       
+
        ((null (cdr rlist)) 
 	(setcdr rlist (list read-info))
 	(setq continue nil))
@@ -302,7 +251,11 @@ will be inserted before it."
 	(setcdr rlist (cons read-info (cdr rlist)))
 	(setq continue nil))
        (t 
-	(setq rlist (cdr rlist)))))))
+	(setq rlist (cdr rlist)))))
+
+    (when (eq type 'CONF)
+      (lp--maybe-update-unreads (conf-stat->conf-no conf-stat)) )))
+
 
 (defun read-list-delete-read-info (conf-no rlist)
   "Destructively removes all the entries for the conf CONF-NO in RLIST.
@@ -312,9 +265,9 @@ RLIST is a list of read-info."
 				      (car (cdr rlist))))
 	    conf-no)
 	(setcdr rlist (cdr (cdr rlist)))
-      (setq rlist (cdr rlist)))))
+      (setq rlist (cdr rlist))))
+  (lp--maybe-update-unreads conf-no))
  
-	  
 (defun read-list-rotate (read-list)
   "Put the first element of READ-LIST last in the same list. The second
 element will be the new first element."
@@ -380,15 +333,32 @@ element will be the new first element."
   (car-safe (cdr queue)))
 
 
+;;UNUSED: lyskom-queue->last
 (defsubst lyskom-queue->last (queue)
   "Return the lastelement of QUEUE or nil if it is empty."
   (car-safe (cdr (cdr queue))))
 
 
+(defun lyskom-queue-remove-matching (queue pred)
+  "Remove all elements from QUEUE that satisfy PRED."
+  (let ((prev nil)
+        (ptr (car (cdr queue))))
+    (while ptr
+      (if (funcall pred (car ptr))
+        (progn (if prev
+                   (setcdr prev (cdr ptr))
+                 (setcar (cdr queue) (cdr ptr)))
+               (unless (cdr ptr)
+                 (setcdr (cdr queue) prev)))
+        (setq prev ptr))
+      (setq ptr (cdr ptr)))))
+
+
 (defsubst lyskom-queue-make-empty (queue)
   "Make the queue QUEUE empty."
   (setcdr queue (cons nil nil)))
 
+;;UNUSED: lyskom-queue-set-data
 (defsubst lyskom-queue-set-data (queue data)
   (lyskom-queue-make-empty queue)
   (setcdr queue (cons data (last data))))
@@ -445,11 +415,12 @@ is empty, return nil"
   (car-safe (cdr stack)))
 
 
+;;UNUSED: lyskom-stack->length
 (defun lyskom-stack->length (stack)
   "Return the number of elements on STACK."
   (length (cdr stack)))
 
-
+;;UNUSED: lyskom-stack->nth
 (defun lyskom-stack->nth (stack n)
   "Return element no (second arg) N of the stack (first arg) STACK.
 N counts from zero. If the length of STACK is less than N, nil is returned."
@@ -467,155 +438,21 @@ The element last pushed is first in the list."
   (not (cdr stack)))
 
 
-;;; ================================================================
-;;;                          who-buffer-info
-
-;;; Author: Inge Wallin
-
-
-;;; Constructor:
-
-(defun lyskom-create-who-buffer-info (info
-				      start-marker
-				      end-marker)
-  "Create a who-buffer-info from all parameters."
-  (cons
-   'WHO-BUFFER-INFO
-   (vector info start-marker end-marker )))
-
-
-;;; Selectors:
-
-(defun who-buffer-info->info (who-buffer-info)
-  "Get info from who-buffer-info."
-  (elt (cdr who-buffer-info) 0))
-
-(defun who-buffer-info->start-marker (who-buffer-info)
-  "Get start-marker from who-buffer-info."
-  (elt (cdr who-buffer-info) 1))
-
-(defun who-buffer-info->end-marker (who-buffer-info)
-  "Get end-marker from who-buffer-info."
-  (elt (cdr who-buffer-info) 2))
-
-
-;;; Modifiers:
-
-(defun set-who-buffer-info->info (who-buffer-info newval)
-  "Set info in who-buffer-info to NEWVAL."
-  (aset (cdr who-buffer-info) 0 newval))
-
-(defun set-who-buffer-info->start-marker (who-buffer-info newval)
-  "Set start-marker in who-buffer-info to NEWVAL."
-  (aset (cdr who-buffer-info) 1 newval))
-
-(defun set-who-buffer-info->end-marker (who-buffer-info newval)
-  "Set end-marker in who-buffer-info to NEWVAL."
-  (aset (cdr who-buffer-info) 2 newval))
-
-
-
-;;; Predicate:
-
-(defun lyskom-who-buffer-info-p (object)
-  "Return t if OBJECT is a who-buffer-info."
-  (eq (car-safe object) 'WHO-BUFFER-INFO))
-
-
-;;; ================================================================
-;;;			format-props
-
-(defun make-format-props (arg propl)
-  (cons 'format-props (vector arg propl)))
-
-(defsubst format-props-p (arg)
-  (eq 'format-props (car-safe arg)))
-
-(defsubst format-props->arg (arg)
-  (elt (cdr arg) 0))
-
-(defsubst format-props->propl (arg)
-  (elt (cdr arg) 1))
-
-;;;
-;;;	Help functions
-;;;
-
-
 
 ;;; ================================================================
 ;;;			format-state
 
 (def-komtype format-state
-  format-string
-  start
-  argl
-  length
-  result
-  delayed-propl
-  delayed-overlays
-  delayed-content
-  depth)
-;; 
-;; 
-;; 
-;; (defun make-format-state (format-string
-;; 			  start
-;; 			  argl
-;; 			  result)
-;;   (cons 'format-state
-;; 	(vector format-string start argl (length argl) result nil nil nil nil)))
-;; 
-;; (defsubst format-state-p (arg)
-;;   (eq 'format-state (car-safe arg)))
-;; 
-;; (defsubst format-state->format-string (arg)
-;;   (elt (cdr arg) 0))
-;; 
-;; (defsubst set-format-state->format-string (arg str)
-;;   (aset (cdr arg) 0 str))
-;; 
-;; (defsubst format-state->start (arg)
-;;   (elt (cdr arg) 1))
-;; 
-;; (defsubst set-format-state->start (arg pos)
-;;   (aset (cdr arg) 1 pos))
-;; 
-;; (defsubst format-state->args (arg)
-;;   (elt (cdr arg) 2))
-;; 
-;; (defsubst set-format-state->args (arg argl)
-;;   (aset (cdr arg) 2 argl)
-;;   (aset (cdr arg) 3 (length argl)))
-;; 
-;; (defsubst format-state->args-length (arg)
-;;   (elt (cdr arg) 3))
-;; 
-;; (defsubst format-state->result (arg)
-;;   (elt (cdr arg) 4))
-;; 
-;; (defsubst set-format-state->result (arg output-list)
-;;   (aset (cdr arg) 4 output-list))
-;; 
-;; (defsubst format-state->delayed-propl (arg)
-;;   (elt (cdr arg) 5))
-;; 
-;; (defsubst set-format-state->delayed-propl (arg propl)
-;;   (aset (cdr arg) 5 propl))
-;; 
-;; (defsubst format-state->delayed-overlays (arg)
-;;   (elt (cdr arg) 7))
-;; 
-;; (defsubst set-format-state->delayed-overlays (arg overlays)
-;;   (aset (cdr arg) 7 overlays))
-;; 
-;; (defsubst format-state->delayed-content (arg)
-;;   (elt (cdr arg) 6))
-;; 
-;; (defsubst set-format-state->delayed-content (arg string)
-;;   (aset (cdr arg) 6 string))
-;; 
-;; (defsubst
+  (format-string
+   start
+   argl
+   length
+   result
+   delayed-propl
+   delayed-overlays
+   delayed-content
+   depth)
+  :nil-safe)
 
 ;;; ================================================================
 
@@ -670,14 +507,37 @@ The element last pushed is first in the list."
   "Set the calue of a collector"
   (setcdr collector value))
 
-(defun set-value-of-collector (value collector)
-  "For use with lyskom handlers. In other cases, use set-collector->value"
-  (set-collector->value collector value))
-
 (defun collector-push (value collector)
   "Push VALUE onto the front of COLLECTOR's value"
   (setcdr collector (cons value (cdr collector))))
 
+
+;;; ================================================================
+;;; Specification for text links (cached)
+;;;
+;;; We don't use def-komtype since the accessors should be able to
+;;; work on user settings too.
+
+(defun lyskom-create-lyskom-text-link (pattern replacement 
+                                               highlight ignore-case)
+  (cons 'LYSKOM-TEXT-LINK (vector pattern replacement highlight ignore-case)))
+
+(defsubst lyskom-text-link->pattern (link)
+  (if (symbolp (car link)) (elt (cdr link) 0) (elt link 0)))
+
+(defsubst lyskom-text-link->replacement (link)
+  (if (symbolp (car link)) (elt (cdr link) 1) (elt link 1)))
+
+(defsubst lyskom-text-link->highlight (link)
+  (if (symbolp (car link)) (elt (cdr link) 2) (elt link 2)))
+
+(defsubst lyskom-text-link->ignore-case (link)
+  (if (symbolp (car link)) (elt (cdr link) 3) (elt link 3)))
+
+
+
+
+
 (eval-and-compile (provide 'lyskom-clienttypes))
 
 ;;; clienttypes.el ends here
