@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;;; $Id: command.el,v 44.45 2002/06/24 17:12:19 byers Exp $
+;;;;; $Id: command.el,v 44.55 2005/01/11 07:35:52 _cvs_pont_lyskomelisp Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -34,13 +34,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: command.el,v 44.45 2002/06/24 17:12:19 byers Exp $\n"))
-
-;;; (eval-when-compile
-;;;   (require 'lyskom-vars "vars")
-;;;   (require 'lyskom-services "services")
-;;;   (require 'lyskom-language "language")
-;;;   (require 'lyskom-clienttypes "clienttypes"))
+	      "$Id: command.el,v 44.55 2005/01/11 07:35:52 _cvs_pont_lyskomelisp Exp $\n"))
 
 
 ;;; ======================================================================
@@ -71,32 +65,75 @@
 
 
 
-(defmacro def-kom-command (cmd args doc interactive-decl &rest forms)
-  (if (not (stringp doc))
-      (progn (message "!! No docstring for command %S" cmd)
-             (setq forms (cons interactive-decl forms))
-             (setq interactive-decl doc)
-             (setq doc "")))
-  (if (not (eq (car interactive-decl) 'interactive))
-      (progn (message "!! Missing interactive declaration for %S; assuming \(interactive\)" cmd)
-             (setq forms (cons interactive-decl forms))
-             (setq interactive-decl '(interactive))))
-  (let ((bufsym (intern (format "%S-start-buffer" cmd))))
-    `(defun ,cmd ,args
-       ,doc
-       ,(lyskom-fix-interactive-decl interactive-decl cmd)
-       (lyskom-start-of-command ',cmd)
-       (let ((,bufsym (current-buffer)))
-         (unwind-protect
-             (condition-case nil
-                 (progn ,@forms)
-               (quit (ding)
-                     (lyskom-insert-before-prompt
-                      (lyskom-get-string 'interrupted))))
-           (lyskom-save-excursion
-            (when (buffer-live-p ,bufsym)
-              (set-buffer ,bufsym))
-             (lyskom-end-of-command)))))))
+(defmacro def-kom-command (cmd args doc &rest forms)
+  "Define a LysKOM command CMD with args ARGS and docstring DOC.
+
+This is like a defun for LysKOM commands. After the docstring you may
+have a series of keywords that alter the behavior of the command,
+followed by an interactive declaration (required) and the body of the
+command.
+
+The following keywords are recognized:
+
+:dead-ok        See the DEAD-OK paramter to lyskom-start-of-command
+:may-interrupt  See the MAY-INTERRUPT parameter to lyskom-start-of-command
+:prompt-format  See the PROMPT-FORMAT parameter to lyskom-start-of-command
+
+Note that :prompt-format requires an argument."
+  (let (interactive-decl may-interrupt dead-ok prompt-format)
+    (while (memq (car forms) '(:prompt-format :dead-ok :may-interrupt))
+      (cond ((eq (car forms) :prompt-format)
+             (setq prompt-format (car (cdr forms)))
+             (setq forms (cdr forms)))
+            ((eq (car forms) :dead-ok)
+             (setq dead-ok t))
+            ((eq (car forms) :may-interrupt)
+             (setq may-interrupt t)))
+      (setq forms (cdr forms)))
+
+    (when (and (listp (car forms))
+               (eq 'interactive (car (car forms))))
+      (setq interactive-decl (car forms) forms (cdr forms)))
+
+    (if (not (stringp doc))
+        (progn (message "!! No docstring for command %S" cmd)
+               (setq forms (cons interactive-decl forms))
+               (setq interactive-decl doc)
+               (setq doc "")))
+    (if (not (eq (car interactive-decl) 'interactive))
+        (progn (message "!! Missing interactive declaration for %S; assuming \(interactive\)" cmd)
+               (setq forms (cons interactive-decl forms))
+               (setq interactive-decl '(interactive))))
+    (let ((bufsym (make-symbol (format "%S-start-buffer" cmd)))
+          (winsym (make-symbol (format "%S-start-window" cmd))))
+      `(defun ,cmd ,args
+         ,doc
+         ,(lyskom-fix-interactive-decl interactive-decl cmd)
+         (let ((lyskom-command-point (point)))
+           (lyskom-start-of-command ',cmd ,may-interrupt ,dead-ok ,prompt-format)
+           (let ((,bufsym (current-buffer))
+                 (,winsym (selected-window)))
+             (unwind-protect
+                 (condition-case nil
+                     (progn ,@forms)
+                   (quit (ding)
+                         (lyskom-insert-before-prompt
+                          (lyskom-get-string 'interrupted))))
+               (lyskom-save-excursion
+                 (cond ((and (buffer-live-p ,bufsym)
+                             (window-live-p ,winsym)
+                             (eq (window-buffer ,winsym) ,bufsym))
+                        (save-selected-window
+                          (set-buffer ,bufsym)
+                          (select-window ,winsym)
+                          (lyskom-end-of-command)))
+
+                       ((buffer-live-p ,bufsym)
+                        (set-buffer ,bufsym)
+                        (lyskom-end-of-command))
+
+                       (t (lyskom-end-of-command)))))))))))
+
 
 ;;
 ;; def-kom-emacs-command works like def-kom-command, but the template 
@@ -127,40 +164,66 @@
 ;;       (and kom-cmd-running-as-kom-command (lyskom-end-of-command)))))
 ;; 
 
-(defmacro def-kom-emacs-command (cmd args doc interactive-decl &rest forms)
-  (if (not (stringp doc))
-      (progn (message "!! No docstring for command %S" cmd)
-             (setq forms (cons interactive-decl forms))
-             (setq interactive-decl doc)
-             (setq doc "")))
-  (if (not (eq (car interactive-decl) 'interactive))
-      (progn (message "!! Missing interactive declaration for %S; assuming \(interactive\)" cmd)
-             (setq forms (cons interactive-decl forms))
-             (setq interactive-decl '(interactive))))
+(defmacro def-kom-emacs-command (cmd args doc &rest forms)
+  (let (interactive-decl may-interrupt dead-ok prompt-format)
+    (while (memq (car forms) '(:prompt-format :dead-ok :may-interrupt))
+      (cond ((eq (car forms) :prompt-format)
+             (setq prompt-format (car (cdr forms)))
+             (setq forms (cdr forms)))
+            ((eq (car forms) :dead-ok)
+             (setq dead-ok t))
+            ((eq (car forms) :may-interrupt)
+             (setq may-interrupt t)))
+      (setq forms (cdr forms)))
 
-  (let ((rsym (intern (format "%S-running-as-kom-command" cmd)))
-        (bufsym (intern (format "%S-start-buffer" cmd))))
-    `(defun ,cmd ,args
-       ,doc
-       ,(lyskom-fix-interactive-decl interactive-decl cmd)
-       (let ((,rsym nil))
-         (condition-case nil
-             (progn (lyskom-start-of-command ',cmd)
-                    (setq ,rsym t))
-           (error nil))
-         (let ((,bufsym (current-buffer)))
-           (unwind-protect
-               (condition-case nil
-                   (progn ,@forms)
-                 (quit (ding)
-                       (lyskom-insert-before-prompt
-                        (lyskom-get-string 'interrupted))))
-             (and ,rsym
-                  (lyskom-save-excursion
-                   (when (buffer-live-p ,bufsym)
-                     (set-buffer ,bufsym))
-                   (lyskom-end-of-command)))))))))
+    (when (and (listp (car forms))
+               (eq 'interactive (car (car forms))))
+      (setq interactive-decl (car forms) forms (cdr forms)))
 
+    (if (not (stringp doc))
+        (progn (message "!! No docstring for command %S" cmd)
+               (setq forms (cons interactive-decl forms))
+               (setq interactive-decl doc)
+               (setq doc "")))
+    (if (not (eq (car interactive-decl) 'interactive))
+        (progn (message "!! Missing interactive declaration for %S; assuming \(interactive\)" cmd)
+               (setq forms (cons interactive-decl forms))
+               (setq interactive-decl '(interactive))))
+
+    (let ((rsym (intern (format "%S-running-as-kom-command" cmd)))
+          (bufsym (make-symbol (format "%S-start-buffer" cmd)))
+          (winsym (make-symbol (format "%S-start-window" cmd))))
+      `(defun ,cmd ,args
+         ,doc
+         ,(lyskom-fix-interactive-decl interactive-decl cmd)
+         (let ((,rsym nil))
+           (condition-case nil
+               (progn (lyskom-start-of-command ',cmd ,may-interrupt ,dead-ok ,prompt-format)
+                      (setq ,rsym t))
+             (error nil))
+           (let ((,bufsym (current-buffer))
+                 (,winsym (selected-window)))
+             (unwind-protect
+                 (condition-case nil
+                     (progn ,@forms)
+                   (quit (ding)
+                         (lyskom-insert-before-prompt
+                          (lyskom-get-string 'interrupted))))
+               (and ,rsym
+                    (lyskom-save-excursion
+                      (cond ((and (buffer-live-p ,bufsym)
+                                  (window-live-p ,winsym)
+                                  (eq (window-buffer ,winsym) ,bufsym))
+                             (save-selected-window
+                               (set-buffer ,bufsym)
+                               (select-window ,winsym)
+                               (lyskom-end-of-command)))
+
+                            ((buffer-live-p ,bufsym)
+                             (set-buffer ,bufsym)
+                             (lyskom-end-of-command))
+
+                            (t (lyskom-end-of-command))))))))))))
 
 
 (put 'def-kom-command 'edebug-form-spec
@@ -181,10 +244,10 @@
 ;;;;                User-level commands and functions.
 
 
-(defsubst lyskom-command-name (command)
+(defsubst lyskom-command-name (command &optional language)
   "Get the command name for the command COMMAND"
   (condition-case nil
-      (lyskom-get-string command 'lyskom-command)
+      (lyskom-get-string command 'lyskom-command language)
     (error nil)))
 
 (defun lyskom-ok-command (alternative administrator)
@@ -212,7 +275,7 @@
 (defvar lyskom-command-minibuffer-local-must-match-map
   (let ((map (copy-keymap minibuffer-local-must-match-map)))
     (lyskom-xemacs-or-gnu 
-     (progn (set-keymap-parent map lyskom-minibuffer-local-completion-map)
+     (progn (lyskom-set-keymap-parent map lyskom-minibuffer-local-completion-map)
             (define-key map " " 'lyskom-command-complete-word))
      (define-key map " " 'lyskom-command-complete-word))
     map)
@@ -250,19 +313,20 @@
            lyskom-command-minibuffer-local-completion-map)
           (minibuffer-local-must-match-map 
            lyskom-command-minibuffer-local-must-match-map))
-      (lyskom-with-lyskom-minibuffer
-       (setq name (lyskom-completing-read prompt
-                                          'lyskom-complete-command
-                                        ;                                        (lyskom-maybe-frob-completion-table
-                                        ;                                         alternatives)
-                                          ;; lyskom-is-administrator is buffer-local and
-                                          ;; must be evalled before the call to 
-                                          ;; completing-read
-                                          ;; Yes, this is not beautiful
-                                          (list 'lambda '(alternative)	     
-                                                (list 'lyskom-ok-command 'alternative
-                                                      lyskom-is-administrator))
-                                          t nil 'lyskom-command-history))))
+      (while (or (null name)
+                 (string= "" name))
+        (setq name (lyskom-completing-read prompt
+                                           'lyskom-complete-command
+                                           ;; lyskom-is-administrator is buffer-local and
+                                           ;; must be evalled before the call to 
+                                           ;; completing-read
+                                           ;; Yes, this is not beautiful
+                                           (list 'lambda '(alternative) ;
+                                                 (list 'lyskom-ok-command 'alternative
+                                                       lyskom-is-administrator))
+                                           t
+                                           nil
+                                           'lyskom-command-history))))
     (cdr (lyskom-string-assoc name alternatives))))
 
 
@@ -396,7 +460,7 @@ and back of the string."
     (let ((completion (try-completion string
                                       minibuffer-completion-table
                                       minibuffer-completion-predicate)))
-      (cond ((null completion) (minibuffer-message " [No match]") nil)
+      (cond ((null completion) (lyskom-minibuffer-message " [No match]") nil)
             ((eq completion t) nil)
             (t (let* ((tmp string))
                  (when (and (string-equal (lyskom-unicase completion)
@@ -492,7 +556,7 @@ and back of the string."
 ;;;  (minibuffer-complete-word))
 
 
-(defun lyskom-start-of-command (function &optional may-interrupt dead-ok)
+(defun lyskom-start-of-command (function &optional may-interrupt dead-ok prompt-format)
   "This function is run at the beginning of every LysKOM command.
 It moves the cursor one line down, and +++ later it will tell the server
 that the previous text has been read.
@@ -507,6 +571,10 @@ don't signal an error if this call is interrupting another command.
 If optional DEAD-OK is non-nil, don't signal an error if the session 
 has been closed.
 
+If optional PROMPT-FORMAT is non-nil, this will be used to format the prompt.
+It should be a format string suitable for lyskom-format. Argument one is the
+command name (a symbol or string).
+
 Special: if lyskom-is-waiting then we are allowed to break if we set 
 lyskom-is-waiting nil.
 
@@ -519,10 +587,15 @@ chosen according to this"
              (and (null lyskom-proc) (null lyskom-buffer)))
          (lyskom-error "%s" (lyskom-get-string 'not-lyskom-buffer)))
 
-        ((and (not dead-ok)
+        ((and (not (or dead-ok
+		       (memq function kom-relogin-inhibit-commands)))
               (or (not lyskom-proc)
                   (memq (process-status lyskom-proc) '(closed signal exited nil))))
-      (lyskom-error "%s" (lyskom-get-string 'dead-session))))
+	 (if (or (eq t kom-relogin-behaviour)
+		 (and (eq 'ask kom-relogin-behaviour) 
+		      (lyskom-j-or-n-p (lyskom-get-string 'resurrect-session))))
+	     (lyskom lyskom-server-name lyskom-pers-no)
+      (lyskom-error "%s" (lyskom-get-string 'dead-session)))))
 
   (if (and lyskom-is-waiting
            (listp lyskom-is-waiting))
@@ -538,9 +611,10 @@ chosen according to this"
       (cond
        (lyskom-first-time-around)
        ((stringp function) (lyskom-insert function))
-       ((and function (symbolp function))
-        (let ((name (lyskom-command-name function)))
-          (if name (lyskom-insert name)))))
+       ((lyskom-command-name function)
+        (if prompt-format 
+            (lyskom-format-insert prompt-format function)
+          (lyskom-insert (lyskom-command-name function)))))
     (save-excursion
       (if lyskom-current-prompt
           (let ((inhibit-read-only t))
@@ -583,38 +657,38 @@ chosen according to this"
 (defun lyskom-end-of-command ()
   "Print prompt, maybe scroll, prefetch info."
   (lyskom-save-excursion
-   (message "")
-   (lyskom-clean-all-buffer-lists)
-   (while (and lyskom-to-be-printed-before-prompt
-               (lyskom-queue->first lyskom-to-be-printed-before-prompt))
-     (if (not (bolp)) (lyskom-insert "\n"))
-     (lyskom-insert (car (lyskom-queue->first 
-                          lyskom-to-be-printed-before-prompt)))
-     (lyskom-queue-delete-first lyskom-to-be-printed-before-prompt))
-   (setq lyskom-executing-command nil)
-   (setq lyskom-current-command nil)
-   (setq lyskom-current-prompt nil)	; Already set in s-o-c really
-   (lyskom-scroll)
-   (setq mode-line-process (lyskom-get-string 'mode-line-waiting))
-   (if (pos-visible-in-window-p (point-max) (selected-window))
-       (lyskom-set-last-viewed))
-   (lyskom-prefetch-and-print-prompt)
-   (run-hooks 'lyskom-after-command-hook)
-   (when (and (lyskom-have-feature idle-time)
-              (not lyskom-is-anonymous))
-     (save-excursion (set-buffer lyskom-buffer)
-                     (initiate-user-active 'background nil)))
-   (if kom-inhibit-typeahead
-       (discard-input))
-   ;; lyskom-pending-commands should probably be a queue or a stack.
-   (when lyskom-pending-commands
+    (message "")
+    (lyskom-clean-all-buffer-lists)
+    (while (and lyskom-to-be-printed-before-prompt
+                (lyskom-queue->first lyskom-to-be-printed-before-prompt))
+      (if (not (bolp)) (lyskom-insert "\n"))
+      (lyskom-insert (car (lyskom-queue->first 
+                           lyskom-to-be-printed-before-prompt)))
+      (lyskom-queue-delete-first lyskom-to-be-printed-before-prompt))
+    (setq lyskom-executing-command nil)
+    (setq lyskom-current-command nil)
+    (setq lyskom-current-prompt nil)	; Already set in s-o-c really
+    (lyskom-scroll)
+    (setq mode-line-process (lyskom-get-string 'mode-line-waiting))
+    (if (pos-visible-in-window-p (point-max) (selected-window))
+        (lyskom-set-last-viewed))
+    (lyskom-prefetch-and-print-prompt)
+    (run-hooks 'lyskom-after-command-hook)
+    (when (and (lyskom-have-feature idle-time)
+               (not lyskom-is-anonymous))
+      (save-excursion (set-buffer lyskom-buffer)
+                      (initiate-user-active 'background nil)))
+    (if kom-inhibit-typeahead
+        (discard-input))
+    ;; lyskom-pending-commands should probably be a queue or a stack.
+    (when lyskom-pending-commands
       (let ((command (car lyskom-pending-commands)))
-       (setq lyskom-pending-commands (cdr lyskom-pending-commands))
-       (if (symbolp command)
-           (call-interactively command)
-         (eval command))))
-   (when lyskom-slow-mode
-     (buffer-enable-undo))))
+        (setq lyskom-pending-commands (cdr lyskom-pending-commands))
+        (if (symbolp command)
+            (call-interactively command)
+          (eval command))))
+    (when lyskom-slow-mode
+      (buffer-enable-undo))))
 
 (eval-and-compile (provide 'lyskom-command))
 
