@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;;; $Id: completing-read.el,v 44.37 2002/02/24 20:23:26 joel Exp $
+;;;;; $Id: completing-read.el,v 44.52 2003/08/25 17:36:39 byers Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -36,12 +36,11 @@
 (setq lyskom-clientversion-long 
       (concat
        lyskom-clientversion-long
-       "$Id: completing-read.el,v 44.37 2002/02/24 20:23:26 joel Exp $\n"))
+       "$Id: completing-read.el,v 44.52 2003/08/25 17:36:39 byers Exp $\n"))
 
 (defvar lyskom-name-hist nil)
 
 
-
 ;;; ============================================================
 ;;;
 ;;; Name lookup caches
@@ -110,11 +109,13 @@ but first checks a cache."
 (defvar lyskom-minibuffer-local-must-match-map
   (let ((map (copy-keymap minibuffer-local-must-match-map)))
     (lyskom-xemacs-or-gnu 
-     (set-keymap-parent map lyskom-minibuffer-local-completion-map)
+     (lyskom-set-keymap-parent map lyskom-minibuffer-local-completion-map)
      (define-key map " " nil))
     map)
   "Keymap used for reading LysKOM names.")
 
+
+
 (defsubst lyskom-completing-match-string-regexp (string)
   (concat "^"
           (replace-in-string (regexp-quote (lyskom-unicase (lyskom-completing-strip-name string)))
@@ -180,23 +181,148 @@ See lyskom-read-conf for a description of the parameters."
 	  ((lyskom-uconf-stat-p conf-z-info) (uconf-stat->name conf-z-info))
 	  (t (conf-z-info->name conf-z-info)))))
 
+
+;;; ================================================================
+;;; Code to guess defaults for initial input
+;;;
+
+(defun lyskom-default-conference-at-point (&rest args)
+  (let* ((pos (or lyskom-command-point (point)))
+         (type (and pos (get-text-property pos 'lyskom-button-type))))
+    (and (memq type '(conf pers))
+         (list (get-text-property pos 'lyskom-button-arg)))))
+
+(defun lyskom-default-conference-current (&rest args)
+  (list lyskom-current-conf))
+
+(defun lyskom-default-conference-self (&rest args)
+  (list lyskom-pers-no))
+
+(defun lyskom-default-conference-last-author (&rest args)
+  (and (lyskom-get-last-read-text)
+       (list (text-stat->author
+              (blocking-do 'get-text-stat (lyskom-get-last-read-text))))))
+
+(defun lyskom-default-conference-restriction (predicate &rest args)
+  (and (assq 'restrict predicate)
+       (cdr (assq 'restrict predicate))))
+
+(defun lyskom-default-conference-empty (&rest args)
+  nil)
+
+
+(defun lyskom-default-conference-saved (sym &rest args)
+  (save-excursion
+    (when lyskom-buffer (set-buffer lyskom-buffer))
+    (and (cdr (assq (car sym) lyskom-read-conf-saved-inputs))
+         (list (conf-z-info->conf-no 
+                (cdr (assq (car sym) lyskom-read-conf-saved-inputs)))))))
+
+(defun lyskom-default-conference-not-self (uc &rest args)
+  (not (eq (uconf-stat->conf-no uc) lyskom-pers-no)))
+
+(defun lyskom-default-conference-not-current (uc &rest args)
+  (not (eq (uconf-stat->conf-no uc) lyskom-current-conf)))
+
+
+
+(defun lyskom-get-initial-conf-strategy (prompt)
+  (when (listp prompt) (setq prompt (car prompt)))
+  (let* ((spec-1 (cdr (assq (or lyskom-current-command this-command)
+                            lyskom-default-conference-strategy)))
+         (default-spec 
+           (cdr (assq t (assq t lyskom-default-conference-strategy))))
+         (prompt-spec (cdr (assq prompt spec-1)))
+         (cmd-spec (cdr (assq t spec-1))))
+    (lyskom-debug-forms
+     (unless spec-1 
+       (save-excursion
+         (when lyskom-buffer (set-buffer lyskom-buffer))
+         (lyskom-format-insert-before-prompt 
+          "%[%#2@%#1s%]\n"
+          (format "Warning: no strategy for %S/%S"
+                  (or lyskom-current-command this-command)
+                  prompt)
+          `(face ,kom-warning-face)))))
+    (list (or (assq 'default prompt-spec)
+              (assq 'default cmd-spec)
+              (assq 'default default-spec))
+          (or (assq 'filter prompt-spec)
+              (assq 'filter cmd-spec)
+              (assq 'filter default-spec))
+          (or (assq 'save prompt-spec)
+              (assq 'save cmd-spec)
+              (assq 'save default-spec)))))
+
+(defun lyskom-read-conf-guess-initial (prompt predicate)
+  "Return a guess for the initial value for lyskom-read-conf."
+  (let* ((strategy (lyskom-get-initial-conf-strategy prompt))
+         (default (cdr (assq 'default strategy)))
+         (filter (cdr (assq 'filter strategy))))
+
+    (uconf-stat->name
+     (car
+      (filter-list 
+       (lambda (uconf-stat)
+         (and uconf-stat
+              (not (memq nil
+                         (mapcar (lambda (fn)
+                                   (funcall fn uconf-stat))
+                                 filter)))
+              (lyskom-read-conf-internal-verify-type
+               (uconf-stat->conf-no uconf-stat)
+               (uconf-stat->conf-type uconf-stat)
+               predicate nil nil)))
+       (mapcar (lambda (conf-no)
+                 (blocking-do 'get-uconf-stat conf-no)) 
+               (apply 'append 
+                      (delq nil (mapcar (lambda (fn)
+                                          (if (listp fn)
+                                              (funcall (car fn) (cdr fn) predicate)
+                                            (funcall fn predicate)))
+                                        default)))))))))
+
+
+
+(defun lyskom-read-conf-save-input (prompt input)
+  "Save INPUT as input for the current completing read command."
+  (save-excursion
+    (when lyskom-buffer (set-buffer lyskom-buffer))
+    (lyskom-traverse sym (cdr (assq 'save (lyskom-get-initial-conf-strategy prompt)))
+      (if (assq sym lyskom-read-conf-saved-inputs)
+          (setcdr (assq sym lyskom-read-conf-saved-inputs) input)
+        (setq lyskom-read-conf-saved-inputs 
+              (cons (cons sym input) lyskom-read-conf-saved-inputs))
+        ))))
+
 (defun lyskom-read-conf (prompt type &optional empty initial mustmatch)
   "Completing read a conference or person from the minibuffer. 
 
-PROMPT is the prompt type type.
+PROMPT is the prompt. It can be a string, symbol or list. If it is a symbol,
+       lyskom-get-string will be used to get the string. If it is a list,
+       it is used as the argument list to lyskom-format to create the
+       prompt. You should use symbols and lists only.
 TYPE   is the type of conferences to return. It is a list of one or
-more of the following:
-    all     Return any conference,
-    conf    Return conferences (not letterboxes),
-    pers    Return persons (letterboxes),
-    login   Return persons who are also logged-in, and
-    none    Return names that do not match anything in the database.
-    (restrict c1 c2 ...) Restrict matching to conference numbers c1, 
-            c2 etc. The implementation is inefficient for long lists.
+       more of the following:
+
+       all     Return any conference.
+       conf    Return conferences (not letterboxes).
+       pers    Return persons (letterboxes).
+       login   Return persons who are also logged-in.
+       membership Return only conferences and letterboxes lyskom-pers-no
+               is a member of.
+       none    Return names that do not match anything in the database.
+       (restrict c1 c2 ...) Restrict matching to conference numbers c1, 
+               c2 etc. The implementation is inefficient for long lists.
 
 Optional arguments
+
 EMPTY     allow nothing to be entered.
-INITIAL   initial contents of the minibuffer
+
+INITIAL   initial contents of the minibuffer. If an integer, use the
+          name of that conference. This is normally computed automatically,
+          so pass nil whenever possible.
+
 MUSTMATCH if non-nil, the user must enter a valid name.
 
 The return value may be one of
@@ -204,33 +330,59 @@ A conf-z-info: The conf-z-info associated with the name entered,
 nil:         Nothing was entered, or
 A string:    A name that matched nothing in the database."
 
-  (lyskom-completing-clear-cache)
-  (let* ((completion-ignore-case t)
-         (minibuffer-local-completion-map 
-          lyskom-minibuffer-local-completion-map)
-         (minibuffer-local-must-match-map 
-          lyskom-minibuffer-local-must-match-map)
-         (read-string nil)
-         (result nil)
-         (keep-going t))
+  ;; We bind lyskom-current-command here so that changes to
+  ;; this-command will not confuse our guesses for the initial value
+  ;; (particularly at the end of this function, where we sometimes
+  ;; save the value the user entered).
 
-    (while keep-going
-      (lyskom-with-lyskom-minibuffer
-       (setq read-string (completing-read (cond ((stringp prompt) prompt)
-                                                ((symbolp prompt) (lyskom-get-string prompt))
-                                                (t (lyskom-get-string 'conf-prompt)))
-                                          'lyskom-read-conf-internal
-                                          type
-                                          mustmatch
-                                          initial
-                                          'lyskom-name-hist)))
-      (setq result
-            (cond ((null read-string) nil)
-                  ((string= "" read-string) nil)
-                  (t (lyskom-lookup-conf-by-name read-string type))))
-      (setq keep-going (and (not empty)
-                            (null result))))
-    result))
+  (let ((lyskom-current-command (or lyskom-current-command this-command)))
+    (lyskom-completing-clear-cache)
+    (setq initial
+          (cond ((integerp initial)
+                 (uconf-stat->name (blocking-do 'get-uconf-stat initial)))
+                ((stringp initial) initial)
+                ((lyskom-conf-stat-p initial)
+                 (conf-stat->name initial))
+                ((lyskom-uconf-stat-p initial)
+                 (uconf-stat->name initial))
+                ((lyskom-conf-z-info-p initial)
+                 (conf-z-info->name initial))
+                ((consp initial) initial)
+                ((lyskom-read-conf-guess-initial prompt type))
+                (t nil)))
+
+    (let* ((completion-ignore-case t)
+           (minibuffer-local-completion-map 
+            lyskom-minibuffer-local-completion-map)
+           (minibuffer-local-must-match-map 
+            lyskom-minibuffer-local-must-match-map)
+           (read-string nil)
+           (old-prompt prompt)
+           (result nil)
+           (keep-going t))
+
+      (setq prompt (cond ((stringp prompt) prompt)
+                         ((symbolp prompt) (lyskom-get-string prompt))
+                         ((listp prompt) (apply 'lyskom-format prompt))
+                         (t (lyskom-get-string 'conf-prompt))))
+
+      (while keep-going
+        (setq read-string (lyskom-completing-read prompt
+                                                  'lyskom-read-conf-internal
+                                                  type
+                                                  mustmatch
+                                                  (if (listp initial)
+                                                      initial
+                                                    (cons initial 0))
+                                                  'lyskom-name-hist))
+        (setq result
+              (cond ((null read-string) nil)
+                    ((string= "" read-string) nil)
+                    (t (lyskom-lookup-conf-by-name read-string type))))
+        (setq keep-going (and (not empty)
+                              (null result))))
+      (lyskom-read-conf-save-input old-prompt result)
+      result)))
 
 
 (defun lyskom-read-conf-get-logins ()
@@ -329,9 +481,11 @@ function work as a name-to-conf-stat translator."
                             (lyskom-read-conf-get-logins)))
            (x-list (lyskom-completing-lookup-z-name string 
                                                     (if (or (memq 'all predicate)
+                                                            (memq 'membership predicate)
                                                             (memq 'conf predicate)
                                                             (memq 'none predicate)) 1 0)
                                                     (if (or (memq 'all predicate)
+                                                            (memq 'membership predicate)
                                                             (memq 'pers predicate)
                                                             (memq 'none predicate)
                                                             (memq 'login predicate)) 1 0)))
@@ -343,11 +497,11 @@ function work as a name-to-conf-stat translator."
                        (lyskom-wait-queue 'main)
                        (delq nil
                              (mapcar (lambda (conf-stat)
-                                       (when (lyskom-completing-match-string string (conf-stat->name conf-stat))
+                                       (when (lyskom-completing-match-string string (uconf-stat->name conf-stat))
                                          (lyskom-create-conf-z-info
-                                          (conf-stat->name conf-stat)
-                                          (conf-stat->conf-type conf-stat)
-                                          (conf-stat->conf-no conf-stat))))
+                                          (uconf-stat->name conf-stat)
+                                          (uconf-stat->conf-type conf-stat)
+                                          (uconf-stat->conf-no conf-stat))))
                                (collector->value result))))))
            (candidate-list 
             (append r-list
@@ -578,29 +732,33 @@ function work as a name-to-conf-stat translator."
            conf-type
            (conf-type->letterbox conf-type))
       (and (memq 'login predicate)
-           conf-type
+           conf-no
            (memq conf-no logins))
+      (and (memq 'membership predicate)
+           conf-no
+           (lyskom-get-membership conf-no t))
       (and (memq 'none predicate) 
            (and (null conf-no)
                 (null x-list)))))
 
+;;; FOR DEBUGGING (DON'T DELETE)     /byers
+;;;
+;;; (defun lyskom-complete-show-data-list (state data)
+;;;   (save-excursion
+;;;     (pop-to-buffer (get-buffer-create "*kom*-complete"))
+;;;     (erase-buffer)
+;;;     (lyskom-set-buffer-multibyte nil)
+;;;    (while data
+;;;       (insert
+;;;        (format "%s\n" (substring (aref (car data) 2)
+;;;                                  (aref (car data) 0)
+;;;                                  (aref (car data) 1))))
+;;;       (setq data (cdr data)))
+;;;     (insert (format "%S %S: %S" (symbol-value current-state)
+;;;                     (elt state 0)
+;;;                     (elt state 1)))
+;;;     (sit-for 5)))
 
-; (defun lyskom-complete-show-data-list (state data)
-;   (save-excursion
-;     (pop-to-buffer (get-buffer-create "*kom*-complete"))
-;     (erase-buffer)
-;     (set-buffer-multibyte nil)
-;    (while data
-;       (insert
-;        (format "%s\n" (substring (aref (car data) 2)
-;                                  (aref (car data) 0)
-;                                  (aref (car data) 1))))
-;       (setq data (cdr data)))
-;     (insert (format "%S %S: %S" (symbol-value current-state)
-;                     (elt state 0)
-;                     (elt state 1)))
-;     (sit-for 5)))
-      
 
 (defun lyskom-complete-string (string-list)
   "Find the longest common prefix of all strings in STRING-LIST according to
@@ -962,7 +1120,7 @@ the LysKOM rules of string matching."
   (lyskom-insert "\n")
   (let* ((s-width (1+ (apply 'max (mapcar (function
 					   (lambda (x)
-					     (string-width (int-to-string x))))
+					     (lyskom-string-width (int-to-string x))))
 					  sessions))))
 	 (format-string-s (lyskom-info-line-format-string s-width "s" "s"))
 	 (format-string-p (lyskom-info-line-format-string s-width "P" "M")))
@@ -1011,15 +1169,14 @@ the LysKOM rules of string matching."
       (lyskom-insert (lyskom-format 'total-users-sans-date (length who-info)))
       (lyskom-scroll)
       (while (string= ""
-                      (lyskom-with-lyskom-minibuffer
-                       (setq result (lyskom-completing-read
+                      (setq result (lyskom-completing-read
 				    (lyskom-get-string 'resolve-session)
 				    (lyskom-maybe-frob-completion-table 
 				     who-info)
 				    nil
 				    t
 				    (car (car who-info))
-				    nil)))))
+				    nil))))
       (list (session-info->connection (cdr (assoc result who-info)))))))
 
 
