@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;;; $Id: edit-text.el,v 44.105 2002/09/18 20:48:32 byers Exp $
+;;;;; $Id: edit-text.el,v 44.119 2005/01/09 01:16:01 byers Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -34,7 +34,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: edit-text.el,v 44.105 2002/09/18 20:48:32 byers Exp $\n"))
+	      "$Id: edit-text.el,v 44.119 2005/01/09 01:16:01 byers Exp $\n"))
 
 
 ;;;; ================================================================
@@ -250,22 +250,17 @@ nil             -> Ingenting."
 				 (substitute-command-keys
 				  (lyskom-get-string 'header-separator))
 				 body 
-				 (lyskom-get-string 'header-subject))
+				 (lyskom-get-string 'header-subject)
+                                 '(read-only t)
+                                 '(read-only t front-sticky nil start-open t rear-nonsticky nil end-open nil)
+                                 '(read-only t rear-nonsticky t end-open t front-sticky t start-open nil)
+                                 )
 		  where-put-misc)
     (set-buffer edit-buffer)
     (goto-char where-put-misc)
     ))
 
 
-(defun lyskom-edit-goto-char (marker)
-  "Positions the editing at MARKER."
-  (let ((curbuf (current-buffer)))
-    (set-buffer (marker-buffer marker))
-    (save-window-excursion
-      (goto-char marker))
-    (set-buffer curbuf)))
-
-
 (defun lyskom-edit-insert-misc-conf (conf-stat string stream number)
   "Insert Recipient:, Carbon copy: or Blind Carbon copy: in
 edit-buffer.
@@ -441,12 +436,16 @@ so it's not as clean as it ought to be."
 ;;;
 
 (defun kom-edit-send-anonymous ()
-  "Send the text anonymously to the server."
+  "Send the text anonymously to the server. Be aware that although the text 
+will be truly anonymous, it is easy to slip up in such a way that the author
+is evident anyway."
   (interactive)
   (lyskom-edit-send 'initiate-create-anonymous-text t))
 
 (defun kom-edit-send ()
-  "Send the text to the server."
+  "Send the text to the server. This command will attempt to send the text
+to the server. If something goes wrong, a prompt will be shown allowing you
+to edit the message and try to send it again."
   (interactive)
    (if (and (lyskom-default-value 'lyskom-is-anonymous)
             (lyskom-j-or-n-p 'do-send-anonymous))
@@ -468,6 +467,15 @@ anonymously and take actions to avoid revealing the sender."
                   (message nil)
                   (aux-list nil))
 
+              ;;
+              ;; Run user hooks
+              ;; ####: ++++: FIXME: We should quit more graciously.
+
+              (if (and (not (run-hook-with-args-until-failure 'kom-send-text-hook))
+                       (not (run-hook-with-args-until-failure 'lyskom-send-text-hook)))
+                  (signal 'lyskom-edit-text-abort nil))
+
+
 	      (save-excursion
 		(setq headers (lyskom-edit-parse-headers)
 		      misc-list (apply 'lyskom-create-misc-list 
@@ -512,13 +520,6 @@ anonymously and take actions to avoid revealing the sender."
                 (rplacd misc-list (delq (rassq lyskom-pers-no (cdr misc-list))
                                         (cdr misc-list))))
 
-              ;;
-              ;; Run user hooks
-              ;; ####: ++++: FIXME: We should quit more graciously.
-
-              (if (and (not (run-hook-with-args-until-failure 'kom-send-text-hook))
-                       (not (run-hook-with-args-until-failure 'lyskom-send-text-hook)))
-                  (signal 'lyskom-edit-text-abort nil))
 
               ;;
               ;; Transform the message text
@@ -540,8 +541,10 @@ anonymously and take actions to avoid revealing the sender."
                               (t (concat (or subject "") "\n" message))))
                        (content-type (lyskom-get-aux-item aux-list 1))
                        (charset (and content-type
-                                     (cdr (lyskom-mime-decode-content-type
-                                           (aux-item->data (car content-type))))))
+                                     (lyskom-mime-content-type-get
+                                      (lyskom-mime-decode-content-type
+                                           (aux-item->data (car content-type)))
+                                      'charset)))
                        (mime-charset (lyskom-mime-string-charset full-message)))
 
                   ;; If the charset isn't already set, encode the string
@@ -655,10 +658,11 @@ anonymously and take actions to avoid revealing the sender."
 (eval-when-compile (defvar ispell-message-end-skip nil))
 
 
+;;USER-HOOK: lyskom-ispell-text
 (defun lyskom-ispell-text ()
   "Check spelling of the text body.
 Put this in kom-send-text-hook"
-  (kom-ispell-message)
+  (kom-edit-ispell-message)
   t)
 
 
@@ -668,10 +672,13 @@ Put this in kom-send-text-hook"
   (defvar ispell-message-start-skip nil)
   (defvar ispell-message-end-skip nil))
 
-(defun kom-ispell-message ()
-  "Check spelling of the text.
-kom-ispell-dictionary is the dictionary to use to check spelling.
-Based on ispell-message."
+(defalias 'kom-ispell-message 'kom-edit-ispell-message)
+(defun kom-edit-ispell-message ()
+  "Check spelling of the text. Spelling is checked using ispell
+and the dictionary indicated by `kom-ispell-dictionary'. If you
+want to check the spelling of every message before sending it,
+read the documentation for `lyskom-ispell-text' and 
+`kom-send-text-hook'"
   (interactive)
   (require 'ispell)
   (let ((ispell-dictionary (or kom-ispell-dictionary ispell-dictionary))
@@ -916,24 +923,7 @@ Cannot be called from a callback."
         ;;
 
         (lyskom-traverse author raw-author-list
-          (let ((send-comments-to
-                 (car (lyskom-get-aux-item (conf-stat->aux-items author) 33))))
-
-            (when send-comments-to
-              (cond ((string-match "^\\([0-9]+\\)\\s-+\\([0-9]+\\)" (aux-item->data send-comments-to))
-                     (setq send-comments-to 
-                           (cons (string-to-number
-                                  (match-string 1 (aux-item->data send-comments-to)))
-                                 (string-to-number
-                                  (match-string 2 (aux-item->data send-comments-to))))))
-                    ((string-match "^\\([0-9]+\\)"
-                                   (aux-item->data send-comments-to))
-                     (setq send-comments-to
-                           (cons (string-to-number
-                                  (match-string 1 (aux-item->data send-comments-to)))
-                                 0)))
-                    (t (setq send-comments-to nil))))
-
+          (let ((send-comments-to (lyskom-get-send-comments-to author t)))
             (cond
 
              ;; Author is in kom-dont-check-commented-authors
@@ -977,6 +967,8 @@ Cannot be called from a callback."
                                            (collector-push x y))
                                          (conf-stat->conf-no author)
                                          recipient
+                                         nil
+                                         0
                                          memberships))
             (lyskom-wait-queue 'sending)
             (setq memberships (collector->value memberships))
@@ -1039,7 +1031,9 @@ Cannot be called from a callback."
                             (conf-stat->conf-no author))
                      extra-headers)))))))
     extra-headers))
-    
+
+
+;;UNUSED: lyskom-send-enriched
 (defun lyskom-send-enriched (message)
   (condition-case err
       (let ((buf (lyskom-get-buffer-create 'lyskom-enriched 
@@ -1071,7 +1065,7 @@ Cannot be called from a callback."
 
 
 (defun kom-edit-quit ()
-  "Kill the text (if any) written so far and continue reading."
+  "Cancel editing the text being written and return to reading LysKOM."
   (interactive)
   (let ((edit-buffer (current-buffer)))
     (goto-char (point-max))
@@ -1091,8 +1085,11 @@ Cannot be called from a callback."
 
 
 (defun kom-edit-insert-commented (arg)
-  "Insert the commented text. Unless an empty prefix argument is
-given, prepend each line with your commenting prefix (or '>')."
+  "Insert the commented text, prepending each line with the
+text in `kom-cite-string' (defaults to \"> \"). Note that citing
+the commented text is not common practise in LysKOM (unlike
+e-mail and news) since there is a strong link to the commented
+text anyway. Use this command sparingly."
   (interactive "P")
   (lyskom-edit-get-commented
    'lyskom-edit-insert-commented
@@ -1101,25 +1098,28 @@ given, prepend each line with your commenting prefix (or '>')."
 
 
 (defun kom-edit-insert-buglist ()
-  "Insert the commented buglist, Roxen Internet Software style."
+  "Insert the commented buglist, Roxen Internet Software style. Excluded from manual."
   (interactive)
   (lyskom-edit-get-commented 'lyskom-edit-insert-buglist))
 
 
 (defun kom-edit-insert-digit-text ()
+  "Prompt for a text to insert. Excluded from manual."
   (interactive)
   (setq unread-command-events (cons last-command-event unread-command-events))
   (call-interactively 'kom-edit-insert-text nil))
 
 
 (defun kom-edit-insert-text (no)
-  "Insert the text number NO with '>' first on each line."
+  "Prompt for a text to insert, prefixing each line with the contents
+of `kom-cite-string' (defaults to \"> \"). Note that citing texts is not
+commonplace in LysKOM (unlike e-mail and news) since it is easy to
+refer to specific texts (see `kom-edit-insert-link' and
+`kom-edit-add-cross-reference'). Use this command sparingly."
   (interactive (list
 		(cond
 		 ((null current-prefix-arg)
-		  (string-to-int 
-		   (lyskom-read-from-minibuffer 
-		    (format "%s" (lyskom-get-string 'which-text-include)))))
+                  (lyskom-read-number 'which-text-include))
 		 ((prefix-numeric-value current-prefix-arg)))))
   (let ((buffer (current-buffer))
         (window (selected-window)))
@@ -1169,8 +1169,12 @@ WINDOW plus any optional arguments given in ARG-LIST."
 (defvar Info-current-file)
 
 ;; NOTUSED: kom-yank-info-nodename
-(defun kom-yank-info-nodename ()
-  "Put the current Info-node on the kill-ring."
+(defalias 'kom-yank-info-nodename 'kom-edit-yank-info-nodename)
+(defun kom-edit-yank-info-nodename ()
+  "When browsing info files, this command will place a reference to the
+current info node in the kill ring, from where it can be pasted into
+another buffer. This command is useful when you want to refer to an 
+info node in a LysKOM text."
   (interactive)
   (kill-new (format "*Note %s: (%s)%s,"
 		    Info-current-node
@@ -1179,7 +1183,8 @@ WINDOW plus any optional arguments given in ARG-LIST."
 
 
 ;; NOTUSED: kom-insert-last-info-nodename
-(defun kom-insert-last-info-nodename ()
+(defalias 'kom-insert-last-info-nodename 'kom-edit-insert-last-info-nodename)
+(defun kom-edit-insert-last-info-nodename ()
   "Insert a reference to the most recently visited info node."
   (interactive)
   (condition-case nil
@@ -1200,7 +1205,10 @@ WINDOW plus any optional arguments given in ARG-LIST."
 ;;;  Changed by: Linus Tolke
 
 (defun kom-edit-add-comment ()
-  "Adds a text as commented to the text being edited."
+  "Makes this text a comment to another text. Using this command it is
+possible to make the text a comment to multiple texts. To remove a 
+comment link, simply remove the corresponding line from the headers
+in the edit buffer."
   (interactive)
   (let* ((edit-buffer (current-buffer))
          (insert-at (point-min-marker))
@@ -1217,33 +1225,40 @@ WINDOW plus any optional arguments given in ARG-LIST."
 
 
 (defun kom-edit-add-recipient ()
-  "Adds a conference as recipient to the text being edited."
+  "Adds a regular recipient to the text or converts an existing
+recipient to a regular recipient. Using this command it is possible to
+add any number of regular recipients. To remove a recipient, simply
+delete the corresponding header line in the edit buffer."
   (interactive)
-  (lyskom-edit-add-recipient/copy (lyskom-get-string 'added-recipient)
-                                 nil
-                                  'RECPT))
+  (lyskom-edit-add-recipient/copy 'added-recipient nil 'RECPT))
 
 
 (defun kom-edit-add-bcc ()
-  "Adds a conference as bcc recipient to the text being edited."
+  "Adds a blind carbon copy recipient to the text, or converts an
+existing recipient to blind carbon copy. Using this command it is
+possible to add any number of regular recipients. To remove a
+recipient, simply delete the corresponding header line in the edit
+buffer."
   (interactive)
-  (lyskom-edit-add-recipient/copy (lyskom-get-string 'added-blank-carbon-copy)
-                                  nil
-                                  'BCC-RECPT))
+  (lyskom-edit-add-recipient/copy 'added-blank-carbon-copy nil 'BCC-RECPT))
 
 
 (defun kom-edit-add-copy ()
-  "Adds a conference to which a copy of the edited text will be sent."
+  "Adds a carbon copy recipient to the text, or converts an existing
+recipient to blind carbon copy. Using this command it is possible to
+add any number of regular recipients. To remove a recipient, simply
+delete the corresponding header line in the edit buffer."
   (interactive)
-  (lyskom-edit-add-recipient/copy (lyskom-get-string 'added-carbon-copy)
-                                  nil
-                                  'CC-RECPT))
+  (lyskom-edit-add-recipient/copy 'added-carbon-copy nil 'CC-RECPT))
 
 (defun kom-edit-move-text ()
-  "Adds a conference as a recipient, and changes all other recipients to
-CC recipients."
+  "Adds a regular recipient to the text, or converts an existing
+recipient to blind carbon copy, and converts all other recipients to
+carbonn copy recipients. This command is intended for situations where
+a commend is being sent to a different recipient than the commented
+text was."
   (interactive)
-  (lyskom-edit-add-recipient/copy (lyskom-get-string 'who-to-move-to-q)
+  (lyskom-edit-add-recipient/copy 'who-to-move-to-q
                                   'lyskom-edit-move-recipients))
 
 
@@ -1293,15 +1308,14 @@ CC recipients."
                                    (elt headers 2)))))
 
 
-(defun lyskom-edit-add-recipient/copy (prompt 
-                                       &optional what-to-do recpt-type)
+(defun lyskom-edit-add-recipient/copy (prompt &optional what-to-do recpt-type)
   "Adds a new recipient or a cc-recipient to the text which is being edited.
 PROMPT is the prompt to use to ask the user for a recipient.
 WHAT-TO-DO is a function to call to do the insertion.
 RECPT-TYPE is the type of recipient to add."
   (let ((edit-buffer (current-buffer))
 	(insert-at (point-min-marker))
-	(conf-stat (lyskom-read-conf-stat prompt '(all) nil "" t)))
+	(conf-stat (lyskom-read-conf-stat prompt '(all) nil nil t)))
     (lyskom-save-excursion
      (save-excursion
      (set-buffer lyskom-buffer)
@@ -1364,12 +1378,21 @@ RECPT-TYPE is the type of recipient to add."
     
 
 (defun kom-edit-add-cross-reference ()
+  "Add a cross reference to this text. This command prompts for the
+type and target of the cross reference. Cross references are used
+to systematically refer from a text to another text, conference or
+person without altering the contents of the text."
   (interactive)
   (let ((item (lyskom-read-cross-reference-and-get-aux-item)))
     (when item
       (lyskom-edit-insert-aux-item item))))
 
 (defun kom-edit-add-read-confirm-request ()
+  "Request that others confirm reading this text. Conforming clients
+will ask each reader of this text to confirm reading the text. Note
+that not all clients understand this request, and that confirmation is
+neither automatic nor mandatory. Use very sparingly as this interrupts
+the normal flow of reading for many users."
   (interactive)
   (lyskom-edit-insert-aux-item
    (lyskom-create-aux-item 0 6 0 0
@@ -1378,6 +1401,10 @@ RECPT-TYPE is the type of recipient to add."
                            0 "")))
 
 (defun kom-edit-add-no-comments ()
+  "Request that nobody comments this text. Conforming clients will
+either prevent users from commenting the text or ask for confirmation
+before commenting the text. Note that not all clients understand this
+request and that it is advisory only."
   (interactive)
   (lyskom-edit-insert-aux-item
    (lyskom-create-aux-item 0 4 0 0
@@ -1386,6 +1413,10 @@ RECPT-TYPE is the type of recipient to add."
                            0 "")))
 
 (defun kom-edit-add-personal-comments ()
+  "Request that all replies to this text are in the form of personal
+replies. Conforming clients will treat a request to comment this text
+as a request to answer privately. Note that not all clients understand
+this request and that it is advisory only."
   (interactive)
   (lyskom-edit-insert-aux-item
    (lyskom-create-aux-item 0 5 0 0
@@ -1394,6 +1425,10 @@ RECPT-TYPE is the type of recipient to add."
                            0 "")))
 
 (defun kom-edit-add-world-readable ()
+  "Make this text readable to all users, even if they are not logged
+in. World readable texts can be read by anyone, regardless of what
+recipients the text has. World readable texts can even be read without
+logging in to LysKOM."
   (interactive)
   (lyskom-edit-insert-aux-item
    (lyskom-create-aux-item 0 34 0 0
@@ -1402,14 +1437,52 @@ RECPT-TYPE is the type of recipient to add."
                            0 "")))
 
 (defun kom-edit-insert-link ()
+  "Insert an in-line link to another text, a conference or a person.
+This command prompts for the target of the link and inserts text that
+most clients will interpret as a link."
   (interactive)
   (let ((item (lyskom-read-link)))
     (when item
       (insert item))))
-    
 
 
 
+(defun lyskom-read-link ()
+  "Query user about link type and value, and return the corresponding
+link as a string."
+  (let* ((type (lyskom-a-or-b-or-c-p 'link-type
+                                     '(abc-conference
+                                       abc-person
+                                       abc-text)
+                                     nil))
+         (obj nil))
+    (cond
+     ((eq type 'abc-text)
+      (let ((prompt 'which-text-to-link))
+        (while (null obj)
+          (setq obj (blocking-do 'get-text-stat
+                                 (lyskom-read-number prompt)))
+          (setq prompt 'which-text-to-link-err )))
+      (let* ((text-no (text-stat->text-no obj))
+             (text (blocking-do 'get-text text-no))
+	     (txt (text->decoded-text-mass text obj))
+	     (eos (string-match (regexp-quote "\n") txt))
+	     (subject (substring txt 0 eos)))
+	(format "<text %d: %s>" text-no subject)))
+     
+     ((eq type 'abc-conference)
+      (while (null obj)
+        (setq obj (lyskom-read-conf-stat 'which-conf-to-link '(conf) nil nil t)))
+      (format "<mte %d: %s>" (conf-stat->conf-no obj)
+			   (conf-stat->name obj)))
+
+     ((eq type 'abc-person)
+      (while (null obj)
+        (setq obj (lyskom-read-conf-stat 'which-pers-to-link '(pers) nil nil t)))
+      (format "<person %d: %s>" (conf-stat->conf-no obj)
+			   (conf-stat->name obj))))))
+
+
 (defun lyskom-edit-insert-aux-item (item)
   "Insert the aux item ITEM in the current buffer"
   (save-excursion
@@ -1531,6 +1604,7 @@ to lyskom-edit-replace-headers"
   "Replace all headers with SUBJECT and MISC-LIST"
   (save-excursion
     (let ((start nil)
+          (inhibit-read-only t)
           (end nil))
       (goto-char (point-min))
       (setq start (point-marker))
@@ -1676,14 +1750,6 @@ Point must be located on the line where the subject is."
 			  (point))))))
 
 
-(defun lyskom-edit-on-aux-item ()
-  "Return non-nil if point is on the same line as an aux-item"
-  (save-excursion
-    (beginning-of-line)
-    (and (lyskom-looking-at (lyskom-get-string 'aux-item-prefix-regexp))
-         (> (lyskom-edit-find-separator) (point)))))
-
-
 (defun lyskom-create-text-handler (text-no edit-buffer 
                                            &optional is-anonymous)
   "Handle an attempt to write a text."
@@ -1776,7 +1842,7 @@ Point must be located on the line where the subject is."
 
 (defun lyskom-edit-fcc-text (conf-stat text text-no is-anonymous)
   (condition-case arg
-      (let ((start 1) (end 0))
+      (let ((start 1) (end 0) (inhibit-read-only t))
         (save-excursion
           (set-buffer (lyskom-get-buffer-create 'fcc "*kom*-fcc" t))
           (erase-buffer)
@@ -1945,6 +2011,7 @@ buglist style, automating the removal of closed subjects and change-marks."
 ;;; Tab between buttons in the header
 
 (defun kom-edit-next-button-or-self-insert (num)
+  "Move to the next link or insert a TAB, depending on context. Excluded from manual."
   (interactive "p")
   (let ((header-end (lyskom-edit-find-separator))
         (start (point))
@@ -1982,6 +2049,7 @@ buglist style, automating the removal of closed subjects and change-marks."
         (setq num (1- num) start (point))))))
 
 (defun kom-edit-prev-button (num)
+  "Move to the previous link. Excluded from manual."
   (interactive "p")
   (let ((header-end (lyskom-edit-find-separator)))
     (while (> num 0)
