@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;;; $Id: faqs.el,v 44.5 2002/10/16 20:22:16 byers Exp $
+;;;;; $Id: faqs.el,v 44.20 2004/06/26 13:32:32 byers Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -33,9 +33,10 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-              "$Id: faqs.el,v 44.5 2002/10/16 20:22:16 byers Exp $\n"))
+              "$Id: faqs.el,v 44.20 2004/06/26 13:32:32 byers Exp $\n"))
 
 (defun lyskom-register-read-faq (conf-no text-no)
+  (unless conf-no (setq conf-no 0))
   (unless (lyskom-faq-is-read conf-no text-no)
     (setq lyskom-read-faqs (cons (cons conf-no text-no) lyskom-read-faqs))
     (initiate-modify-conf-info 
@@ -55,7 +56,7 @@
 
 (defun lyskom-faq-is-read (conf-no text-no)
   "Return non-nil if lyskom-pers-no has read CONF-NOs FAQ TEXT-NO."
-  (member (cons conf-no text-no) lyskom-read-faqs))
+  (member (cons (or conf-no 0) text-no) lyskom-read-faqs))
 
 (defun lyskom-update-read-faqs ()
   "Update the list of read FAQs from the server."
@@ -76,14 +77,27 @@
 ;;;
 
 (def-kom-command kom-add-faq (&optional conf-no text-no)
-  "Add a FAQ to a conference"
-  (interactive (list (lyskom-read-conf-no 'conf-to-add-faq '(conf pers) nil nil t)
-                     (lyskom-read-text-no-prefix-arg 'text-to-add-as-faq nil 'last-seen-written)))
+  "Make an existing text a FAQ of a conference. To write a new FAQ for
+a conference that doesn't have one, or change an existing FAQ, use
+`kom-change-conf-faq'.
+
+This command accepts text number prefix arguments \(see
+`lyskom-read-text-no-prefix-arg')."
+  (interactive (list (lyskom-read-conf-no 'conf-to-add-faq
+                                          '(conf pers) nil nil t)
+                     (lyskom-read-text-no-prefix-arg 'text-to-add-as-faq)))
   (lyskom-add-faq conf-no text-no))
 
+
+
 (def-kom-command kom-add-server-faq (&optional text-no)
-  "Add a FAQ to the server"
-  (interactive (list (lyskom-read-text-no-prefix-arg 'text-to-add-as-faq nil 'last-seen-written)))
+  "Make an existing text a FAQ of the server. To write a new FAQ for
+a conference that doesn't have one, or change an existing FAQ, use
+`kom-change-server-faq'.
+
+This command accepts text number prefix arguments \(see
+`lyskom-read-text-no-prefix-arg')."
+  (interactive (list (lyskom-read-text-no-prefix-arg 'text-to-add-as-faq)))
   (lyskom-add-faq nil text-no))
 
 
@@ -115,13 +129,15 @@ The text to add is passed in TEXT-NO"
                         (list aux-item))))))))
 
 (def-kom-command kom-del-server-faq ()
-  "Remove a FAQ from the server"
+  "Remove a FAQ from the server. You need administrative rights to
+do this. To add a FAQ, use `kom-add-server-faq'."
   (interactive)
   (lyskom-del-faq nil))
 
 
 (def-kom-command kom-del-faq ()
-  "Remove a FAQ from a conference"
+  "Remove a FAQ from a conference. To add a FAQ, use
+`kom-add-faq'."
   (interactive)
   (let* ((conf-stat (lyskom-read-conf-stat 'conf-to-del-faq
                                            '(conf pers) nil nil t)))
@@ -129,15 +145,17 @@ The text to add is passed in TEXT-NO"
 
 (defun lyskom-del-faq (conf-stat)
   (let ((faq-list 
-         (mapcar (lambda (aux)
-                   (cons (aux-item->data aux)
-                         (aux-item->aux-no aux)))
-                 (lyskom-get-aux-item
-                  (if (null conf-stat)
-                      (server-info->aux-item-list
-                       (blocking-do 'get-server-info))
-                    (conf-stat->aux-items conf-stat))
-                  14)))
+         (delq nil
+               (mapcar (lambda (aux)
+                         (and (lyskom-string-to-int (aux-item->data aux))
+                              (cons (aux-item->data aux)
+                                    (aux-item->aux-no aux))))
+                       (lyskom-get-aux-item
+                        (if (null conf-stat)
+                            (server-info->aux-item-list
+                             (blocking-do 'get-server-info))
+                          (conf-stat->aux-items conf-stat))
+                        14))))
         (text-no nil))
 
     (cond
@@ -168,18 +186,21 @@ The text to add is passed in TEXT-NO"
                           nil))))))))
 
 (def-kom-command kom-review-server-faq ()
-  "View the FAQs for the server"
+  "View the FAQs for the server."
   (interactive)
   (lyskom-review-faq nil (server-info->aux-item-list
                           (blocking-do 'get-server-info))))
 
+(def-kom-command kom-unread-server-faq ()
+  "Mark the FAQs for the server as unread."
+  (interactive)
+  (lyskom-unread-faq nil (server-info->aux-item-list
+                          (blocking-do 'get-server-info))))
+
 (def-kom-command kom-review-faq (&optional conf-no)
-  "View the FAQs for a conference"
+  "View the FAQs for a particular conference."
   (interactive 
-   (list 
-    (let* ((conf-stat (blocking-do 'get-conf-stat lyskom-current-conf))
-           (initial (and conf-stat (cons (conf-stat->name conf-stat) 0))))
-      (lyskom-read-conf-no 'view-which-faq '(conf pers) t initial t))))
+   (list (lyskom-read-conf-no 'view-which-faq '(conf pers) t nil t)))
   (if (zerop conf-no)
       (lyskom-review-faq nil (server-info->aux-item-list
                               (blocking-do 'get-server-info)))
@@ -188,11 +209,36 @@ The text to add is passed in TEXT-NO"
           (lyskom-review-faq conf-stat (conf-stat->aux-items conf-stat))
         (lyskom-format-insert 'conf-no-does-not-exist-r conf-no)))))
 
+(def-kom-command kom-unread-faq (&optional conf-no)
+  "Marks the FAQs for a particular conference as unread."
+  (interactive 
+   (list (lyskom-read-conf-no 'unread-which-faq '(conf pers) t nil t)))
+  (if (zerop conf-no)
+      (lyskom-unread-faq nil (server-info->aux-item-list
+                              (blocking-do 'get-server-info)))
+    (let ((conf-stat (blocking-do 'get-conf-stat conf-no)))
+      (if conf-stat
+          (lyskom-unread-faq conf-stat (conf-stat->aux-items conf-stat))
+        (lyskom-format-insert 'conf-no-does-not-exist-r conf-no)))))
+
+
+(defun lyskom-unread-faq (conf-stat aux-list)
+  (let ((faq-list (delq nil 
+                        (mapcar (lambda (aux)
+                                  (lyskom-string-to-int (aux-item->data aux)))
+                                (lyskom-get-aux-item aux-list 14)))))
+    (if (null faq-list) 
+        (lyskom-format-insert 'conf-has-no-faq conf-stat)
+      (lyskom-traverse text-no faq-list
+        (lyskom-format-insert 'marking-text-unread text-no)
+        (lyskom-report-command-answer 
+         (lyskom-mark-unread text-no))))))
 
 (defun lyskom-review-faq (conf-stat aux-list)
-  (let ((faq-list (mapcar (lambda (aux)
-                            (string-to-int (aux-item->data aux)))
-                          (lyskom-get-aux-item aux-list 14))))
+  (let ((faq-list (delq nil
+                        (mapcar (lambda (aux)
+                                  (lyskom-string-to-int (aux-item->data aux)))
+                                (lyskom-get-aux-item aux-list 14)))))
     (cond 
      ((null faq-list) 
       (lyskom-format-insert 'conf-has-no-faq conf-stat))
@@ -211,9 +257,9 @@ The text to add is passed in TEXT-NO"
       (lyskom-do-review-faq faq-list)
       ))))
 
-(defun lyskom-do-review-faq (faq-list)
+(defun lyskom-do-review-faq (faq-list &optional review-tree)
   (read-list-enter-read-info
-   (lyskom-create-read-info 'REVIEW-FAQ
+   (lyskom-create-read-info (if review-tree 'REVIEW-FAQ-TREE 'REVIEW-FAQ)
                             nil
                             (lyskom-review-get-priority)
                             (lyskom-create-text-list faq-list)
@@ -221,24 +267,19 @@ The text to add is passed in TEXT-NO"
    lyskom-reading-list t))
 
 (def-kom-command kom-change-server-faq ()
-  "Change a FAQ for the server."
+  "Change a FAQ for the server. If the server doesn't have a FAQ, 
+create a new one. You need administrative rights to do this."
   (interactive)
   (lyskom-change-faq nil (lyskom-get-aux-item (server-info->aux-item-list
                                                (blocking-do 'get-server-info))
                                               14)))
 
 (def-kom-command kom-change-conf-faq ()
-  "Change a FAQ fo a conference."
+  "Change a FAQ fo a conference. If the conference doesn't have a FAQ,
+create a new FAQ."
   (interactive)
-  (let* ((conf-no (lyskom-read-conf-no
-                   (lyskom-get-string 'what-to-change-faq-you)
-                   '(conf pers) 
-                   nil
-                   (cons (if lyskom-current-conf
-                             (let ((tmp (blocking-do 'get-uconf-stat lyskom-current-conf)))
-                               (if tmp (uconf-stat->name tmp) ""))
-                           "") 0)
-                   t))
+  (let* ((conf-no (lyskom-read-conf-no 'what-to-change-faq-you
+                                       '(conf pers) nil nil t))
          (conf-stat (when conf-no       ; Need this to make sure the conf-stat is up-to-date!
                       (cache-del-conf-stat conf-no)
                       (blocking-do 'get-conf-stat conf-no))))
@@ -251,51 +292,57 @@ The text to add is passed in TEXT-NO"
 
 (defun  lyskom-change-faq (conf-stat aux-list)
   "Change a FAQ for a conference."
-  (if (null aux-list)
-      (lyskom-format-insert 'conf-has-no-faq conf-stat)
+  ;; Get a list of FAQ texts and corresponding aux-item-numbers
+  ;; Get the FAQ to change
+  (let* ((faq-list
+          (delq nil 
+                (mapcar (lambda (x)
+                          (and (lyskom-string-to-int (aux-item->data x))
+                               (cons (aux-item->data x)
+                                     (aux-item->aux-no x))))
+                        aux-list)))
+         (text-no-aux (cond ((= (length faq-list) 1) (car faq-list))
+                            ((null faq-list) nil)
+                            (t (lyskom-string-assoc
+                                (lyskom-completing-read 
+                                 (lyskom-get-string 'text-to-change-as-faq)
+                                 (lyskom-maybe-frob-completion-table 
+                                  faq-list)
+                                 nil t)
+                                faq-list))))
+         (text-no (and text-no-aux (string-to-int (car text-no-aux))))
+         (aux-no (and text-no-aux (cdr text-no-aux))))
 
-    ;; Get a list of FAQ texts and corresponding aux-item-numbers
-    ;; Get the FAQ to change
-    (let* ((faq-list (mapcar (lambda (x)
-                             (cons (aux-item->data x)
-                                   (aux-item->aux-no x)))
-                           aux-list))
-           (text-no-aux (cond ((= (length faq-list) 1) (car faq-list))
-                              (t (lyskom-string-assoc
-                                  (lyskom-completing-read 
-                                   (lyskom-get-string 'text-to-change-as-faq)
-                                   (lyskom-maybe-frob-completion-table 
-                                    faq-list)
-                                   nil t)
-                                  faq-list))))
-           (text-no (string-to-int (car text-no-aux)))
-           (aux-no (cdr text-no-aux)))
+    (cond 
 
-      (cond 
+     ;; If conf-stat is null we are changing the FAQ for the server.
+     ;; Don't do this unless we are running enabled and have the right
+     ;; privileges.
+     ((and (null conf-stat)
+           (or (not lyskom-is-administrator)
+               (not (privs->admin (pers-stat->privileges
+                                   (blocking-do 'get-pers-stat
+                                                lyskom-pers-no))))))
+      (lyskom-format-insert 'not-supervisor-for-server))
 
-       ;; If conf-stat is null we are changing the FAQ for the server.
-       ;; Don't do this unless we are running enabled and have the right
-       ;; privileges.
-       ((and (null conf-stat)
-             (or (not lyskom-is-administrator)
-                 (not (privs->admin (pers-stat->privileges
-                                     (blocking-do 'get-pers-stat
-                                                  lyskom-pers-no))))))
-        (lyskom-format-insert 'not-supervisor-for-server))
+     ;; If we have a conf-stat and are not administrator and not
+     ;; supervisor for the conf, then we are not allowed to change
+     ;; the FAQ.
+     ((and conf-stat 
+           (not lyskom-is-administrator)
+           (not (lyskom-is-supervisor (conf-stat->conf-no conf-stat)
+                                      lyskom-pers-no)))
+      (lyskom-format-insert 'not-supervisor-for conf-stat))
 
-       ;; If we have a conf-stat and are not administrator and not
-       ;; supervisor for the conf, then we are not allowed to change
-       ;; the FAQ.
-       ((and conf-stat 
-             (not lyskom-is-administrator)
-             (not (lyskom-is-supervisor (conf-stat->conf-no conf-stat)
-                                        lyskom-pers-no)))
-        (lyskom-format-insert 'not-supervisor-for conf-stat))
+     ;; OK, it looks like we are allowed to change the FAQ.
+     (t (let ((text-stat nil)
+              (text-mass nil))
+
+          ;; Get the old text (if there is one)
+          (blocking-do-multiple ((ts (get-text-stat text-no))
+                                 (tm (get-text text-no)))
+            (setq text-stat ts text-mass tm))
 
-       ;; OK, it looks like we are allowed to change the FAQ.
-       (t
-        (blocking-do-multiple ((text-stat (get-text-stat text-no))
-                               (text-mass (get-text text-no)))
           (let* ((str (and text-mass (text->decoded-text-mass text-mass
                                                               text-stat)))
                  (subject (if (and str (string-match "\n" str))
@@ -317,8 +364,8 @@ The text to add is passed in TEXT-NO"
              body
              'lyskom-change-faq-2
              conf-stat
-             (text-stat->text-no text-stat)
-             aux-no))))))))
+             (and text-stat (text-stat->text-no text-stat))
+             aux-no)))))))
 
 
 (defun lyskom-change-faq-2 (text-no conf-stat old-text-no old-aux-no)
@@ -348,13 +395,18 @@ The text to add is passed in TEXT-NO"
                                    text-no))))
 
 (defun lyskom-change-faq-3 (retval conf-stat old-text-no text-no)
+  (cache-del-text-stat text-no)
   (if retval
-      (lyskom-format-insert-before-prompt 'changed-faq-for-conf-done conf-stat
-                                          old-text-no
-                                          text-no)
-    (lyskom-format-insert-before-prompt 'changed-faq-for-conf-failed conf-stat 
-                                        old-text-no text-no
-                                        (lyskom-current-error))))
+      (lyskom-format-insert-before-prompt 
+       (if old-text-no 'changed-faq-for-conf-done 'set-faq-for-conf-done)
+       conf-stat
+       old-text-no
+       text-no)
+    (lyskom-format-insert-before-prompt 
+     (if old-text-no 'changed-faq-for-conf-failed 'set-faq-for-conf-failed)
+     conf-stat 
+     old-text-no text-no
+     (lyskom-current-error))))
 
 
 
@@ -369,16 +421,18 @@ The text to add is passed in TEXT-NO"
 
 (defun lyskom-get-unread-faqs (conf-stat)
   "Return a list of unread FAQs for conf CONF-STAT."
-  (let* ((conf-no (and conf-stat (conf-stat->conf-no conf-stat)))
+  (let* ((conf-no (if conf-stat (conf-stat->conf-no conf-stat) 0))
          (aux-list (if conf-stat
                        (conf-stat->aux-items conf-stat)
                      (server-info->aux-item-list lyskom-server-info)))
          (faq-list
-          (filter-list (lambda (faq)
-                         (not (lyskom-faq-is-read conf-no faq)))
-                       (mapcar (lambda (aux)
-                                 (string-to-int (aux-item->data aux)))
-                               (lyskom-get-aux-item aux-list 14)))))
+          (filter-list 
+           (lambda (faq)
+             (not (lyskom-faq-is-read conf-no faq)))
+           (delq nil
+                 (mapcar (lambda (aux)
+                           (lyskom-string-to-int (aux-item->data aux)))
+                         (lyskom-get-aux-item aux-list 14))))))
 
     ;; Filter out FAQs that don't exist
     (let ((collector (make-collector)))
@@ -403,8 +457,37 @@ The text to add is passed in TEXT-NO"
     (when kom-auto-list-faqs
       (lyskom-format-insert 'unread-faqs-header (length faq-list) 
                             (and conf-stat (conf-stat->conf-no conf-stat)))
-      (lyskom-do-list-summary faq-list))
-
+      (lyskom-list-text-summary faq-list nil))
     (when kom-auto-review-faqs
-      (lyskom-do-review-faq faq-list))))
+      (lyskom-do-review-faq faq-list t))))
 
+
+(def-kom-command kom-list-faqs (conf-stat)
+  "List all FAQs for a conference."
+  (interactive (list (lyskom-read-conf-stat 'conf-to-list-faqs
+                                            '(conf) nil nil t)))
+  (lyskom-do-list-faqs conf-stat
+                       (lyskom-get-aux-item 
+                        (conf-stat->aux-items conf-stat)
+                        14)))
+
+(def-kom-command kom-list-server-faqs ()
+  "List all FAQs for the server."
+  (interactive)
+  (lyskom-do-list-faqs
+   nil
+   (lyskom-get-aux-item (server-info->aux-item-list
+                         (blocking-do 'get-server-info)) 14)))
+
+
+(defun lyskom-do-list-faqs (conf-stat faq-list)
+  (setq faq-list (delq nil 
+                       (mapcar (lambda (aux)
+                                 (lyskom-string-to-int (aux-item->data aux)))
+                               faq-list)))
+  (cond (faq-list
+         (lyskom-format-insert 'all-faqs-header 
+                               (and conf-stat
+                                    (conf-stat->conf-no conf-stat)))
+         (lyskom-list-text-summary faq-list nil))
+        (t (lyskom-format-insert 'conf-has-no-faq conf-stat))))
