@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;;; $Id: filter.el,v 44.21 2002/04/10 19:23:23 byers Exp $
+;;;;; $Id: filter.el,v 44.29 2003/08/17 13:21:33 byers Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -33,7 +33,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: filter.el,v 44.21 2002/04/10 19:23:23 byers Exp $\n"))
+	      "$Id: filter.el,v 44.29 2003/08/17 13:21:33 byers Exp $\n"))
 
 (eval-when-compile
   (require 'lyskom-command "command"))
@@ -178,8 +178,7 @@ invalid-value until a filter action has been selected.")
       (lyskom-use 'filter 'lyskom-filter-text-p-3 text-stat))))
     
 (defun lyskom-filter-text-p-3 (author text &rest data)
-  (if (or (null text)
-	  (null author))
+  (if (null text)
       (setq lyskom-filter-hack nil)
     (let (subject text-stat)
       
@@ -302,17 +301,17 @@ invalid-value until a filter action has been selected.")
               (cond 
                ((eq key 'author)
                 (lyskom-filter-check-args 'stringp args)
-                (` (string-match (, (regexp-quote args))
-                                 (conf-stat->name author))))
+                (` (and author (string-match (, (regexp-quote args))
+                                             (conf-stat->name author)))))
 
                ((eq key 'author-re)
                 (lyskom-filter-check-args 'regexpp args)
-                (` (string-match (, args)
-                                 (conf-stat->name author))))
+                (` (and author (string-match (, args)
+                                             (conf-stat->name author)))))
 
                ((eq key 'author-no)
                 (lyskom-filter-check-args 'integerp args)
-                (` (= (, args) (conf-stat->conf-no author))))
+                (` (and author (= (, args) (conf-stat->conf-no author)))))
 
                ((eq key 'recipient)
                 (lyskom-filter-check-args 'stringp args)
@@ -449,35 +448,30 @@ Otherwise return nil."
 ;;; 
 
 (def-kom-command kom-filter-subject (&optional subject)
-  "Interactively filter a subject. Optional SUBJECT is subject to filter."
+  "Interactively filter a subject. This creates a permanent or temporary
+filter for a single subject in one conference or all conferences.
+
+An alternative to this is `kom-super-jump'.
+
+To change existing filters, use `kom-filter-edit'."
   (interactive)
-  (if (/= 0 lyskom-current-conf)
-	  (let ((conf-stat (blocking-do 'get-conf-stat lyskom-current-conf)))
-	    (let (conf perm filter action)
-	      (if (null subject)
-              (setq subject lyskom-current-subject))
-	      (setq subject 
-                (lyskom-read-from-minibuffer 
-		 (lyskom-get-string 'filter-subject)
-		 subject))
-	      (setq filter (cons (cons 'subject subject) filter))
-	      (setq conf (lyskom-read-conf-no
-                      (lyskom-get-string 'filter-in-conf)
-                      '(all)
-                      t
-                      (or (and (conf-stat->conf-no conf-stat)
-                               (cons (conf-stat->name conf-stat) 0))
-                          "")
-                      t))
-	      (if (/= conf 0)
-              (setq filter (cons (cons 'recipient-no conf) filter)))
-	      (setq action (lyskom-filter-read-action))
-	      (setq perm (lyskom-filter-read-permanent))
+  (when (not (eq 0 lyskom-current-conf))
+    (let* (conf perm filter action)
+      (setq subject (lyskom-read-from-minibuffer 
+                     (lyskom-get-string 'filter-subject)
+                     (or subject lyskom-current-subject)))
+      (setq filter (cons (cons 'subject subject) filter))
+      (setq conf (lyskom-read-conf-no 'filter-in-conf
+                                      '(all) t nil t))
+      (if (not (eq conf 0))
+          (setq filter (cons (cons 'recipient-no conf) filter)))
+      (setq action (lyskom-filter-read-action))
+      (setq perm (lyskom-filter-read-permanent))
 	      
-	      (lyskom-add-filter
-	       (make-filter filter
-                        (list (cons 'action action)
-                              (cons 'expire (not perm)))))))))
+      (lyskom-add-filter
+       (make-filter filter
+                    (list (cons 'action action)
+                          (cons 'expire (not perm))))))))
   
 
      
@@ -487,52 +481,35 @@ Otherwise return nil."
 
 
 (def-kom-command kom-filter-author ()
-  "Interactively filter an author."
+  "Interactively filter an author. This creates a permanent or
+temporary filter on a single author in one conference or all
+conferences.
+
+To change existing filters, use `kom-filter-edit'."
   (interactive)
-  (let (auth-stat author conf filter action permanent)
-	(blocking-do-multiple ((text-stat (get-text-stat 
-                                       (or lyskom-current-text 0)))
-                           (conf-stat (get-conf-stat
-                                       lyskom-current-conf)))
-      (if text-stat
-          (setq auth-stat (blocking-do 'get-conf-stat
-                                       (text-stat->author text-stat))))
-      (setq author 
-            (lyskom-read-conf-no (lyskom-get-string 'filter-author)
-                                 '(pers)
-                                 t
-                                 (or (and auth-stat
-                                          (cons (conf-stat->name auth-stat) 0))
-                                     "")
-                                 t))
-      (if (/= author 0)
-          (setq filter (cons (cons 'author-no author) filter)))
-      (setq conf (lyskom-read-conf-no
-                  (lyskom-get-string 'filter-in-conf)
-                  '(all)
-                  t
-                  (or 
-                   (and conf-stat
-                        (cons (conf-stat->name conf-stat) 0))
-                   "")
-                  t))
-      (if (/= conf 0)
-          (setq filter (cons (cons 'recipient-no conf) filter)))
-      (setq action (lyskom-filter-read-action))
-      (setq permanent (lyskom-filter-read-permanent))
-      (lyskom-add-filter
-       (make-filter filter
-                    (list (cons 'action action)
-                          (cons 'expire (not permanent))))))))
+  (let (author conf filter action permanent)
+    (setq author (lyskom-read-conf-no 'filter-author '(pers) t nil t))
+    (if (not (eq author 0))
+        (setq filter (cons (cons 'author-no author) filter)))
+    (setq conf (lyskom-read-conf-no 'filter-in-conf
+                                    '(all) t nil t))
+    (if (not (eq conf 0))
+        (setq filter (cons (cons 'recipient-no conf) filter)))
+    (setq action (lyskom-filter-read-action))
+    (setq permanent (lyskom-filter-read-permanent))
+    (lyskom-add-filter
+     (make-filter filter
+                  (list (cons 'action action)
+                        (cons 'expire (not permanent)))))))
 
 (def-kom-command kom-filter-recipient ()
-  "Interactively filter a recipient."
+  "Interactively filter a recipient. This creates a permanent or
+temporary filter on a particular recipient.
+
+To change existing filters, use `kom-filter-edit'."
   (interactive)
   (let ((conf-no (lyskom-read-conf-no 'filter-recipient
-                                      '(all)
-                                      nil
-                                      nil
-                                      t))
+                                      '(all) nil nil t))
         (action (lyskom-filter-read-action))
         (permanent (lyskom-filter-read-permanent)))
     (cond ((and conf-no (not (zerop conf-no)))
@@ -550,46 +527,67 @@ Otherwise return nil."
 ;;; Superhoppa
 ;;;
 
-(def-kom-command kom-super-jump ()
-  "Skip all texts and comments that share the subject and recipient of 
-the current text"
-  (interactive)
+(def-kom-command kom-super-jump (text-no)
+  "Skip all texts and comments that share the subject and recipient of
+the selected text. This creates a temporary filter on the subject of
+the selected text in one of its recipients. The recipient is selected
+automatically: if the current conference is a recipient, then filter
+in that conference. Otherwise filter in the first recipient that the
+user is a member of.
+
+To change existing filters, use `kom-filter-edit'.
+
+This command accepts text number prefix arguments \(see
+`lyskom-read-text-no-prefix-arg')."
+  (interactive (list (lyskom-read-text-no-prefix-arg 'super-jump-q)))
   (if (or (null lyskom-current-text)
           (zerop lyskom-current-text))
       (lyskom-insert-string 'have-to-read)
     (let ((text-stat (blocking-do 'get-text-stat lyskom-current-text))
-          (recipients nil)
-          (cc-recipients nil)
-          (bcc-recipients nil)
-          (filter-recipient nil)
-          (conf-stat (if (and lyskom-current-conf
-                              (not (zerop lyskom-current-conf)))
-                         (blocking-do 'get-conf-stat lyskom-current-conf))))
+          (all-recipients nil)
+          (recipients-member nil)
+          (cc-recipients-member nil)
+          (bcc-recipients-member nil)
+          (recipients-nonmember nil)
+          (cc-recipients-nonmember nil)
+          (bcc-recipients-nonmember nil)
+          (filter-recipient nil))
+
+      ;; Extract recipients into different lists
       (lyskom-traverse misc
                        (text-stat->misc-info-list text-stat)
-                       (cond ((eq (misc-info->type misc) 'RECPT) 
-                              (setq recipients (cons (misc-info->recipient-no misc)
-                                                     recipients)))
-                             ((eq (misc-info->type misc) 'CC-RECPT)
-                              (setq cc-recipients (cons (misc-info->recipient-no misc)
-                                                        cc-recipients)))
-                             ((eq (misc-info->type misc) 'BCC-RECPT)
-                              (setq bcc-recipients (cons (misc-info->recipient-no misc)
-                                                         bcc-recipients)))))
+        (when (memq (misc-info->type misc) lyskom-recpt-types-list)
+          (let* ((conf-no (misc-info->recipient-no misc))
+                 (symbol
+                  (if (lyskom-get-membership conf-no)
+                      (cond ((eq (misc-info->type misc) 'RECPT)
+                             'recipients-member)
+                            ((eq (misc-info->type misc) 'CC-RECPT)
+                             'cc-recipients-member)
+                            ((eq (misc-info->type misc) 'BCC-RECPT)
+                             'bcc-recipients-member))
+                    (cond ((eq (misc-info->type misc) 'RECPT)
+                           'recipients-nonmember)
+                          ((eq (misc-info->type misc) 'CC-RECPT)
+                           'cc-recipients-nonmember)
+                          ((eq (misc-info->type misc) 'BCC-RECPT)
+                           'bcc-recipients-nonmember)))))
+            (set symbol (cons conf-no (symbol-value symbol)))
+            (setq all-recipients (cons conf-no all-recipients)))))
 
-      (setq filter-recipient (or 
-                              (and conf-stat
-                                   (or 
-                                    (memq lyskom-current-conf recipients)
-                                    (memq lyskom-current-conf cc-recipients))
-                                   lyskom-current-conf)
-                              (car (nreverse recipients))
-                              (car (nreverse cc-recipients))
-                              (car (nreverse bcc-recipients))))
+      ;; Get the recipient to filter in
+      (setq filter-recipient 
+            (or (and (memq lyskom-current-conf all-recipients) lyskom-current-conf)
+                (car (nreverse recipients-member))
+                (car (nreverse cc-recipients-member))
+                (car (nreverse bcc-recipients-member))
+                (car (nreverse recipients-nonmember))
+                (car (nreverse cc-recipients-nonmember))
+                (car (nreverse bcc-recipients-nonmember))))
 
       (if (null filter-recipient)
           (lyskom-insert-string 'no-recipient)
-            
+
         (let ((text lyskom-current-subject))
           (when (string-match "^\\s-*$" lyskom-current-subject) (setq text ""))
 
@@ -618,36 +616,32 @@ the current text"
 ;;;
 
 (def-kom-command kom-filter-text (&optional text)
-  "Interactively filter on text contents. Optional TEXT is subject to filter."
+  "Interactively filter on text contents. This creates a permanent or
+temporary filter on the contents of texts in all conferences or in a
+single conference.
+
+To change existing filters, use `kom-filter-edit'."
   (interactive)
-  (if (/= 0 lyskom-current-conf)
-	  (let ((conf-stat (blocking-do 'get-conf-stat lyskom-current-conf))
-            (conf nil)
-            (action nil)
-            (perm nil)
-            (filter nil))
-	    (if conf-stat
-            (progn
-              (setq text 
-                    (lyskom-read-from-minibuffer (lyskom-get-string 
-						  'filter-which-text)
-						 (or text "")))
-              (setq filter (cons (cons 'text text) filter))
-              (setq conf (lyskom-read-conf-no
-                          (lyskom-get-string 'filter-in-conf)
-                          '(all) t
-                          (or (and (conf-stat->conf-no conf-stat)
-                                   (cons (conf-stat->name conf-stat) 0))
-                              "")
-                          t))
-              (if (/= conf 0)
-                  (setq filter (cons (cons 'recipient-no conf) filter)))
-              (setq action (lyskom-filter-read-action))
-              (setq perm (lyskom-filter-read-permanent))
-              (lyskom-add-filter
-               (make-filter filter
-                            (list (cons 'action action)
-                                  (cons 'expire (not perm))))))))))
+  (when (not (eq 0 lyskom-current-conf))
+    (let ((conf nil)
+          (action nil)
+          (perm nil)
+          (filter nil))
+      (setq text 
+            (lyskom-read-from-minibuffer (lyskom-get-string 
+                                          'filter-which-text)
+                                         (or text "")))
+      (setq filter (cons (cons 'text text) filter))
+      (setq conf (lyskom-read-conf-no 'filter-in-conf
+                                      '(all) t nil t))
+      (if (not (eq conf 0))
+          (setq filter (cons (cons 'recipient-no conf) filter)))
+      (setq action (lyskom-filter-read-action))
+      (setq perm (lyskom-filter-read-permanent))
+      (lyskom-add-filter
+       (make-filter filter
+                    (list (cons 'action action)
+                          (cons 'expire (not perm))))))))
 
 
 ;;;============================================================
@@ -658,7 +652,9 @@ the current text"
 ;;; may not be a good idea, but it works...
 
 (def-kom-command kom-list-filters ()
-  "Display all filters"
+  "Display all filters. This lists all filters in the LysKOM buffer.
+
+To change existing filters, use `kom-filter-edit'."
   (interactive)
   (let ((filters lyskom-filter-list))
     (goto-char (point-max))
