@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;; $Id: lyskom-buttons.el,v 44.74 2002/09/14 21:28:06 byers Exp $
+;;;; $Id: lyskom-buttons.el,v 44.99 2005/01/11 15:00:13 _cvs_pont_lyskomelisp Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -34,7 +34,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: lyskom-buttons.el,v 44.74 2002/09/14 21:28:06 byers Exp $\n"))
+	      "$Id: lyskom-buttons.el,v 44.99 2005/01/11 15:00:13 _cvs_pont_lyskomelisp Exp $\n"))
 
 (lyskom-external-function glyph-property)
 (lyskom-external-function widget-at)
@@ -43,11 +43,9 @@
 (lyskom-external-function w3-popup-menu)
 (lyskom-external-function Info-goto-node)
 (lyskom-external-function term-char-mode)
+(lyskom-external-function w3m)
 
 
-(defun lyskom-menu-selection nil
-  "Variable used to work around the handling of menus in XEmacs.")
-
 (defun lyskom-add-button-action (type text func)
   "Add a new action to the popup menu for a class of objects.
 Arguments are TYPE, the type of object to adjust, TEXT the menu text
@@ -68,16 +66,6 @@ documentation for the variable lyskom-text-buttons."
   (nconc (nth 3 (assq type lyskom-button-actions))
 	 (list (cons text func))))
 
-(defun lyskom-set-default-button-action (type func)
-  "Set the default action for buttons of the type TYPE to FUNC. 
-
-FUNC must be a valid button action function. For more information
-on such functions see the documentation for lyskom-add-button-action."
-  (let ((el (assq type lyskom-button-actions)))
-    (setcdr (nthcdr 1 el)
-	    (cons func (nthcdr 3 el)))))
-      
-
 
 (defun kom-previous-link (num)
   "Move the cursor to the previous active area in the LysKOM buffer."
@@ -127,8 +115,8 @@ on such functions see the documentation for lyskom-add-button-action."
 If optional argument do-default is non-nil, call the default binding of
 this-command-keys."
   (interactive "@e")
-  (let* ((pos (event-point event))
-         (glyph (event-glyph event))
+  (let* ((pos (lyskom-event-point event))
+         (glyph (lyskom-event-glyph event))
          (widget (and pos 
                       (or (and glyph (glyph-property glyph 'widget))
                           (widget-at pos))))
@@ -136,15 +124,24 @@ this-command-keys."
          (href (or (and widget (widget-get widget ':href))
                    (and parent (widget-get parent ':href))
                    (and widget (widget-get widget 'href))
-                   (and parent (widget-get parent 'href)))))
+                   (and parent (widget-get parent 'href))))
+         (w3m-url (and pos (get-text-property pos 'w3m-href-anchor)))
+         (type  (and pos (get-text-property pos 'lyskom-button-type)))
+         (data  (and type (assq type lyskom-button-actions)))
+         (hint (and pos (get-text-property pos 'lyskom-button-hint)))
+         (act  (or (and kom-use-button-hints hint)
+                   (and data (elt data 2)))))
     (cond (href (require 'w3)
                 (w3-widget-button-click event))
+          (w3m-url (require 'w3m)
+               (w3m w3m-url))
           ((and do-default
                 (or (null pos)
                     (null (get-text-property pos 'lyskom-button-type))))
            (let ((fn (lookup-key global-map (this-command-keys))))
              (when (commandp fn)
                (call-interactively fn))))
+          ((null act) (kom-popup-menu event))
           (t (lyskom-button-press pos)))))
 
 (defun kom-button-click-or-yank (event)
@@ -156,8 +153,8 @@ If there is no active area, then do something else."
 (defun kom-popup-menu (event)
   "Pop up a menu of actions to be taken at the active area under the mouse."
   (interactive "@e")
-  (let* ((pos (event-point event))
-         (glyph (event-glyph event))
+  (let* ((pos (lyskom-event-point event))
+         (glyph (lyskom-event-glyph event))
          (widget (and pos 
                       (or (and glyph (glyph-property glyph 'widget))
                           (widget-at pos))))
@@ -315,65 +312,31 @@ If there is no active area, then do something else."
       url)))
 
 
-(defun lyskom-button-transform-text (text)
-  "Add text properties to the string TEXT according to the definition of
-lyskom-text-buttons. Returns the modified string."
-  (let ((blist lyskom-text-buttons)
-        (start 0)
-        (el nil))
-    (while blist
-      (setq el (car blist))
-      (setq start 0)
-      (while (string-match (elt el 0) text start)
-        (add-text-properties 
-         (match-beginning (or (elt el 2) 0))
-         (match-end (or (elt el 2) 0))
-         (cond ((and (eq (elt el 1) 'text)
-                     (not lyskom-transforming-external-text))
-                (lyskom-generate-button 'text
-                                        (lyskom-button-get-arg el text)
-                                        (lyskom-button-get-text el text)
-                                        (lyskom-button-get-face el)))
-               ((eq (elt el 1) 'conf)
-                (lyskom-generate-button 'conf
-                                        (lyskom-button-get-arg el text)
-                                        (lyskom-button-get-text el text)
-                                        (lyskom-button-get-face el)))
-               ((eq (elt el 1) 'pers)
-                (lyskom-generate-button 'pers
-                                        (lyskom-button-get-arg el text)
-                                        (lyskom-button-get-text el text)
-                                        (lyskom-button-get-face el)))
-               ((eq (elt el 1) 'url)
-                (lyskom-generate-button 'url
-                                        nil
-                                        (lyskom-button-get-text el text)
-                                        (lyskom-button-get-face el)))
 
-               ((eq (elt el 1) 'pseudo-url)
-                (let ((url (lyskom-fix-pseudo-url
-                            (lyskom-button-get-text el text))))
-                  (lyskom-generate-button 'url
-                                          nil
-                                          url
-                                          (lyskom-button-get-face el))))
+(defun lyskom-button-add-links (text links)
+  "Add links to TEXT according to LINKS.
+LINKS is a list of lyskom-text-link objects."
+  (lyskom-traverse spec links
+    (let ((case-fold-search (lyskom-text-link->ignore-case spec)) 
+          (start 0))
+      (while (string-match (lyskom-text-link->pattern spec) text start)
+         ;; Need to do this here since replace-in-string fscks up match-data
+        (setq start (match-end 0))
+        (add-text-properties
+         (or (match-beginning (lyskom-text-link->highlight spec))
+             (match-beginning 0))
+         (or (match-end (lyskom-text-link->highlight spec))
+             (match-end 0))
+         (lyskom-generate-button 'url
+                                 nil
+                                 (replace-in-string 
+                                  (match-string 0 text)
+                                  (lyskom-text-link->pattern spec)
+                                  (lyskom-text-link->replacement spec))
+                                 kom-url-face)
+         text)))))
+
 
- 	       ((eq (elt el 1) 'info-node)
- 		(lyskom-generate-button 'info-node
- 					(lyskom-button-get-arg el text)
- 					(lyskom-button-get-text el text)
- 					(lyskom-button-get-face el)))
-               
-               ((eq (elt el 1) 'email)
-                (lyskom-generate-button 'email
-                                        nil
-                                        (lyskom-button-get-text el text)
-                                        (lyskom-button-get-face el)))
-               (t nil))
-         text)
-        (setq start (match-end 0)))
-      (setq blist (cdr blist))))
-  text)
 
 (defun lyskom-button-get-arg (el text)
   "Get the button argument for button type EL from TEXT according to
@@ -390,7 +353,91 @@ the current match-data."
 (defun lyskom-button-get-face (el)
   "Get the button face for button type EL from TEXT according to
 the current match-data."
-  (elt el 4))
+  (let ((face (elt el 4)))
+    (or (and (boundp face) (symbol-value face))
+        (and (facep face) face))))
+
+(defsubst lyskom-button-get-pred (el)
+  "Return the match predicate for bitton type EL."
+  (elt el 5))
+
+(defun lyskom-button-transform-text (text &optional text-stat)
+  "Add text properties to the string TEXT according to the definition of
+lyskom-text-buttons. Returns the modified string."
+  (let ((blist lyskom-text-buttons)
+        (rcpts (reverse (and text-stat (lyskom-text-recipients text-stat))))
+        (start 0)
+        (el nil))
+
+    ;; Do other text buttons
+    (while blist
+      (setq el (car blist))
+      (setq start 0)
+      (while (string-match (elt el 0) text start)
+        (if (or (null (lyskom-button-get-pred el))
+                  (funcall (lyskom-button-get-pred el)
+                           (lyskom-button-get-text el text)))
+          (progn
+            (add-text-properties 
+             (match-beginning (or (elt el 2) 0))
+             (match-end (or (elt el 2) 0))
+             (cond ((and (eq (elt el 1) 'text)
+                         (not lyskom-transforming-external-text))
+                    (lyskom-generate-button 'text
+                                            (lyskom-button-get-arg el text)
+                                            (lyskom-button-get-text el text)
+                                            (lyskom-button-get-face el)))
+                   ((eq (elt el 1) 'conf)
+                    (lyskom-generate-button 'conf
+                                            (lyskom-button-get-arg el text)
+                                            (lyskom-button-get-text el text)
+                                            (lyskom-button-get-face el)))
+                   ((eq (elt el 1) 'pers)
+                    (lyskom-generate-button 'pers
+                                            (lyskom-button-get-arg el text)
+                                            (lyskom-button-get-text el text)
+                                            (lyskom-button-get-face el)))
+                   ((eq (elt el 1) 'url)
+                    (lyskom-generate-button 'url
+                                            nil
+                                            (lyskom-button-get-text el text)
+                                            (lyskom-button-get-face el)))
+
+                   ((eq (elt el 1) 'pseudo-url)
+                    (let ((url (lyskom-fix-pseudo-url
+                                (lyskom-button-get-text el text))))
+                      (lyskom-generate-button 'url
+                                              nil
+                                              url
+                                              (lyskom-button-get-face el))))
+
+                   ((eq (elt el 1) 'info-node)
+                    (lyskom-generate-button 'info-node
+                                            (lyskom-button-get-arg el text)
+                                            (lyskom-button-get-text el text)
+                                            (lyskom-button-get-face el)))
+
+
+                   ((eq (elt el 1) 'email)
+                    (lyskom-generate-button 'email
+                                            nil
+                                            (lyskom-button-get-text el text)
+                                            (lyskom-button-get-face el)))
+                   (t nil))
+             text)
+            (setq start (match-end 0)))
+          (setq start (1+ (match-beginning 0)))))
+      (setq blist (cdr blist)))
+
+    ;; Do links from kom-text-links last -- they have precedence
+    ;; Do the first recipient last so links for that rcpt have prio
+    (lyskom-traverse rcpt rcpts
+      (lyskom-button-add-links text (cdr (assq rcpt kom-text-links))))
+
+    ;; Do global links from kom-text-links very last
+    (lyskom-button-add-links text (cdr (assq t kom-text-links)))
+    )
+  text)
 
 
 (defun lyskom-get-button-hint (hints)
@@ -429,53 +476,81 @@ argument MENU-TITLE defines the title for the popup menu. See
 lyskom-default-button for more information. Optional argument SUBTLE
 means don't set the lyskom-button property if non-nil. that means
 kom-next- and -previous-link won't notice the button"
-  (car menu-title)		; produce error if menu-title not cons
-  (let* ((persno (cond ((boundp 'lyskom-pers-no) lyskom-pers-no)
-                       ((and (boundp 'lyskom-buffer) lyskom-buffer)
-                        (save-excursion
-                          (set-buffer lyskom-buffer)
-                          lyskom-pers-no))
-                       (t -1)))
-         (numarg (cond ((numberp arg) arg)
+  (setq lyskom-dummy-variable-to-fool-the-byte-compiler
+        (car menu-title))		; produce error if menu-title not cons
+  (let* ((numarg (cond ((numberp arg) arg)
                        ((stringp arg) (string-to-number arg))
+                       ((lyskom-conf-stat-p arg) (conf-stat->conf-no arg))
+                       ((lyskom-uconf-stat-p arg) (uconf-stat->conf-no arg))
+                       ((lyskom-pers-stat-p arg) (pers-stat->pers-no arg))
                        (t nil)))
          (data (assq type lyskom-button-actions))
          (hints (and data (elt data 4)))
          (the-hint (lyskom-get-button-hint hints))
          (props 
-          (cond ((and (memq type '(conf pers))
-                      numarg)
+          (cond ((and (memq type '(conf pers)) numarg)
                  (list 'face 
                        (or face
-                           (cond ((eq persno numarg) 'kom-me-face)
-                                 ((memq numarg kom-friends) 'kom-friends-face)
-                                 ((memq numarg kom-morons) 'kom-morons-face)
-                                 (t 'kom-active-face)))
-                       'mouse-face 'kom-highlight-face
+                           (or 
+                            (lyskom-traverse el kom-highlight-conferences
+                              (cond ((and (symbolp (car el))
+                                          (boundp (car el))
+                                          (listp (symbol-value (car el)))
+                                          (memq numarg (symbol-value (car el))))
+                                     (lyskom-traverse-break
+                                      (cond ((facep (cdr el)) (cdr el))
+                                            ((and (symbolp (cdr el))
+                                                  (boundp (cdr el)))
+                                             (symbol-value (cdr el))))))
+                                    ((and (symbolp (car el))
+                                          (boundp (car el))
+                                          (eq numarg (symbol-value (car el))))
+                                     (lyskom-traverse-break
+                                      (cond ((facep (cdr el)) (cdr el))
+                                            ((and (symbolp (cdr el))
+                                                  (boundp (cdr el)))
+                                             (symbol-value (cdr el))))))
+                                    ((and (functionp (car el))
+                                          (funcall (car el) arg))
+                                     (lyskom-traverse-break
+                                      (cond ((facep (cdr el)) (cdr el))
+                                            ((and (symbolp (cdr el))
+                                                  (boundp (cdr el)))
+                                             (symbol-value (cdr el))))))
+                                    ((and (listp (car el))
+                                          (memq numarg (car el)))
+                                     (lyskom-traverse-break
+                                      (cond ((facep (cdr el)) (cdr el))
+                                            ((and (symbolp (cdr el))
+                                                  (boundp (cdr el)))
+                                             (symbol-value (cdr el)))))))
+                              nil)
+                            kom-active-face))
+                       'mouse-face kom-highlight-face
                        'lyskom-button-text text
                        'lyskom-button-type type
                        'lyskom-button-arg numarg
                        'lyskom-button-menu-title menu-title
                        'lyskom-buffer lyskom-buffer))
                 ((and (eq type 'text) numarg)
-                 (list 'face (or face 'kom-text-no-face)
-                       'mouse-face 'kom-highlight-face
+                 (list 'face (or face kom-text-no-face)
+                       'mouse-face kom-highlight-face
                        'lyskom-button-text text
                        'lyskom-button-type type
                        'lyskom-button-arg numarg
                        'lyskom-button-menu-title menu-title
                        'lyskom-buffer lyskom-buffer))
                 ((eq type 'url)
-                 (list 'face (or face 'kom-active-face)
-                       'mouse-face 'kom-highlight-face
+                 (list 'face (or face kom-active-face)
+                       'mouse-face kom-highlight-face
                        'lyskom-button-text text
                        'lyskom-button-type type
                        'lyskom-button-arg arg
                        'lyskom-button-menu-title menu-title
                        'lyskom-buffer lyskom-buffer))
                 (t
-		 (list 'face (or face 'kom-active-face)
-		       'mouse-face 'kom-highlight-face
+		 (list 'face (or face kom-active-face)
+		       'mouse-face kom-highlight-face
 		       'lyskom-button-text text
 		       'lyskom-button-type type
 		       'lyskom-button-arg arg
@@ -488,9 +563,9 @@ kom-next- and -previous-link won't notice the button"
 		(cons 'lyskom-button-hint
 		      (cons the-hint props))
 	      props))))
-      
-        
-           
+
+
+
 
 (defun lyskom-default-button (type arg &optional menu-title)
   "Generate a button of type TYPE from data in ARG. ARG can be almost any
@@ -505,45 +580,57 @@ up."
 		(cond ((lyskom-conf-stat-p arg)
 		       (if (conf-type->letterbox (conf-stat->conf-type arg))
 			   (setq type 'pers))
-		       (setq xarg (conf-stat->conf-no arg)
+		       (setq xarg arg
 			     text (conf-stat->name arg)))
                       ((lyskom-uconf-stat-p arg)
 		       (if (conf-type->letterbox (uconf-stat->conf-type arg))
 			   (setq type 'pers))
 		       (setq xarg (uconf-stat->conf-no arg)
 			     text (uconf-stat->name arg)))
+                      ((lyskom-conf-z-info-p arg)
+                       (if (conf-type->letterbox (conf-z-info->conf-type arg))
+                           (setq type 'pers))
+                       (setq xarg (conf-z-info->conf-no arg)
+                             text (conf-z-info->name arg)))
 		      ((numberp arg)
-                       (if (setq xarg (cache-get-conf-stat arg))
+                       (if (setq xarg (cache-get-uconf-stat arg))
                            (progn
                              (if (conf-type->letterbox
-                                  (conf-stat->conf-type xarg))
+                                  (uconf-stat->conf-type xarg))
                                  (setq type 'pers))
-                             (setq text (conf-stat->name xarg))
-                             (setq xarg (conf-stat->conf-no xarg)))
-                         (setq text ""
-                               xarg arg)))
+                             (setq text (uconf-stat->name xarg))
+                             (setq xarg arg))
+                         (setq text "" xarg arg)))
 		      (t (setq text "" xarg 0))))
+
 	       ((eq type 'pers)
 		(cond ((lyskom-conf-stat-p arg)
-		       (setq xarg (conf-stat->conf-no arg)
+		       (setq xarg arg
 			     text (conf-stat->name arg)))
                       ((lyskom-uconf-stat-p arg)
-		       (setq xarg (uconf-stat->conf-no arg)
+		       (setq xarg arg
 			     text (uconf-stat->name arg)))
+                      ((lyskom-conf-z-info-p arg)
+                       (setq xarg arg
+                             text (conf-z-info->name arg)))
 		      ((lyskom-pers-stat-p arg)
 		       (setq xarg (pers-stat->pers-no arg)
-			     text 
-                             (or (conf-stat->name
-                                  (cache-get-conf-stat
-                                   (pers-stat->pers-no arg)))
-                                 "")))
+			     text (or (conf-stat->name
+                                       (cache-get-conf-stat
+                                        (pers-stat->pers-no arg)))
+                                      "")))
 		      ((numberp arg) 
-                       (setq text
-                             (or (conf-stat->name
-                                  (cache-get-conf-stat arg))
-                                 "")
-                             xarg arg))
+                       (if (setq xarg (cache-get-uconf-stat arg))
+                           (progn
+                             (if (conf-type->letterbox
+                                  (uconf-stat->conf-type xarg))
+                                 (setq type 'pers))
+                             (setq text (uconf-stat->name xarg))
+                             (setq xarg arg))
+                         (setq text "" xarg arg)))
+
 		      (t (setq text "" xarg 0))))
+
 	       ((eq type 'text)
 		(cond ((stringp arg) (setq xarg (string-to-number arg)
 					   text arg))
@@ -553,25 +640,26 @@ up."
 		       (setq xarg (text-stat->text-no arg)
 			     text (number-to-string (text-stat->text-no arg))))
 		      (t (setq xarg 0 text ""))))
+
 	       ((eq type 'url)
-                (setq face 'kom-url-face)
+                (setq face kom-url-face)
 		(cond ((stringp arg) (setq xarg nil text arg))
 		      (t (setq xarg nil text ""))))
 
                ((eq type 'email)
-                (setq face 'kom-url-face)
+                (setq face kom-url-face)
                 (cond ((stringp arg) (setq xarg nil text arg))
                       (t (setq xarg nil text ""))))
 
 	       ((eq type 'timestamp)
-		(setq face 'kom-text-face
+		(setq face 'default
 		      subtle t)
-		(cond ((null arg) (setq xarg (current-time)
-					;text (format-time-string "%Y-%m-%d %H:%M")
-					))
+                (cond ((null arg) (setq xarg (lyskom-current-client-time)
+                                        ;text (format-time-string "%Y-%m-%d %H:%M")
+                                        ))
 		      (t (setq xarg arg
-			       ;text (format-time-string "%Y-%m-%d %H:%M" arg)
-			       ))))
+                               ;text (format-time-string "%Y-%m-%d %H:%M" arg)
+                               ))))
 	       (t (setq xarg arg
 			text "")))
 	 (lyskom-generate-button type xarg text face menu-title subtle))))
@@ -593,6 +681,13 @@ This is a LysKOM button action."
            (goto-char (point-max))
            (kom-view arg))))
 
+(defun lyskom-button-unread-text (buf arg text)
+  "In the LysKOM buffer BUF, unread the text ARG. 
+Last argument TEXT is ignored. This is a LysKOM button action."
+  (cond ((not (integerp arg)) nil)
+        (t (pop-to-buffer buf)
+           (kom-mark-unread arg))))
+
 (defun lyskom-button-copy-text-no (but arg text)
   "In the LysKOM buffer BUF, ignore ARG and copy TEXT to the kill ring.
 This is a LysKOM button action."
@@ -607,6 +702,22 @@ Last argument TEXT is ignored. This is a LysKOM button action."
            (goto-char (point-max))
            (kom-review-noconversion arg))))
 
+(defun lyskom-button-review-converted (buf arg text)
+  "In the LysKOM buffer BUF, view the text ARG converted. 
+Last argument TEXT is ignored. This is a LysKOM button action."
+  (cond ((not (integerp arg)) nil)
+        (t (pop-to-buffer buf)
+           (goto-char (point-max))
+           (kom-review-converted arg))))
+
+(defun lyskom-button-review-rot13 (buf arg text)
+  "In the LysKOM buffer BUF, view the text ARG rot13:ed. 
+Last argument TEXT is ignored. This is a LysKOM button action."
+  (cond ((not (integerp arg)) nil)
+        (t (pop-to-buffer buf)
+           (goto-char (point-max))
+           (kom-review-rot13 arg))))
+
 (defun lyskom-button-find-root-review (buf arg text)
   "In the LysKOM buffer BUF, view the text ARG. Last argument TEXT is ignored.
 This is a LysKOM button action."
@@ -792,18 +903,27 @@ This is a LysKOM button action."
       (compose-mail text)
     (mail nil text)))
 
+
 (defun lyskom-button-copy-url (but arg text)
   "In the LysKOM buffer BUF, ignore ARG and copy TEXT to the kill ring.
 This is a LysKOM button action."
   (kill-new (replace-in-string text "\\s-+" "")))
 
 
+(defun lyskom-transform-url (url)
+  (let ((rules kom-url-transformation-rules))
+    (while rules
+      (setq url (replace-in-string url (car (car rules)) (cdr (car rules)))
+            rules (cdr rules))))
+  url)
+
 (defun lyskom-button-open-url (buf arg text)
   "In the LysKOM buffer BUF, ignore ARG and open TEXT as an URL.
 This is a LysKOM button action."
   (let* ((url (lyskom-fix-pseudo-url (replace-in-string text "\\s-+" "")))
          protocol
          url-manager)
+    (setq url (lyskom-transform-url url))
     (string-match lyskom-url-protocol-regexp url)
     (setq protocol (match-string 1 url))
     (setq url-manager (lyskom-get-url-manager protocol))
@@ -832,10 +952,11 @@ This is a LysKOM button action."
 ;;;	Timestamp button
 ;;;
 
-(defun lyskom-button-copy-timestamp (but arg text)
+(defun lyskom-button-copy-timestamp (buf arg text)
   "In the LysKOM buffer BUF, ignore TEXT and copy ARG to the kill ring
 after formating it as time. This is a LysKOM button action."
-  (kill-new (format-time-string "%Y-%m-%d %H:%M" arg)))
+  (let ((kom-print-relative-dates nil))
+    (kill-new (lyskom-format-time 'date-and-time arg))))
 
 
 ;;;
@@ -873,9 +994,10 @@ after formating it as time. This is a LysKOM button action."
   (require 'browse-url)
   (funcall browse-url-browser-function url))
 
-(defun lyskom-view-url-w3 (url manager)
-  "View the URL URL using W3. Second argument MANAGER is ignored."
-  (w3-fetch url))
+(lyskom-with-external-functions (w3-fetch)
+  (defun lyskom-view-url-w3 (url manager)
+    "View the URL using W3. Second argument MANAGER is ignored."
+    (w3-fetch url)))
 
 (defun lyskom-view-url-dired (url manager)
   "View the URL URL using dired. Second argument MANAGER is ignored."
@@ -950,7 +1072,12 @@ after formating it as time. This is a LysKOM button action."
 
 (defun lyskom-view-url-windows (url manager)
   "View the URL URL in Microsoft Windows. MANGER is the URL manager.
-Fall back on Netscape if not running in Microsoft Windows."
+Fall back on Netscape if not running in Microsoft Windows.
+
+In theory, opening an URL on Windows should be a matter of doing
+\(w32-shell-execute \"open\" URL\), but this sometimes fails, so
+we protect ourselves by also trying to start some common defaults
+using start-process."
   (cond
    ((memq window-system '(win32 mswindows w32))
     (cond
@@ -975,18 +1102,22 @@ Fall back on Netscape if not running in Microsoft Windows."
                               " kom-windows-browser-command"
                               " (%s)")
                       kom-windows-browser-command))))
-     ((not (memq 'w32-shell-execute lyskom-compatibility-definitions))
-      (w32-shell-execute "open" url)
+     ((and (not (memq 'w32-shell-execute lyskom-compatibility-definitions))
+           (condition-case nil
+               (progn (w32-shell-execute "open" url) t)
+             (error nil)))
       (lyskom-url-manager-starting manager))
-      ;;(lyskom-message "Webb via [%s \"%s\" \"%s\"] ..."
-      ;;                "w32-shell-execute" "open" url))
-     (t (let ((programs (list
-                         "start"
-                         "explorer"
-                         (concat "C:\\Program Files\\Netscape"
-                           "\\Communicator\\Program\\netscape.exe")
-                         (concat "C:\\Program Files\\Netscape"
-                                 "\\Navigator\\Program\\netscape.exe"))))
+     ;;(lyskom-message "Webb via [%s \"%s\" \"%s\"] ..."
+     ;;                "w32-shell-execute" "open" url))
+     (t (let ((programs
+               '("c:\\Program\\Internet Explorer\\iexplore.exe"
+                 "c:\\Programs\\Internet Explorer\\iexplore.exe"
+                 "c:\\Program Files\\Internet Explorer\\iexplore.exe"
+                 "start"
+                 "explorer"
+                 "C:\\Program Files\\Netscape\\Communicator\\Program\\netscape.exe"
+                 "C:\\Program Files\\Netscape\\Navigator\\Program\\netscape.exe"
+                 )))
           (while programs
             (condition-case nil
                 (progn
@@ -1012,7 +1143,13 @@ that, starts a new one."
   (let* ((url-string (if (memq window-system '(win32 mswindows w32))
                          (list url)
                        (list "-remote"
-                             (format "openUrl(%s)" url))))
+                             (format
+                              (cond ((eq kom-netscape-variant 'new-window)
+                                     "openUrl(%s, new-window)")
+                                    ((eq kom-netscape-variant 'new-tab)
+                                     "openUrl(%s, new-tab)")
+                                    (t "openUrl(%s)"))
+                              url))))
          
          (proc (apply 'start-process "netscape"
                       nil
@@ -1247,3 +1384,48 @@ depending on the value of `kom-lynx-terminal'."
            
 (defun lyskom-button-apply (buf arg text)
   (apply (car arg) (cdr arg)))
+
+
+;;; ================================================================
+;;; Functions to put in kom-highlight-conferences
+
+(defun lyskom-highlight-function-get-conf-stat (arg)
+  "Try to translate ARG to a conf-stat without making server calls.
+
+ARG may be a conf-stat, pers-stat, uconf-stat, conf-z-info, integer or string."
+  (if (lyskom-conf-stat-p arg)
+      arg
+    (let* ((conf-no (cond ((lyskom-pers-stat-p arg) (pers-stat->pers-no arg))
+                          ((lyskom-uconf-stat-p arg) (uconf-stat->conf-no arg))
+                          ((numberp arg) arg)
+                          ((lyskom-conf-z-info-p arg) (conf-z-info->conf-no arg))
+                          ((stringp arg) (string-to-int arg)))))
+      (when conf-no
+        (cond ((cache-get-conf-stat conf-no))
+              (conf-no (initiate-get-conf-stat 'background nil conf-no)
+                       nil))))))
+
+
+(defun lyskom-highlight-i-am-supervisor (arg)
+  "Returns non-nil if lyskom-pers-no is the supervisor of ARG.
+
+ARG interpreted by `lyskom-highlight-function-get-conf-stat'.
+
+This function never makes server calls, so if the information required
+to answer accurately is not cached, this function will return an incorrect
+result (nil instead of t)."
+  (let ((conf-stat (lyskom-highlight-function-get-conf-stat arg)))
+    (lyskom-i-am-supervisor conf-stat nil)))
+
+(defun lyskom-highlight-has-no-presentation (arg)
+  "Returns non-nil if lyskom-pers-no is the supervisor of CONF-STAT.
+
+ARG is interpreted by `lyskom-highlight-function-get-conf-stat'.
+
+This function never makes server calls, so if the information required
+to answer accurately is not cached, this function will return an incorrect
+result (nil instead of t)."
+  (let ((conf-stat (lyskom-highlight-function-get-conf-stat arg)))
+    (cond ((null conf-stat) nil)
+          ((conf-type->secret (conf-stat->conf-type conf-stat)) nil)
+          (t (eq 0 (conf-stat->presentation conf-stat))))))
