@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;;; $Id: option-edit.el,v 44.79 2002/10/20 13:20:00 byers Exp $
+;;;;; $Id: option-edit.el,v 44.114 2005/01/12 11:42:13 _cvs_pont_lyskomelisp Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -34,7 +34,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: option-edit.el,v 44.79 2002/10/20 13:20:00 byers Exp $\n"))
+	      "$Id: option-edit.el,v 44.114 2005/01/12 11:42:13 _cvs_pont_lyskomelisp Exp $\n"))
 
 (lyskom-external-function widget-default-format-handler)
 (lyskom-external-function popup-mode-menu)
@@ -71,14 +71,21 @@
     [kom-auto-list-faqs]
     [kom-auto-review-faqs]
     "\n"
+    [kom-auto-confirm-new-conferences]
+    "\n"
     [kom-idle-hide]
+    [kom-relogin-behaviour]
+    [kom-relogin-inhibit-commands]
+    [kom-remember-password]
     [kom-show-where-and-what]
     [kom-show-since-and-when]
     "\n"
+    [kom-highlight-conferences]
     [kom-friends]
     [kom-morons]
     "\n"
     [kom-show-unread-in-frame-title]
+    [kom-unread-mode-line-type]
     [kom-presence-messages-in-echo-area]
     [kom-presence-messages-in-buffer]
     "\n"
@@ -90,6 +97,7 @@
     [kom-trim-buffer-minimum]
     [kom-bury-buffers]
     [kom-keep-alive-interval]
+    [kom-show-sync-messages]
     "\n"
     [kom-agree-text]
     [kom-mercial]
@@ -117,6 +125,35 @@
     [kom-view-commented-in-window]
     [kom-personal-messages-in-window]
     
+    "\n\n"
+    section
+    (predefined-fonts bold centered)
+    section
+    "\n"
+    [kom-active-face]
+    [kom-url-face]
+    [kom-me-face]
+    [kom-highlight-face]
+    [kom-subject-face]
+    [kom-text-no-face]
+    [kom-friends-face]
+    [kom-morons-face]
+    [kom-presence-face]
+    [kom-first-line-face]
+    [kom-warning-face]
+    [kom-mark-face]
+    [kom-dim-face]
+    [kom-text-body-face]
+    [kom-dashed-lines-face]
+    [kom-async-text-body-face]
+    [kom-async-dashed-lines-face]
+    "\n\n"
+    section
+    (other-fonts bold centered)
+    section
+    "\n"
+    [kom-active-highlight-face]
+    [kom-active-strikethrough-face]
     "\n\n"
     section
     (audio-cues bold centered)
@@ -155,6 +192,7 @@
     [kom-review-priority]
     [kom-server-priority]
     [kom-server-priority-breaks]
+    [kom-session-nickname]
     "\n"
     [kom-read-depth-first]
     [kom-reading-puts-comments-in-pointers-last]
@@ -170,20 +208,26 @@
     [kom-saved-file-name]
     [kom-default-mark]
     [kom-symbolic-marks-alist]
+    [kom-membership-default-message-flag]
     [kom-membership-default-priority]
     [kom-membership-default-placement]
     [kom-unsubscribe-makes-passive]
     [kom-postpone-default]
     "\n"
     [kom-autowrap]
+    [kom-autowrap-timeout]
     [kom-show-author-at-end]
     [kom-show-creating-software]
     [kom-show-imported-importer]
     [kom-show-imported-external-recipients]
     [kom-show-imported-envelope-sender]
+    [kom-show-imported-message-id]
+    [kom-text-properties]
+    [kom-fontify-text]
     [kom-smileys]
     [kom-w3-simplify-body]
     [kom-format-html-authors]
+    [kom-format-show-images]
     "\n"
     [kom-max-overlays]
     [kom-highlight-first-line]
@@ -226,11 +270,18 @@
     (urls bold centered)
     section
     "\n"
+    [kom-text-links]
+    [kom-url-transformation-rules]
     [kom-url-viewer-preferences]
     [kom-windows-browser-command]
     [kom-mosaic-command]
     [kom-netscape-command]
+    [kom-netscape-variant]
     [kom-galeon-command]
+    [kom-lynx-terminal]
+    [kom-lynx-xterm-command]
+    [kom-lynx-terminal-command]
+
 
     "\n\n"
     section
@@ -288,8 +339,7 @@ customize buffer but do not save them to the server."
           (set-default (car (car tmp)) (widget-value (cdr (car tmp))))
           (when (and (listp kom-dont-read-saved-variables)
                      (or (memq (car (car tmp)) lyskom-elisp-variables)
-                         (memq (car (car tmp)) lyskom-global-boolean-variables)
-                         (memq (car (car tmp)) lyskom-global-non-boolean-variables)))
+                         (lyskom-flag-global-variable-from-elisp (car (car tmp)))))
             (setq kom-dont-read-saved-variables 
                   (cons (car (car tmp)) kom-dont-read-saved-variables))))
         (setq tmp (cdr tmp))))))
@@ -312,15 +362,14 @@ customize buffer but do not save them to the server."
                      (symbolp (elt e 0))
                      (or 
                       (and (not (memq (elt e 0) lyskom-elisp-variables))
-                           (not (memq (elt e 0) lyskom-global-boolean-variables))
-                           (not (memq (elt e 0) lyskom-global-non-boolean-variables)))
+                           (not (lyskom-flag-global-variable-from-elisp (elt e 0))))
                       (memq (elt e 0) kom-dont-read-saved-variables))
                      (boundp (elt e 0)))
             (setq var-list (cons (cons (elt e 0)
                                        (symbol-value (elt e 0)))
                                  var-list)))))
        lyskom-customize-buffer-format)
-      
+
       (let* ((actual-save-options-init-file
               (or (and (boundp 'save-options-init-file)
                        save-options-init-file)
@@ -399,11 +448,59 @@ customize buffer but do not save them to the server."
 
 (defun lyskom-widget-click (event)
   (interactive "e")
-  (let ((pos (event-point event)))
+  (let ((pos (lyskom-event-point event)))
     (if (and pos (widget-at pos))
         (widget-button-click event)
       (popup-mode-menu))))
 
+(defun lyskom-customize-mode ()
+  "Major mode for editing LysKOM settings.
+See the top of the buffer for localized help.
+
+These commands work outside editable text fields.
+\\[lyskom-customize-save-and-quit] to save and quit,
+\\[lyskom-customize-save-and-quit] to save and quit,
+\\[lyskom-customize-save] to save without quitting,
+\\[lyskom-customize-quit] to quit without saving.
+\\[widget-forward] moves to the next setting
+\\[widget-button-press] changes the value
+Press C-h m for a complete list of commands.
+
+Documentation:  [?] Show documentation    [!] Hide documentation
+Lists etc.   :  [INS] Add a line   [DEL] Remove a line   [*] Modify
+
+If the box before the name of the setting is selected, the setting will
+be saved in the server. Otherwise it will be saved in your .emacs.
+
+Click prompts with the middle mouse button to change values and bring 
+up menus.
+
+All key bindings:
+\\{lyskom-customize-map}
+"
+  (kill-all-local-variables)
+  (use-local-map lyskom-customize-map)
+  (make-local-variable 'lyskom-widgets)
+  (setq lyskom-widgets nil)
+  (let ((inhibit-read-only t))
+    (erase-buffer))
+  (condition-case nil
+      (lyskom-copy-face kom-active-face 'widget-button-face)
+    (error nil))
+  (mapcar 'lyskom-custom-insert lyskom-customize-buffer-format)
+  (widget-setup)
+  (mapcar (function
+           (lambda (variable)
+             (widget-value-set (cdr variable)
+                               (save-excursion
+                                 (set-buffer lyskom-buffer)
+                                 (symbol-value (car variable))))))
+          lyskom-widgets)
+  (widget-setup)
+  (setq mode-name "LysKOM Settings")
+  (setq major-mode 'lyskom-customize-mode)
+  )
+
 (defun kom-customize ()
   "Open the customize buffer"
   (interactive)
@@ -415,27 +512,10 @@ customize buffer but do not save them to the server."
     (unwind-protect
         (progn
           (lyskom-start-of-command 'kom-customize)
-           (save-excursion
-             (set-buffer buf)
-             (kill-all-local-variables)
-             (make-local-variable 'lyskom-widgets)
-             (setq lyskom-widgets nil)
-             (let ((inhibit-read-only t))
-               (erase-buffer))
-             (use-local-map lyskom-customize-map)
-             (condition-case nil
-                 (lyskom-copy-face 'kom-active-face 'widget-button-face)
-               (error nil))
-             (mapcar 'lyskom-custom-insert lyskom-customize-buffer-format)
-             (widget-setup)
-             (mapcar (function
-                      (lambda (variable)
-                        (widget-value-set (cdr variable)
-                                          (save-excursion
-                                            (set-buffer lyskom-buffer)
-                                            (symbol-value (car variable))))))
-                     lyskom-widgets)
-             (widget-setup))
+          (sit-for 0)
+          (save-excursion
+            (set-buffer buf)
+            (lyskom-customize-mode))
            (lyskom-display-buffer buf))
       (save-excursion
         (set-buffer lyskom-buffer)
@@ -469,9 +549,11 @@ customize buffer but do not save them to the server."
                                           (lyskom-create-widget (aref w 0)))
                                     lyskom-widgets)))))
 
+;;UNUSED: lyskom-custom-insert-bold
 (defun lyskom-custom-insert-bold (s e)
   (add-text-properties s e (list 'face 'bold 'end-closed nil)))
 
+;;UNUSED: lyskom-custom-insert-centered
 (defun lyskom-custom-insert-centered (s e)
   (save-excursion (goto-char s)
                   (center-line 1)))
@@ -485,9 +567,13 @@ customize buffer but do not save them to the server."
 (defun lyskom-custom-string (s)
   (lyskom-get-string s 'lyskom-custom-strings))
 
-
 (defvar lyskom-custom-variables-missing '(kom-permanent-filter-list
-                                          kom-quit-hook)
+                                          kom-quit-hook 
+                                          lyskom-last-known-conf-no
+                                          lyskom-last-known-pers-no
+                                          kom-presence-messages
+                                          kom-dont-complain-about-missing-presentation
+                                          lyskom-settings-version)
   "List of user variables not in lyskom-custom-variables.")
 
 (defvar lyskom-custom-variables
@@ -512,7 +598,7 @@ customize buffer but do not save them to the server."
     (kom-list-membership-in-window (open-window))
     (kom-customize-format (choice ((const (long-format long))
                                    (const (short-format short)))))
-    (kom-default-language (language-choice))
+    (kom-default-language (repeat (language-choice nil :format "%[%t%] %v\n")))
     (kom-user-prompt-format (string))
     (kom-user-prompt-format-executing (string))
     (kom-anonymous-prompt-format (string))
@@ -541,6 +627,8 @@ customize buffer but do not save them to the server."
     (kom-autowrap (choice ((const (on t))
                            (const (off nil))
                            (number nil :tag max-text-length))))
+    (kom-autowrap-timeout (choice ((const (no-timeout nil))
+                                   (number nil :tag autowrap-timeout))))
     (kom-auto-list-faqs (toggle (on off)))
     (kom-auto-review-faqs (toggle (on off)))
     (kom-dashed-lines (toggle (on off)))
@@ -567,6 +655,7 @@ customize buffer but do not save them to the server."
                       :menu-tag some-persons))))
     (kom-show-where-and-what (toggle (yes no)))
     (kom-show-since-and-when (toggle (yes no)))
+    (kom-remember-password (toggle (yes no)))
     (kom-idle-hide (number))
     (kom-show-footnotes-immediately (toggle (yes no)))
     (kom-follow-comments-outside-membership (toggle (yes no)))
@@ -585,6 +674,14 @@ customize buffer but do not save them to the server."
                                          (const (after-conf-letters-server-break after-conf-letters))
                                          (const (when-done-server-break when-done))
                                          (const (no-server-break nil)))))
+    (kom-session-nickname (choice ((const (automatic-session-name nil))
+				   (string nil 
+					   :tag given-session-name
+					   :help-echo select-buffer))))			 
+
+    (kom-relogin-inhibit-commands (repeat (command nil :tag command)
+					  :indent 4))
+
     (kom-login-hook (repeat (command nil :tag command)
                             :indent 4))
     (kom-do-when-done (repeat (choice ((command nil :tag command)
@@ -612,11 +709,17 @@ customize buffer but do not save them to the server."
                       :help-echo select-priority
                       :format "%[%t%] (%v)"
                       :size 0))))
+    (kom-membership-default-message-flag
+     (choice ((const (ask-every-time ask))
+              (const (yes t))
+              (const (no nil)))))
     (kom-show-personal-messages-in-buffer
      (choice ((const (messages-in-lyskom-buffer t))
               (const (discard-messages nil))
               (string nil :tag in-named-buffer :help-echo select-buffer))))
-    (kom-pop-personal-messages (toggle (yes no)))
+    (kom-pop-personal-messages (choice ((const (yes t))
+                                        (const (yes-dont-pop yes))
+                                        (const (no nil)))))
     (kom-ding-on-new-letter (ding))
     (kom-ding-on-priority-break (ding))
     (kom-ding-on-wait-done (ding))
@@ -631,14 +734,54 @@ customize buffer but do not save them to the server."
               (const (sender-rcpt sender))
               (const (last-recipient-recpt last-recipient)))))
     (kom-filter-outgoing-messages (noggle (yes no)))
+    (kom-highlight-conferences
+     (repeat (cons ((choice ((const (morons kom-morons))
+                             (const (friends kom-friends))
+                             (const (me lyskom-pers-no))
+                             (const (i-am-supervisor lyskom-highlight-i-am-supervisor))
+                             (const (has-no-presentation lyskom-highlight-has-no-presentation))
+                             (repeat (person nil :tag name) 
+                                     :indent 12
+                                     :tag highlight-conflist
+                                     :lyskom-predicate (pers conf)
+                                     :menu-tag highlight-conflist)
+                             )
+                            :tag highlight-conferences
+                            :format "%[%t%] %v\n"
+                            )
+                    (face (kom-active-face
+                           kom-active-highlight-face
+                           kom-active-strikethrough-face
+                           kom-friends-face
+                           kom-morons-face
+                           kom-me-face) :tag highlight-face))
+                   :format "%v"
+                   )
+             ))
     (kom-friends (repeat (person nil :tag name) :indent 4))
     (kom-morons (repeat (person nil :tag name) :indent 4))
     (kom-url-viewer-preferences (repeat (url-viewer nil :tag viewer-program)
                                         :indent 4))
     (kom-windows-browser-command (file))
-    (kom-mosaic-command (file))
-    (kom-netscape-command (file))
-    (kom-galeon-command (file))
+    (kom-mosaic-command (choice ((file nil :tag ext-simple-command :format "%[%t:%] %v\n")
+                                     (repeat (string nil :tag ext-complex-component :format "%[%t:%] %v\n")
+                                             :tag ext-complex-command
+                                             :menu-tag ext-complex-command))))
+    (kom-netscape-command (choice ((file nil :tag ext-simple-command :format "%[%t:%] %v\n")
+                                     (repeat (string nil :tag ext-complex-component :format "%[%t:%] %v\n")
+                                             :tag ext-complex-command
+                                             :menu-tag ext-complex-command))))
+    (kom-galeon-command (choice ((file nil :tag ext-simple-command :format "%[%t:%] %v\n")
+                                     (repeat (string nil :tag ext-complex-component :format "%[%t:%] %v\n")
+                                             :tag ext-complex-command
+                                             :menu-tag ext-complex-command))))
+    (kom-lynx-terminal (choice ((const (lynx-xterm xterm))
+                                (const (lynx-emacs terminal)))))
+    (kom-lynx-terminal-command (file))
+    (kom-lynx-xterm-command  (choice ((file nil :tag ext-simple-command :format "%[%t:%] %v\n")
+                                     (repeat (string nil :tag ext-complex-component :format "%[%t:%] %v\n")
+                                             :tag ext-complex-command
+                                             :menu-tag ext-complex-command))))
     (kom-confirm-multiple-recipients
      (choice ((const (dont-check nil))
               (const (check-before-open before))
@@ -660,9 +803,20 @@ customize buffer but do not save them to the server."
                                     :indent 4))
     (kom-self-control (toggle (yes no)))
     (kom-ispell-dictionary (ispell-dictionary))
-    (kom-show-namedays (toggle (on off)))
+    (kom-show-namedays (choice ((const (off nil))
+                                (const (default-namedays t))
+                                (repeat (nameday nil :format "%[%t%]: `%v'\n" :tag specific-namedays)
+                                        :tag several-name-lists
+                                        :menu-tag several-name-lists
+                                        :indent 4)
+                                (nameday nil :tag specific-namedays))))
 
     (kom-show-week-number (toggle (on off)))
+
+    (kom-relogin-behaviour (choice ((const (off nil))
+				    (const (ask ask))
+				    (const (on t)))))
+
     (kom-membership-default-placement (choice ((const (last last))
                                                (const (first first))
                                                (number nil
@@ -671,6 +825,7 @@ customize buffer but do not save them to the server."
     (kom-show-imported-importer (toggle (on off)))
     (kom-show-imported-envelope-sender (toggle (on off)))
     (kom-show-imported-external-recipients (toggle (on off)))
+    (kom-show-imported-message-id (toggle (on off)))
     (kom-agree-text (choice ((string nil :tag a-string)
                              (repeat (string nil
                                              :tag a-string
@@ -707,6 +862,7 @@ customize buffer but do not save them to the server."
                                                      (const (all-others t) :format "%t\n"))
                                                     :format "%[%v%]")
                                             (toggle (yes no) :tag format-html)))))
+    (kom-format-show-images (toggle (on off)))
     (kom-bury-buffers (toggle (on off)))
     (kom-ansaphone-replies (ansaphone))
     (kom-complete-numbers-before-names (toggle (on off)))
@@ -717,15 +873,70 @@ customize buffer but do not save them to the server."
     (kom-saved-file-name (file))
     (kom-follow-attachments (toggle (yes no)))
     (kom-show-unread-in-frame-title (toggle (yes no)))
+
+    (kom-unread-mode-line-type (choice ((const (simple-unread-mode-line nil))
+					(const (explicit-unread-mode-line t)))))
+
     (kom-confirm-add-recipients (toggle (yes no)))
     (kom-trim-buffer-minimum (number))
     (kom-dont-check-commented-authors (repeat (person nil 
                                                       :tag name) :indent 4))
     (kom-print-relative-dates (toggle (yes no)))
     (kom-print-seconds-in-time-strings (toggle (yes no)))
-    (kom-extended-status-information (toggle (yes no)))
+    (kom-extended-status-information (choice ((const (yes t))
+                                              (repeat (cons ((choice ((const (extended-conf-info server))
+                                                                      (const (extended-pers-info pers))
+                                                                      (const (extended-server-info server))
+                                                                      (const (extended-read-faq-info read-faq))
+                                                                      (const (extended-statistics-info raw-server-stats))
+                                                                      (const (extended-all-other-info t))
+                                                                      )
+                                                                     :tag extended-info-type
+                                                                     :format "%[%t%] %v ")
+                                                             (toggle (yes no) :tag extended-info-show)))
+                                                      :tag specific-extended-info 
+                                                      :menu-tag specific-extended-info))))
     (kom-edit-hide-add-button (noggle (yes no)))
     (kom-keyboard-menu-immediate-selection (noggle (yes no)))
+    (kom-url-transformation-rules (repeat (cons ((string nil :tag url-transform-regexp)
+                                                 (string nil :tag url-transform-newtext)))))
+    (kom-text-links (repeat (cons ((choice ((person nil :tag link-specific-rcpt :lyskom-predicate (pers conf))
+                                            (const (all-conferences t)))
+                                           :tag link-pattern-for
+                                           :format "%[%t%]: %v")
+                                   (repeat (list ((string nil :tag link-pattern :format "%[%t%]: %v\n")
+                                                  (string nil :tag link-replace :format "%[%t%]: %v\n")
+                                                  (number (0 9) :tag link-highlight-match :format "%[%t%]: %v\n")
+                                                  (toggle (yes no) :tag link-fold-case :format "%[%t%]: %v"))
+                                                 ) :indent 8 :tag "")
+                                   ))
+                            :indent 4))
+    (kom-url-face (face))
+    (kom-text-no-face (face))
+    (kom-active-face (face))
+    (kom-active-highlight-face (face))
+    (kom-active-strikethrough-face (face))
+    (kom-highlight-face (face))
+    (kom-me-face (face))
+    (kom-friends-face (face))
+    (kom-morons-face (face))
+    (kom-subject-face (face))
+    (kom-presence-face (face))
+    (kom-first-line-face (face))
+    (kom-warning-face (face))
+    (kom-mark-face (face))
+    (kom-dim-face (face))
+    (kom-text-body-face (face t))
+    (kom-dashed-lines-face (face t))
+    (kom-async-text-body-face (face t))
+    (kom-async-dashed-lines-face (face t))
+    (kom-show-sync-messages (toggle (yes no)))
+    (kom-netscape-variant (choice ((const (default-netscape nil))
+                                   (const (new-window-netscape new-window))
+                                   (const (new-tab-netscape new-tab)))))
+    (kom-auto-confirm-new-conferences (toggle (yes no)))
+    (kom-text-properties (toggle (yes no)))
+    (kom-fontify-text (toggle (yes no)))
 ))
 
 (defvar lyskom-widget-functions 
@@ -738,6 +949,9 @@ customize buffer but do not save them to the server."
     (const .  lyskom-item-widget)
     (repeat . lyskom-repeat-widget)
     (cons . lyskom-cons-widget)
+    (list . lyskom-list-widget)
+    (face . lyskom-face-widget)
+    (nameday . lyskom-nameday-widget)
     (kbd-macro . lyskom-kbd-macro-widget)
     (url-viewer . lyskom-url-viewer-widget)
     (ispell-dictionary . lyskom-ispell-dictionary-widget)
@@ -746,6 +960,7 @@ customize buffer but do not save them to the server."
     (person . lyskom-person-widget)
     (language-choice . lyskom-language-widget)
     (file . lyskom-file-widget)
+    (ext-command . lyskom-external-command-widget)
     (ansaphone . lyskom-ansaphone-reply-widget)
     (mark-association . lyskom-mark-association-widget)
 ))
@@ -759,7 +974,7 @@ customize buffer but do not save them to the server."
 (defun lyskom-create-widget (variable)
   (let* ((el (assq variable lyskom-custom-variables))
          (dummy (or el (error "Unknown variable: %S" variable)))
-         (spec (lyskom-widget-convert-specification (car (cdr el))))
+         (spec (lyskom-widget-convert-specification (car (cdr el)) variable))
          (tag-sym (intern (concat (symbol-name variable) "-tag")))
          (doc-sym (intern (concat (symbol-name variable) "-doc")))
          (help-sym (intern (concat (symbol-name variable) "-help")))
@@ -770,18 +985,15 @@ customize buffer but do not save them to the server."
     (lyskom-ignore value help-sym dummy)       ; Are they ever used?
 
     (setq storage-widget
-          (condition-case nil
-              (prog1
-                (widget-create 'checkbox
-                               ':value (and (not (memq variable kom-dont-read-saved-variables))
-                                            (or (memq variable lyskom-elisp-variables)
-                                                (memq variable lyskom-global-boolean-variables)
-                                                (memq variable lyskom-global-non-boolean-variables)))
-                               ':args variable
-                               ':format "%[%v%]"
-                               ':help-echo (lyskom-custom-string 'variable-type-help))
-                (widget-insert " "))
-            (error nil)))
+	  (prog1
+	      (widget-create 'checkbox
+			     ':value (and (not (memq variable kom-dont-read-saved-variables))
+					  (or (memq variable lyskom-elisp-variables)
+					      (lyskom-flag-global-variable-from-elisp variable)))
+			     ':args (list variable)
+			     ':format "%[%v%]"
+			     ':help-echo (lyskom-custom-string 'variable-type-help))
+	    (widget-insert " ")))
 
     (setq spec 
           (cons (car spec)
@@ -799,11 +1011,15 @@ customize buffer but do not save them to the server."
                        ':lyskom-storage-widget storage-widget
                        )
                  (cdr spec))))
+
     (let ((widget (apply 'widget-create spec)))
       (condition-case nil
           (progn
             (if (string= "" (lyskom-custom-string doc-sym))
-                (widget-insert "\n")
+                (progn (widget-insert (lyskom-format " %#1@(%#2s)" 
+                                                     `(face ,kom-dim-face)
+                                                     (symbol-name variable)))
+                       (widget-insert "\n"))
               (widget-insert "  ")
               (widget-create 'lyskom-widget-help
                              ':value (lyskom-default-value 'kom-customize-format)
@@ -823,12 +1039,14 @@ customize buffer but do not save them to the server."
                                          ""
                                          (lyskom-custom-string 'show-doc)
                                          "?"))
-                             ':format "%[[%T]%]\n%D")))
+                             ':format (lyskom-format "%%[[%%T]%%] %[%#1@(%#2s)%]\n%%D"
+                                                     `(face ,kom-dim-face)
+                                                     (symbol-name variable)))))
         (error (widget-insert "\n")))
         widget)))
 
 
-(defun lyskom-widget-convert-specification (spec)
+(defun lyskom-widget-convert-specification (spec var)
   "Convert a LysKOM widget specification to something widget-create accepts"
   (let ((convertfn (assq (car spec) lyskom-widget-functions)))
     (if (null convertfn)
@@ -836,7 +1054,8 @@ customize buffer but do not save them to the server."
     (nconc (funcall (cdr convertfn)
                     (car spec)
                     (car (cdr spec))
-                    (cdr (cdr spec)))
+                    (cdr (cdr spec))
+                    var)
            (lyskom-widget-convert-props spec))))
 
 (defun lyskom-widget-convert-props (spec)
@@ -872,26 +1091,26 @@ customize buffer but do not save them to the server."
     (cons type new-props)))
 
 
-(defun lyskom-file-widget (type &optional args propl)
+(defun lyskom-file-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec 'file
                                    '(:format "%[%t%] %v" :size 0)
                                    propl))
 
-(defun lyskom-person-widget (type &optional args propl)
+(defun lyskom-person-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec 'lyskom-name nil propl))
 
-(defun lyskom-mark-association-widget (type &optional args propl)
+(defun lyskom-mark-association-widget (type &optional args propl ar)
   (lyskom-build-simple-widget-spec 'lyskom-mark-association nil propl))
 
-(defun lyskom-command-widget (type &optional args propl)
+(defun lyskom-command-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec 'lyskom-command nil propl))
 
-(defun lyskom-kbd-macro-widget (type &optional args propl)
+(defun lyskom-kbd-macro-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec 'lyskom-kbd-macro
                                    (list ':macro-buffer lyskom-buffer)
                                    propl))
 
-(defun lyskom-item-widget (type &optional args propl)
+(defun lyskom-item-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec 'item
                                    (list ':format "%t"
                                          ':tag (lyskom-custom-string 
@@ -899,11 +1118,12 @@ customize buffer but do not save them to the server."
                                          ':value (elt args 1))
                                    propl))
 
-(defun lyskom-language-widget (type &optional args propl)
+(defun lyskom-language-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec
    'menu-choice
    (list ':format "%[%t%] %v"
          ':case-fold t
+         ':tag (lyskom-custom-string 'language)
          ':args
          (mapcar
           (function
@@ -915,7 +1135,22 @@ customize buffer but do not save them to the server."
           lyskom-languages))
    propl))
 
-(defun lyskom-ispell-dictionary-widget (type &optional args propl)
+(defun lyskom-nameday-widget (type &optional args propl var)
+  (lyskom-build-simple-widget-spec
+   'menu-choice
+   (list ':format "%[%t%] %v"
+         ':case-fold t
+         ':args
+         (mapcar
+          (lambda (x)
+            (list 'item
+                  ':tag (nameday-data->name (cdr x))
+                  ':format "%t"
+                  ':value (nameday-data->code (cdr x))))
+          lyskom-namedays))
+   propl))
+
+(defun lyskom-ispell-dictionary-widget (type &optional args propl var)
   (let ((tmp-dictionary-alist nil))
     (condition-case nil 
         (progn (require 'ispell)
@@ -953,7 +1188,38 @@ customize buffer but do not save them to the server."
                         tmp-dictionary-alist))))
      propl)))
 
-(defun lyskom-url-viewer-widget (type &optional args propl)
+(defun lyskom-face-widget (type &optional args propl var)
+  (let ((wargs (append (mapcar (lambda (f)
+                                 (list 'item 
+                                       :tag (symbol-name f)
+                                       :value f
+                                       :format "%t"))
+                               (or
+                                (and (listp args) args)
+                                (cdr (assq var lyskom-predefined-faces))))
+                       (list (list 'symbol
+                                   ':size 30
+                                   ':tag (lyskom-custom-string 
+                                          'other-face))))))
+    (when (eq args t)
+      (setq wargs 
+            (append (list (list 'item
+                                :tag (lyskom-custom-string 'generated-face)
+                                :value nil
+                                :format "%t"))
+                    wargs)))
+    (if wargs
+        (lyskom-build-simple-widget-spec 'menu-choice
+                                         (list ':format "%[%t%] %v\n"
+                                               ':args wargs)
+                                         propl)
+      (lyskom-build-simple-widget-spec 'symbol
+                                       (list ':format "%[%t%] %v\n"
+                                             ':size 30
+                                             ':args wargs)
+                                       propl))))
+
+(defun lyskom-url-viewer-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec
    'menu-choice 
    (list ':format "%[%v%]\n"
@@ -1010,7 +1276,7 @@ customize buffer but do not save them to the server."
                      ':value "galeon")))
    propl))
 
-(defun lyskom-open-window-widget (type &optional args propl)
+(defun lyskom-open-window-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec
    'menu-choice
    (list ':case-fold t
@@ -1039,7 +1305,7 @@ customize buffer but do not save them to the server."
                      ':size 0)))
    propl))
 
-(defun lyskom-ding-widget (type &optional args propl)
+(defun lyskom-ding-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec
    'menu-choice
    (list  ':case-fold t
@@ -1053,7 +1319,7 @@ customize buffer but do not save them to the server."
                 (list 'lyskom-number
                       ':tag (lyskom-custom-string 'number-of-times)
                       ':help-echo (lyskom-custom-string 'select-number)
-                      ':value "1"
+                      ':value 1
                       ':format "%[%t%]: (%v)"
                       ':size 0
                       ':min-value 1
@@ -1092,7 +1358,7 @@ customize buffer but do not save them to the server."
                                        :match (lambda (w v) (or (null v) (eq v 0))))
                                  (lyskom-number :tag ,(lyskom-custom-string 'number-of-times)
                                                 :help-echo ,(lyskom-custom-string 'select-number)
-                                                :value "1"
+                                                :value 1
                                                 :format "%[%t%]: (%v)"
                                                 :size 0
                                                 :min-value 1
@@ -1104,7 +1370,7 @@ customize buffer but do not save them to the server."
                 ))
    propl))
 
-(defun lyskom-toggle-widget-inverse (type &optional args propl)
+(defun lyskom-toggle-widget-inverse (type &optional args propl var)
   (lyskom-build-simple-widget-spec
    'menu-choice
    (list ':case-fold t
@@ -1120,7 +1386,7 @@ customize buffer but do not save them to the server."
                      ':format "%t")))
    propl))
 
-(defun lyskom-toggle-widget (type &optional args propl)
+(defun lyskom-toggle-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec
    'menu-choice
    (list ':case-fold t
@@ -1137,34 +1403,42 @@ customize buffer but do not save them to the server."
    propl))
 
 
-(defun lyskom-repeat-widget (type &optional args propl)
+(defun lyskom-repeat-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec
    'editable-list 
    (list ':format "%[%t%]\n%v%i"
          ':args
-         (list (lyskom-widget-convert-specification args)))
+         (list (lyskom-widget-convert-specification args var)))
    propl))
 
-(defun lyskom-cons-widget (type &optional args propl)
+(defun lyskom-cons-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec
    'cons
    (list ':format "%v\n"
          ':tag ""
-         ':args (list (lyskom-widget-convert-specification (elt args 0))
-                      (lyskom-widget-convert-specification (elt args 1))))
+         ':args (list (lyskom-widget-convert-specification (elt args 0) var)
+                      (lyskom-widget-convert-specification (elt args 1) var)))
    propl))
 
-(defun lyskom-choice-widget (type &optional args propl)
+(defun lyskom-list-widget (type &optional args propl var)
+  (lyskom-build-simple-widget-spec
+   'list
+   (list ':format "%v\n"
+         ':tag ""
+         ':args (mapcar (lambda (x) (lyskom-widget-convert-specification x var)) args))
+   propl))
+
+(defun lyskom-choice-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec
    'menu-choice
    (list ':case-fold t
          ':format "%[%t%] %v"
          ':args
-         (mapcar 'lyskom-widget-convert-specification args))
+         (mapcar (lambda (x) (lyskom-widget-convert-specification x var)) args))
    propl))
 
 
-(defun lyskom-string-widget (type &optional args propl)
+(defun lyskom-string-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec
    'lyskom-string 
    (list ':size 0 
@@ -1172,23 +1446,24 @@ customize buffer but do not save them to the server."
    propl))
 
 
-(defun lyskom-number-widget (type &optional args propl)
+(defun lyskom-number-widget (type &optional args propl var)
   (if args
       (lyskom-build-simple-widget-spec
        'lyskom-number
        (list ':min-value (elt args 0)
              ':max-value (elt args 1)
+	     ':value 0
              ':size 0)
        propl)
     (lyskom-build-simple-widget-spec
      'lyskom-number
-     (list ':size 0)
+     (list ':size 0 ':value 0)
      propl)))
 
 ;;;
 ;;; The ansaphone reply widget (whew!)
 
-(defun lyskom-ansaphone-reply-widget (type &optional args propl)
+(defun lyskom-ansaphone-reply-widget (type &optional args propl var)
   (lyskom-build-simple-widget-spec
    'editable-list
    (list ':format "%t:\n%v%i\n"
@@ -1289,9 +1564,7 @@ customize buffer but do not save them to the server."
                                'which-conf)
                               (t 'which-name)))
                        (widget-get widget ':lyskom-predicate)
-                       nil
-                       nil
-                       t)))
+                       nil "" t)))
   (widget-setup))
                
 (defun lyskom-widget-name-value-create (widget)
@@ -1456,10 +1729,16 @@ customize buffer but do not save them to the server."
     (widget-setup)))
 
 (defun lyskom-widget-number-value-to-external (widget value)
-  (string-to-int value))
+  (cond ((stringp value) (string-to-int value))
+	((numberp value) value)
+	((null value) 0)
+	(t (error "unable to convert settings value"))))
 
 (defun lyskom-widget-number-value-to-internal (widget value)
-  (format "% 4d " value))
+  (cond ((stringp value) value)
+	((numberp value) (format "% 4d" value))
+	((null value) "")
+	(t (format "%S" value))))
 
 (defun lyskom-widget-number-validate (widget)
   (if (numberp (widget-value widget))
