@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;;; $Id: parse.el,v 44.43 2002/08/08 07:44:21 ceder Exp $
+;;;;; $Id: parse.el,v 44.59 2004/06/26 13:32:32 byers Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -35,7 +35,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: parse.el,v 44.43 2002/08/08 07:44:21 ceder Exp $\n"))
+	      "$Id: parse.el,v 44.59 2004/06/26 13:32:32 byers Exp $\n"))
 
 
 ;;; ================================================================
@@ -108,7 +108,7 @@ first non-white character was not equal to CHAR."
   (lyskom-string-skip-whitespace     
    (buffer-substring lyskom-parse-pos (point-max))))
 
-			  
+
 (defun lyskom-parse-num ()
   "Parse the next token as a number.
 Signal lyskom-parse-incomplete if the number is not followed by whitespace.
@@ -129,15 +129,31 @@ Signal lyskom-protocol-error if the next token is not a number."
                              (lyskom-string-to-parse))))
 )
 
+(defun lyskom-parse-float ()
+  "Parse the next token as a float.
+Signal lyskom-parse-incomplete if the number is not followed by whitespace.
+Signal lyskom-protocol-error if the next token is not a number."
+  (goto-char lyskom-parse-pos)
+  (cond ((looking-at "[ \n]*[-+]?[0-9]*\\(\\.\\([0-9]*\\([eE]\\([-+]?[0-9]*\\)?\\)?\\)?\\)?")
+         (if (char-after (match-end 0))
+             (progn (setq lyskom-parse-pos (goto-char (match-end 0)))
+                    (string-to-number (match-string 0)))
+           (signal 'lyskom-parse-incomplete nil)))
+        ((looking-at "[ \n]*\\'")
+         (goto-char (point-max))
+         (signal 'lyskom-parse-incomplete nil))
+        (t (lyskom-protocol-error 'lyskom-parse-num
+                                  "Expected float, got %S"
+                                  (lyskom-string-to-parse)))))
+
 
 
 (defun lyskom-parse-string ()
   "Parse the next token as a string in the server coding system.
 Signal lyskom-parse-incomplete if the string is not complete.
 Signal lyskom-protocol-error if the next token is not a string."
-  (decode-coding-string 
-   (lyskom-parse-raw-string)
-   lyskom-server-coding-system))
+  (lyskom-decode-coding-string (lyskom-parse-raw-string)
+                               lyskom-server-coding-system))
 
 (defun lyskom-parse-raw-string ()
   "Parse next token as a raw string.
@@ -237,13 +253,14 @@ Signal lyskom-protocol-error if the next token is not a string."
             (lyskom-setq-default lyskom-string-bytes-missing 0)
 	    (setq lyskom-parse-pos (+ lyskom-parse-pos len))))))))))
 
+;;UNUSED: lyskom-parse-coded-string
 (defun lyskom-parse-coded-string ()
   "Parse a string with explicit coding."
   (let ((coding (intern (lyskom-parse-coding)))
         (data (lyskom-parse-raw-string)))
     (condition-case nil
-        (progn (check-coding-system coding)
-               (decode-coding-string data coding))
+        (progn (lyskom-check-coding-system coding)
+               (lyskom-decode-coding-string data coding))
       (nil data))))
 
 
@@ -295,7 +312,7 @@ Signal lyskom-parse-incomplete if there is no nonwhite char to parse."
 
 (defun lyskom-parse-time ()
   "Parse a time from server. Args: none."
-  (lyskom-create-time
+  (lyskom-create-time-from-utc
    (lyskom-parse-num)			;sec
    (lyskom-parse-num)			;min
    (lyskom-parse-num)			;hour
@@ -303,7 +320,7 @@ Signal lyskom-parse-incomplete if there is no nonwhite char to parse."
    (1+ (lyskom-parse-num))		;mon
    (+ 1900 (lyskom-parse-num))		;year
    (lyskom-parse-num)			;wday
-   (lyskom-parse-num)			;yday
+   (1+ (lyskom-parse-num))		;yday
    (lyskom-parse-num)))			;isdst
 
 
@@ -378,7 +395,9 @@ Each element is parsed by PARSER, a function that takes no arguments."
         (while (> len 0)
           (setq result (cons (funcall parser) result))
           (setq len (1- len)))
-        (nreverse result)))))
+        (prog1
+            (nreverse result)
+          (lyskom-expect-char ?}))))))
 
 
 (defun lyskom-fill-vector (vector parser)
@@ -594,8 +613,14 @@ than 0. Args: ITEMS-TO-PARSE PRE-FETCHED. Returns -1 if ITEMS-TO-PARSE is
 
 (defun lyskom-parse-membership-type ()
   "Parse a membership type"
-  (apply 'lyskom-create-membership-type
-         (lyskom-parse-bitstring '(nil nil nil nil nil nil nil nil))))
+  (let ((type
+         (apply 'lyskom-create-membership-type
+                (lyskom-parse-bitstring '(nil nil nil nil nil nil nil nil)))))
+    (set-membership-type->message-flag
+     type
+     (eq (membership-type->passive type)
+         (membership-type->message-flag type)))
+    type))
 
 (defun lyskom-parse-member-old ()
   "Parse an old-style member record."
@@ -620,7 +645,38 @@ than 0. Args: ITEMS-TO-PARSE PRE-FETCHED. Returns -1 if ITEMS-TO-PARSE is
    (lyskom-parse-vector (lyskom-parse-num)
                         'lyskom-parse-member)))
 
-(defun lyskom-parse-membership ()
+(defun lyskom-convert-read-ranges-to-map (read-ranges)
+  (let ((result nil)
+        a b)
+    (while read-ranges
+      (setq a (car (car read-ranges))
+            b (cdr (car read-ranges))
+            read-ranges (cdr read-ranges))
+      (while (<= a b) (setq result (cons a result) a (1+ a))))
+    (and result (apply 'vector (nreverse result)))))
+
+(defun lyskom-parse-membership-11 ()
+  "Parse a membership, converting read-ranges to the old style."
+  (let ((read-ranges nil))
+    (lyskom-create-membership
+     (lyskom-parse-num)                 ;position
+     (lyskom-parse-time)                ;last-time-read
+     (lyskom-parse-num)			;conf-no
+     (lyskom-parse-num)			;priority
+     (progn (setq read-ranges 
+                  (listify-vector
+                   (lyskom-parse-vector
+                    (lyskom-parse-num) 'lyskom-parse-read-range)))
+            (if (eq (car (car read-ranges)) 1)
+                (prog1 (cdr (elt read-ranges 0))
+                       (setq read-ranges (cdr read-ranges)))
+              0))
+     (lyskom-convert-read-ranges-to-map read-ranges)
+     (lyskom-parse-num)                 ;added-by
+     (lyskom-parse-time)                ;added-at
+     (lyskom-parse-membership-type))))
+
+(defun lyskom-parse-membership-10 ()
   "Parse a membership."
     (lyskom-create-membership
      (lyskom-parse-num)                 ;position
@@ -1020,7 +1076,7 @@ Args: TEXT-NO. Value: text-stat."
 (defun lyskom-parse-local-to-global-block (block-type)
   "Parse a Local-To-Global-Block"
   (cond ((eq block-type 'sparse)
-         (let ((len (lyskom-parse-string)))
+         (let ((len (lyskom-parse-num)))
            (lyskom-parse-list len 'lyskom-parse-text-number-pair)))
         ((eq block-type 'dense)
          (lyskom-parse-map))))
@@ -1029,13 +1085,60 @@ Args: TEXT-NO. Value: text-stat."
   "Parse a Text-Number-Pair"
   (lyskom-create-text-pair (lyskom-parse-num) (lyskom-parse-num)))
 
+(defun lyskom-parse-read-range ()
+  "Parse a Read-Range"
+  (lyskom-create-text-pair (lyskom-parse-num) (lyskom-parse-num)))
+
+(defun lyskom-parse-stats-description ()
+  "Parse a Stats-Description"
+  (let ((tmp (lyskom-create-stats-description
+              (lyskom-parse-vector (lyskom-parse-num) 'lyskom-parse-string)
+              (lyskom-parse-vector (lyskom-parse-num) 'lyskom-parse-num))))
+    (lyskom-save-excursion
+      (set-buffer lyskom-buffer)
+      (setq lyskom-stats-description tmp))))
+
+(defun lyskom-parse-stats ()
+  "Parse a Stats"
+  (lyskom-create-stats (lyskom-parse-float)
+                       (lyskom-parse-float)
+                       (lyskom-parse-float)))
+
+(defun lyskom-parse-stats-array ()
+  "Parse an array of Stats"
+  (lyskom-parse-vector (lyskom-parse-num) 'lyskom-parse-stats))
+
+(defun lyskom-parse-static-server-info ()
+  "Parse a static-server-info"
+  (let ((tmp (lyskom-create-static-server-info (lyskom-parse-time)
+                                               (lyskom-parse-time)
+                                               (lyskom-parse-string)
+                                               (lyskom-parse-num)
+                                               (lyskom-parse-num)
+                                               (lyskom-parse-num)
+                                               (lyskom-parse-num)
+                                               (lyskom-parse-num))))
+    (lyskom-save-excursion
+      (set-buffer lyskom-buffer)
+      (setq lyskom-static-server-info tmp))))
+
+(defun lyskom-parse-scheduling-info ()
+  "Parse a scheduling-info"
+  (lyskom-create-scheduling-info (lyskom-parse-num)
+                                 (lyskom-parse-num)))
+
+
 ;;; ================================================================
 ;;;          Parsing of complex datatypes without cache.
 
 
-(defun lyskom-parse-membership-list ()
+(defun lyskom-parse-membership-list-11 ()
   "Parse a membership-list. Returns a vector."
-   (lyskom-parse-vector (lyskom-parse-num) 'lyskom-parse-membership))
+   (lyskom-parse-vector (lyskom-parse-num) 'lyskom-parse-membership-11))
+
+(defun lyskom-parse-membership-list-10 ()
+  "Parse a membership-list. Returns a vector."
+   (lyskom-parse-vector (lyskom-parse-num) 'lyskom-parse-membership-10))
 
 (defun lyskom-parse-membership-list-old ()
   "Parse a membership-list. Returns a vector."
@@ -1065,8 +1168,10 @@ i.e creates the buffer, sets all markers and pointers."
 	 (concat (if lyskom-debug-communications-to-buffer "" " ")
 		 (buffer-name)
 		 "-replies")))
+  (lyskom-set-buffer-parent lyskom-unparsed-buffer lyskom-buffer)
+  (lyskom-update-inherited-variables lyskom-unparsed-buffer)
   (save-excursion (set-buffer lyskom-unparsed-buffer)
-		  (set-buffer-multibyte nil))
+		  (lyskom-set-buffer-multibyte nil))
   (setq lyskom-unparsed-marker 
 	(lyskom-save-excursion
 	 (let ((proc lyskom-proc))
@@ -1150,8 +1255,8 @@ CALL-INFO is destructively changed to
 (defun lyskom-tr-call-to-parsed (call-info result)
   "Transform a CALL to a PARSED."
   (setcar call-info 'PARSED)
-  (setcar (cdr call-info) result)
-  (setcdr (cdr call-info) (cdr (cdr (cdr (cdr call-info))))))
+  (setcar (cdr (cdr call-info)) result)
+  (setcdr (cdr (cdr call-info)) (cdr (cdr (cdr (cdr call-info))))))
 
 
 (defun lyskom-parse-error (ref-no buffer)
@@ -1211,31 +1316,32 @@ functions and variables that are connected with the lyskom-buffer."
         (set-buffer lyskom-unparsed-buffer)
         (while (not (zerop (1- (point-max)))) ;Parse while replies.
           (let* ((lyskom-parse-pos 1)
-                 (key (lyskom-parse-nonwhite-char)))
-            (condition-case err
-                (let ((inhibit-quit t))	; Used to be nil, but that can
+                 (key (lyskom-parse-nonwhite-char))
+                 (endpos nil))
+            (unwind-protect
+                (condition-case err
+                    (let ((inhibit-quit t)) ; Used to be nil, but that can
 					; cause hard-to-repair
 					; problems
-                  (cond
-                   ((= key ?=)		;The call succeeded.
-                    (lyskom-parse-success (lyskom-parse-num) lyskom-buffer))
-                   ((= key ?%)		;The call was not successful.
-                    (lyskom-parse-error (lyskom-parse-num) lyskom-buffer))
-                   ((= key ?:)		;An asynchronous message.
-                    (lyskom-parse-async (lyskom-parse-num) lyskom-buffer))
-                   (t
-                    (lyskom-protocol-error 'lyskom-parse-unparsed
-                                           "Expected =, %% or :, got %S"
-                                           (lyskom-string-to-parse))))
-                  (delete-region (point-min) lyskom-parse-pos))
-              ;; One reply is now parsed.
-              (lyskom-protocol-error
-               (delete-region (point-min) (min (point-max) (1+ lyskom-parse-pos)))
-               (signal 'lyskom-protocol-error err))
-              (lyskom-parse-incomplete (signal (car err) (cdr err)))
-              (error (delete-region (point-min) lyskom-parse-pos)
-                     (signal (car err) (cdr err)))
-)
+                      (cond
+                       ((= key ?=)      ;The call succeeded.
+                        (lyskom-parse-success (lyskom-parse-num) lyskom-buffer))
+                       ((= key ?%)      ;The call was not successful.
+                        (lyskom-parse-error (lyskom-parse-num) lyskom-buffer))
+                       ((= key ?:)      ;An asynchronous message.
+                        (lyskom-parse-async (lyskom-parse-num) lyskom-buffer))
+                       (t
+                        (lyskom-protocol-error 'lyskom-parse-unparsed
+                                               "Expected =, %% or :, got %S"
+                                               (lyskom-string-to-parse))))
+                      (setq endpos lyskom-parse-pos))
+                  ;; One reply is now parsed.
+                  (lyskom-protocol-error
+                   (setq endpos (min (point-max) (1+ lyskom-parse-pos)))
+                   (signal 'lyskom-protocol-error err))
+                  (lyskom-parse-incomplete (setq endpos (point-min))
+                                           (signal (car err) (cdr err))))
+              (delete-region (point-min) (or endpos lyskom-parse-pos)))
             (goto-char (point-min))
             (if (looking-at "[ \n]+")
                 (delete-region (match-beginning 0) (match-end 0)))
