@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;;; $Id: services.el,v 44.34 2002/08/09 15:14:50 byers Exp $
+;;;;; $Id: services.el,v 44.49 2004/10/28 18:51:55 byers Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -32,7 +32,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: services.el,v 44.34 2002/08/09 15:14:50 byers Exp $\n"))
+	      "$Id: services.el,v 44.49 2004/10/28 18:51:55 byers Exp $\n"))
 
 
 ;;; ================================================================
@@ -125,6 +125,9 @@ Args: KOM-QUEUE HANDLER &rest DATA."
   "Change working conference.
 Args: KOM-QUEUE HANDLER CONF-NO &rest DATA."
   (lyskom-server-call
+    (let ((mship (lyskom-try-get-membership conf-no t)))
+      (when mship
+        (set-membership->last-time-read mship (lyskom-current-client-time))))
     (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-void)
     (lyskom-send-packet kom-queue (lyskom-format-objects 2 conf-no))))
 
@@ -193,18 +196,31 @@ Args: KOM-QUEUE HANDLER PERS-NO OLD-PW NEW-PW &rest DATA."
 ;;; by another call.
 ;;;
 (defun initiate-query-read-texts (kom-queue handler
-					    pers-no conf-no &rest data)
+					    pers-no 
+                                            conf-no
+                                            want-read-ranges
+                                            max-ranges
+                                            &rest data)
   "Get a membership struct describing the membership of PERS-NO in CONF-NO.
 Args: KOM-QUEUE HANDLER PERS-NO CONF-NO &rest DATA"
   (lyskom-server-call
-    (lyskom-call kom-queue lyskom-ref-no handler data 
-                 (if (lyskom-have-call 98)
-                     'lyskom-parse-membership
-                   'lyskom-parse-membership-old))
-    (lyskom-send-packet kom-queue 
-                        (lyskom-format-objects
-                         (if (lyskom-have-call 98) 98 9)
-                         pers-no conf-no))))
+    (cond ((lyskom-have-call 107)
+           (lyskom-call kom-queue lyskom-ref-no handler data 
+                        'lyskom-parse-membership-11)
+           (lyskom-send-packet kom-queue
+                               (lyskom-format-objects 107 pers-no conf-no
+                                                      want-read-ranges
+                                                      max-ranges)))
+          ((lyskom-have-call 98)
+           (lyskom-call kom-queue lyskom-ref-no handler data 
+                        'lyskom-parse-membership-10)
+           (lyskom-send-packet kom-queue
+                               (lyskom-format-objects 98 pers-no conf-no)))
+          (t
+           (lyskom-call kom-queue lyskom-ref-no handler data 
+                        'lyskom-parse-membership-old)
+           (lyskom-send-packet kom-queue
+                               (lyskom-format-objects 9 pers-no conf-no))))))
 
 
 (defun initiate-create-conf (kom-queue handler
@@ -395,7 +411,7 @@ Args: KOM-QUEUE HANDLER TEXT-NO &rest DATA."
                                                              0 lyskom-max-int)))
        (t
                                         ;Cached info. 
-        (lyskom-call-add kom-queue 'PARSED text handler data)
+        (lyskom-call-add kom-queue 'PARSED (lyskom-ref-no) text handler data)
         (lyskom-check-call kom-queue))))))
 
 
@@ -421,7 +437,7 @@ Args: KOM-QUEUE HANDLER TEXT-NO &rest DATA."
                             text-no)))
       (t
                                         ;Cached info. 
-       (lyskom-call-add kom-queue 'PARSED text-stat handler data)
+       (lyskom-call-add kom-queue 'PARSED (lyskom-ref-no) text-stat handler data)
        (lyskom-check-call kom-queue))))))
 
 
@@ -672,15 +688,24 @@ Args: KOM-QUEUE HANDLER MESSAGE &rest DATA."
   "Get membership-list for PERS-NO from server.
 Args: KOM-QUEUE HANDLER PERS-NO &rest DATA."
   (lyskom-server-call
-    (lyskom-call kom-queue lyskom-ref-no handler data
-                 (if (lyskom-have-call 99)
-                     'lyskom-parse-membership-list
-                   'lyskom-parse-membership-list-old))
-    (lyskom-send-packet kom-queue (lyskom-format-objects
-                                   (if (lyskom-have-call 99) 99 46)
-                                   pers-no
-                                   0 lyskom-max-int ;all confs.
-                                   1)))) ;want read texts.
+    (cond ((lyskom-have-call 108)
+           (lyskom-call kom-queue lyskom-ref-no handler data
+                        'lyskom-parse-membership-list-11)
+           (lyskom-send-packet 
+            kom-queue 
+            (lyskom-format-objects 108 pers-no 0 lyskom-max-int
+                                   1 lyskom-max-int)))
+          ((lyskom-have-call 99)
+           (lyskom-call kom-queue lyskom-ref-no handler data
+                        'lyskom-parse-membership-list-10)
+           (lyskom-send-packet 
+            kom-queue 
+            (lyskom-format-objects 99 pers-no 0 lyskom-max-int 1)))
+          (t (lyskom-call kom-queue lyskom-ref-no handler data
+                          'lyskom-parse-membership-list-old)
+             (lyskom-send-packet 
+              kom-queue 
+              (lyskom-format-objects 46 pers-no 0 lyskom-max-int 1))))))
 
 
 (defun initiate-get-part-of-membership (kom-queue handler pers-no first length
@@ -688,15 +713,24 @@ Args: KOM-QUEUE HANDLER PERS-NO &rest DATA."
   "Get membership-list for PERS-NO from server.
 Args: KOM-QUEUE HANDLER PERS-NO FIRST-IN-LIST LENGHT &rest DATA."
   (lyskom-server-call
-    (lyskom-call kom-queue lyskom-ref-no handler data
-                 (if (lyskom-have-call 99)
-                     'lyskom-parse-membership-list
-                   'lyskom-parse-membership-list-old))
-    (lyskom-send-packet kom-queue (lyskom-format-objects 
-                                   (if (lyskom-have-call 99) 99 46)
-                                   pers-no
-                                   first length ;all confs.
-                                   1))))
+    (cond ((lyskom-have-call 108)
+           (lyskom-call kom-queue lyskom-ref-no handler data
+                        'lyskom-parse-membership-list-11)
+           (lyskom-send-packet 
+            kom-queue 
+            (lyskom-format-objects 108 pers-no first length
+                                   1 1)))
+          ((lyskom-have-call 99)
+           (lyskom-call kom-queue lyskom-ref-no handler data
+                        'lyskom-parse-membership-list-10)
+           (lyskom-send-packet 
+            kom-queue 
+            (lyskom-format-objects 99 pers-no first length 0)))
+          (t (lyskom-call kom-queue lyskom-ref-no handler data
+                          'lyskom-parse-membership-list-old)
+             (lyskom-send-packet 
+              kom-queue 
+              (lyskom-format-objects 46 pers-no first length 0))))))
 
 
 (defun initiate-get-created-texts (kom-queue handler pers-no first-local
@@ -744,7 +778,7 @@ Args: KOM-QUEUE HANDLER PERS-NO &rest DATA."
         (lyskom-send-packet kom-queue (lyskom-format-objects 49 pers-no)))
        (t
                                         ;Cached info. 
-        (lyskom-call-add kom-queue 'PARSED pers-stat handler data)
+        (lyskom-call-add kom-queue 'PARSED (lyskom-ref-no) pers-stat handler data)
         (lyskom-check-call kom-queue))))))
 
 (defun initiate-get-conf-stat (kom-queue handler conf-no &rest data)
@@ -754,7 +788,7 @@ Args: KOM-QUEUE HANDLER CONF-NO &rest DATA."
    (let ((conf-stat (cache-get-conf-stat conf-no)))
      (cond
       ((zerop conf-no)			;No real user.
-       (lyskom-call-add kom-queue 'PARSED nil handler data)
+       (lyskom-call-add kom-queue 'PARSED (lyskom-ref-no) nil handler data)
        (lyskom-check-call kom-queue))
       ((null conf-stat)			;Cached info?
        (lyskom-call kom-queue		;No, ask the server.
@@ -772,7 +806,7 @@ Args: KOM-QUEUE HANDLER CONF-NO &rest DATA."
                             conf-no)))
       (t
                                         ;Cached info. 
-       (lyskom-call-add kom-queue 'PARSED conf-stat handler data)
+       (lyskom-call-add kom-queue 'PARSED (lyskom-ref-no) conf-stat handler data)
        (lyskom-check-call kom-queue))))))
 
 
@@ -837,12 +871,17 @@ Args: KOM-QUEUE HANDLER PERS-NO TEXT-NO &rest DATA."
     (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-void)
     (lyskom-send-packet kom-queue (lyskom-format-objects 57 pers-no text-no))))
 
-(defun initiate-get-last-text (kom-queue handler before &rest data)
-  "Get text created before BEFORE.
-Args: KOM-QUEUE HANDLER BEFORE &rest DATA"
-  (lyskom-server-call
-    (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-num)
-    (lyskom-send-packet kom-queue (lyskom-format-objects 58 before))))
+
+;; WARNING: If you start using this you have to figure out a way to
+;; convert local time (before) to UTC. Since this doesn't work at the
+;; moment, I have commented this functoun into oblivion.
+;;
+;; (defun initiate-get-last-text (kom-queue handler before &rest data)
+;;   "Get text created before BEFORE.
+;; Args: KOM-QUEUE HANDLER BEFORE &rest DATA"
+;;   (lyskom-server-call
+;;     (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-num)
+;;     (lyskom-send-packet kom-queue (lyskom-format-objects 58 before))))
 
 
 (defun initiate-create-anonymous-text (kom-queue 
@@ -1072,7 +1111,7 @@ Args: KOM-QUEUE HANDLER CONF-NO &rest DATA."
   (lyskom-server-call
     (let ((conf-stat (cache-get-uconf-stat conf-no)))
       (cond ((zerop conf-no)
-             (lyskom-call-add kom-queue 'PARSED nil handler data)
+             (lyskom-call-add kom-queue 'PARSED (lyskom-ref-no) nil handler data)
              (lyskom-check-call kom-queue))
             ((null conf-stat)
              (lyskom-call kom-queue
@@ -1081,7 +1120,7 @@ Args: KOM-QUEUE HANDLER CONF-NO &rest DATA."
                           'lyskom-parse-uconf-stat conf-no)
              (lyskom-send-packet kom-queue (lyskom-format-objects 78 conf-no)))
             (t
-             (lyskom-call-add kom-queue 'PARSED conf-stat handler data)
+             (lyskom-call-add kom-queue 'PARSED (lyskom-ref-no) conf-stat handler data)
              (lyskom-check-call kom-queue))))))
 
 (defun initiate-set-info (kom-queue handler 
@@ -1154,7 +1193,7 @@ Args: KOM-QUEUE HANDLER SESSION-NO &rest DATA"
        (lyskom-send-packet kom-queue (lyskom-format-objects
                                       84 session-no)))
       (t                                ; Cached
-       (lyskom-call-add kom-queue 'PARSED info handler data)
+       (lyskom-call-add kom-queue 'PARSED (lyskom-ref-no) info handler data)
        (lyskom-check-call kom-queue)))))) ;This might call the handler.
 
 
@@ -1228,13 +1267,33 @@ Args: KOM-QUEUE HANDLER SESSION-NO &rest DATA"
                                            no-of-texts &rest data)
   "Send local-to-global to server."
   (lyskom-server-call
-    (lyskom-call kom-queue lyskom-ref-no handler data
-                 'lyskom-parse-text-mapping no-of-texts)
-    (lyskom-send-packet kom-queue
-                        (lyskom-format-objects 103 
-                                               conf-no
-                                               first-local-no
-                                               no-of-texts))))
+    (cond ((lyskom-have-call 103)
+           (lyskom-call kom-queue lyskom-ref-no handler data
+                        'lyskom-parse-text-mapping no-of-texts)
+           (lyskom-send-packet kom-queue
+                               (lyskom-format-objects 103 
+                                                      conf-no
+                                                      first-local-no
+                                                      no-of-texts)))
+          (t (lyskom-call kom-queue lyskom-ref-no 
+                          'lyskom-l2g-fake-callback
+                          (cons handler data)
+                          'lyskom-parse-map)
+                 (lyskom-send-packet kom-queue 
+                                     (lyskom-format-objects 34 conf-no
+                                                            first-local-no
+                                                            no-of-texts))))))
+
+(defun lyskom-l2g-fake-callback (map handler &rest data)
+  (let ((mapping 
+         (and map (lyskom-create-text-mapping (map->first-local map)
+                                              (+ (map->first-local map)
+                                                 (length (map->text-nos map)))
+                                              (length (map->text-nos map))
+                                              t
+                                              'dense
+                                              map))))
+    (apply handler mapping data)))
 
 (defun initiate-map-created-texts (kom-queue handler
                                              author
@@ -1265,6 +1324,68 @@ Args: KOM-QUEUE HANDLER SESSION-NO &rest DATA"
     (lyskom-send-packet kom-queue (lyskom-format-objects 106 pers-no flags))
     (cache-del-pers-stat pers-no)))
 
+(defun initiate-mark-as-unread (kom-queue handler conf-no text &rest data)
+  (lyskom-server-call
+    (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-void)
+    (lyskom-send-packet kom-queue (lyskom-format-objects 109 conf-no text))))
+
+(defun initiate-get-stats-description (kom-queue handler &rest data)
+  (lyskom-server-call
+    (if lyskom-stats-description
+        (progn (lyskom-call-add kom-queue 'PARSED (lyskom-ref-no) lyskom-stats-description
+                                handler data)
+          (lyskom-check-call kom-queue))
+      (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-stats-description)
+      (lyskom-send-packet kom-queue (lyskom-format-objects 111)))))
+
+(defun initiate-get-stats (kom-queue handler what &rest data)
+  (lyskom-server-call
+    (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-stats-array)
+    (lyskom-send-packet kom-queue (lyskom-format-objects 112 what))))
+
+(defun initiate-get-boottime-info (kom-queue handler &rest data)
+  (lyskom-server-call
+    (if lyskom-static-server-info
+        (progn (lyskom-call-add kom-queue 'PARSED (lyskom-ref-no) lyskom-static-server-info
+                                handler data)
+          (lyskom-check-call kom-queue))
+      (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-static-server-info)
+      (lyskom-send-packet kom-queue (lyskom-format-objects 113)))))
+
+(defun initiate-first-unused-conf-no (kom-queue handler &rest data)
+  (lyskom-server-call
+    (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-num)
+    (lyskom-send-packet kom-queue (lyskom-format-objects 114))))
+
+(defun initiate-first-unused-text-no (kom-queue handler &rest data)
+  (lyskom-server-call
+    (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-num)
+    (lyskom-send-packet kom-queue (lyskom-format-objects 115))))
+
+(defun initiate-find-next-conf-no (kom-queue handler conf-no &rest data)
+  (lyskom-server-call
+    (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-num)
+    (lyskom-send-packet kom-queue (lyskom-format-objects 116 conf-no))))
+
+(defun initiate-find-previous-conf-no (kom-queue handler conf-no &rest data)
+  (lyskom-server-call
+    (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-num)
+    (lyskom-send-packet kom-queue (lyskom-format-objects 117 conf-no))))
+
+(defun initiate-get-scheduling (kom-queue handler session-no &rest data)
+  (lyskom-server-call
+    (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-scheduling-info)
+    (lyskom-send-packet kom-queue (lyskom-format-objects 118 (or session-no 0)))))
+
+(defun initiate-set-scheduling (kom-queue handler session-no priority weight &rest data)
+  (lyskom-server-call
+    (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-void)
+    (lyskom-send-packet kom-queue (lyskom-format-objects 119 (or session-no 0) priority weight))))
+
+(defun initiate-set-connection-time-format (kom-queue handler use-utc &rest data)
+  (lyskom-server-call
+    (lyskom-call kom-queue lyskom-ref-no handler data 'lyskom-parse-void)
+    (lyskom-send-packet kom-queue (lyskom-format-objects 120 use-utc))))
 
 
 ;;; ================================================================
