@@ -1,6 +1,6 @@
 ;;;;; -*-coding: iso-8859-1;-*-
 ;;;;;
-;;;;; $Id: startup.el,v 44.82 2002/10/20 13:20:00 byers Exp $
+;;;;; $Id: startup.el,v 44.111 2005/01/09 01:16:02 byers Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -36,7 +36,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: startup.el,v 44.82 2002/10/20 13:20:00 byers Exp $\n"))
+	      "$Id: startup.el,v 44.111 2005/01/09 01:16:02 byers Exp $\n"))
 
 
 ;;; ================================================================
@@ -82,7 +82,8 @@ clients of the event. See lyskom-mode for details on lyskom."
   (interactive (list (lyskom-read-server-name
 		      (lyskom-format 'server-q
 				     (or (getenv "KOMSERVER")
-					 lyskom-default-server)))
+					 lyskom-default-server
+					 kom-default-server)))
 		     nil
 		     nil
 		     (if current-prefix-arg
@@ -92,15 +93,27 @@ clients of the event. See lyskom-mode for details on lyskom."
 
   (run-hooks 'lyskom-init-hook)
   (setq username
-	(or username (getenv "KOMNAME")))
+	(or username 
+	    lyskom-default-user-name
+	    kom-default-user-name
+	    (getenv "KOMNAME")))
   (setq password
-	(or password (getenv "KOMPASSWORD")))
+	(or password
+	    (when (and lyskom-default-password ; Got default password?
+		       (or (not lyskom-server-name) ; For this host?
+			   (string= lyskom-server-name
+				    host)))
+	      lyskom-default-password)
+	    kom-default-password
+	    (getenv "KOMPASSWORD")))
   (if (zerop (length host))
       (let* ((env-kom (getenv "KOMSERVER"))
-	     (canon (lyskom-string-rassoc env-kom kom-server-aliases)))
+	     (canon (or (lyskom-string-rassoc env-kom kom-server-aliases)
+                        (lyskom-string-rassoc env-kom kom-builtin-server-aliases))))
 	(setq host (or (car canon)
 		       env-kom
-		       lyskom-default-server))))
+		       lyskom-default-server
+		       kom-default-server))))
   (let ((port 4894)
 	(init-done nil))
     (cond				;Allow "nanny:4892" to use port 4892.
@@ -108,7 +121,7 @@ clients of the event. See lyskom-mode for details on lyskom."
       (setq port (string-to-int (substring host (match-end 0))))
       (cond
        ((zerop (match-beginning 0))
-	(setq host lyskom-default-server))
+	(setq host (or lyskom-default-server kom-default-server)))
        (t
 	(setq host (substring host 0 (match-beginning 0)))))))
 
@@ -194,7 +207,7 @@ clients of the event. See lyskom-mode for details on lyskom."
                                                               www-proxy-host
                                                               www-proxy-port))
                        ;; We do explicit coding
-                       (set-process-coding-system proc 'no-conversion 'no-conversion)
+                       (lyskom-set-process-coding-system proc 'no-conversion 'no-conversion)
 
 		       ;; Install our filter.
 		       ;; Do this before we send the CONNECT command to
@@ -202,7 +215,7 @@ clients of the event. See lyskom-mode for details on lyskom."
 		       (setq lyskom-www-proxy-connect-phase 1)
 		       (set-process-filter proc
 					   'lyskom-www-proxy-connect-filter)
-		       (process-kill-without-query proc nil)
+		       (lyskom-set-process-query-on-exit-flag proc nil)
 
                        (lyskom-process-send-string 
                         proc
@@ -230,7 +243,7 @@ clients of the event. See lyskom-mode for details on lyskom."
                       (t (setq proc (lyskom-open-network-stream name buffer
                                                                 host port))
                          ;; We do explicit coding
-                         (set-process-coding-system proc 'no-conversion 'no-conversion))))
+                         (lyskom-set-process-coding-system proc 'no-conversion 'no-conversion))))
 	      (switch-to-buffer buffer)
 	      (lyskom-mode)		;Clearing lyskom-default...
 	      (setq lyskom-buffer buffer)
@@ -239,12 +252,12 @@ clients of the event. See lyskom-mode for details on lyskom."
 	      (setq lyskom-server-name host)
 	      (setq lyskom-server-port port)
 	      (setq lyskom-proc proc)
-	      (lyskom-setup-faces)
+	      (lyskom-setup-faces-for-buffer)
               (lyskom-check-configuration)
 	      (lyskom-insert
 	       (lyskom-format 'try-connect lyskom-clientversion host))
 	      (set-process-filter proc 'lyskom-connect-filter)
-              (process-kill-without-query proc nil)
+              (lyskom-set-process-query-on-exit-flag proc nil)
 	      (lyskom-process-send-string
 	       proc
 	       (concat "A"
@@ -260,6 +273,9 @@ clients of the event. See lyskom-mode for details on lyskom."
 	      (save-excursion
 		(lyskom-init-parse buffer))
 
+              ;; Set up timestamps and stuff
+              (lyskom-set-connection-time-format t)
+
               ;; Async messages
               (lyskom-accept-async)
 
@@ -307,14 +323,14 @@ clients of the event. See lyskom-mode for details on lyskom."
                              (make-string kom-text-header-dash-length ?-)
                            "")
                          (when kom-highlight-dashed-lines
-                           '(face kom-dashed-lines-face)))
+                           `(face ,(or kom-dashed-lines-face
+                                       lyskom-default-dashed-lines-face))))
 
                         (lyskom-format-insert "%#2$%#1t\n"
                                               str
                                               (when kom-highlight-text-body
-                                                '(face kom-text-body-face))
-                                              (when kom-highlight-text-body
-                                                '(face kom-text-body-face)))
+                                                `(face ,(or kom-text-body-face
+                                                            lyskom-default-text-body-face))))
 
                         (lyskom-format-insert
                          "%#2$%#1s\n"
@@ -329,7 +345,8 @@ clients of the event. See lyskom-mode for details on lyskom."
                           kom-text-footer-format
                           lyskom-last-text-format-flags)
                          (when kom-highlight-dashed-lines
-                           '(face kom-dashed-lines-face)))
+                           `(face ,(or kom-dashed-lines-face
+                                       lyskom-default-dashed-lines-face))))
                         ))))
 
 	      ;; Can't use lyskom-end-of-command here.
@@ -354,10 +371,11 @@ clients of the event. See lyskom-mode for details on lyskom."
 
 
 (defun lyskom-accept-async ()
-  (blocking-do 'accept-async '(5 7 8 9 11 12 13 14 15 16 17 18))
+  (blocking-do 'accept-async '(5 7 8 9 11 12 13 14 15 16 17 18 19 20 21 22))
   (let* ((ans (blocking-do 'query-async)))
     (unless (memq 15 (listify-vector ans))
-      (blocking-do 'accept-async '(0 5 7 8 9 11 12 13 14 16 17 18)))))
+      ;; async-new-text is not implemented, so use async-new-text-old
+      (blocking-do 'accept-async '(0 5 7 8 9 11 12 13 14 16 17 18 19 20 21 22)))))
 
 
 (defun lyskom-www-proxy-connect-filter (proc output)
@@ -427,12 +445,12 @@ clients of the event. See lyskom-mode for details on lyskom."
                       (setq proc (start-process
                                   procname
                                   bufname
-                                  "ssh" "-n" "-x"
+                                  kom-ssh-command "-n" "-x"
                                   "-L" (format "%d:%s:%d" relay-port server port)
                                   kom-ssh-relay-host
                                   "sh -c \"while :; do echo ok; sleep 600; done\"")))
                   (setenv "LC_ALL" old-lc-all)))
-              (process-kill-without-query proc))
+              (lyskom-set-process-query-on-exit-flag proc nil))
             (while (progn
                      (goto-char (point-max))
                      (re-search-backward "^--- .* ---$" nil t)
@@ -472,18 +490,23 @@ clients of the event. See lyskom-mode for details on lyskom."
   (let ((protocol-version 
          (version-info->protocol-version lyskom-server-version-info)))
 
+  (when (>= protocol-version 8)
+    (lyskom-set-feature long-conf-types t))
+
+  (when (>= protocol-version 9)
+    (lyskom-set-feature dynamic-session-info t)
+    (lyskom-set-feature idle-time t))
+
   (when (>= protocol-version 10)
     (lyskom-set-feature bcc-misc t)
     (lyskom-set-feature aux-items t)
     (lyskom-set-feature highest-call 105)
     (lyskom-set-feature local-to-global t))
 
-  (when (>= protocol-version 9)
-    (lyskom-set-feature dynamic-session-info t)
-    (lyskom-set-feature idle-time t))
-
-  (when (>= protocol-version 8)
-    (lyskom-set-feature long-conf-types t))))
+  (when (>= protocol-version 11)
+    (lyskom-set-feature read-ranges t)
+    (lyskom-set-feature highest-call 110))
+  ))
 
 
 (defun lyskom-connect-filter (proc output)
@@ -528,17 +551,17 @@ shown to other users."
                        lyskom-default-user-name)
                   ;; This is nil if we can't find a unique match.
                   (setq new-me
-                        (conf-z-info->conf-no
+			(if (integerp lyskom-default-user-name)
+			    lyskom-default-user-name
+			(conf-z-info->conf-no
                          (lyskom-lookup-conf-by-name lyskom-default-user-name
-                                                     '(pers)))))
+                                                     '(pers))))))
               (if new-me
                   nil
                 (let ((name nil))
-                  (while (or (null name)
-                             (string= name ""))
-                    (setq name (lyskom-read-conf-name
-                                (lyskom-get-string 'what-is-your-name) 
-                                '(pers none) t "" t)))
+                  (while (or (null name) (string= name ""))
+                    (setq name (lyskom-read-conf-name 'what-is-your-name
+                                                      '(pers none) nil nil t)))
                   (setq new-me
                         (or (conf-z-info->conf-no 
                              (lyskom-lookup-conf-by-name name '(pers)))
@@ -566,19 +589,16 @@ shown to other users."
                     (lyskom-insert (concat (conf-stat->name conf-stat) "\n"))
                     (setq lyskom-first-time-around nil)
                     (if (blocking-do 'login new-me
-                                     (if lyskom-default-password
-                                         (prog1
-                                             lyskom-default-password
-                                           (setq lyskom-default-password nil)
-                                           (set-default 'lyskom-default-password
-                                                        nil))
-                                       ;; Use password read when creating
-                                       ;; the person when loggin in new
-                                       ;; users
-                                       (or lyskom-is-new-user
-                                           (silent-read
-                                            (lyskom-get-string 'password))))
-                                     (if invisiblep 1 0))
+                                     (setq lyskom-default-password
+					   (if lyskom-default-password
+					       lyskom-default-password
+					     ;; Use password read when creating
+					     ;; the person when loggin in new
+					     ;; users
+					     (or lyskom-is-new-user
+						 (silent-read
+						  (lyskom-get-string 'password)))))
+					   (if invisiblep 1 0))
                         (progn
                           (if lyskom-is-new-user
                               (blocking-do 'add-member
@@ -588,8 +608,9 @@ shown to other users."
                                            1
                                            (lyskom-create-membership-type
                                             nil nil nil nil nil nil nil nil)))
-                          (setq login-successful t))
+			  (setq login-successful t))
                       (lyskom-insert-string 'wrong-password)
+		      (setq lyskom-default-password nil)
                       (when (lyskom-get-aux-item 
                              (server-info->aux-item-list lyskom-server-info)
                              13)        ; e-mail
@@ -606,6 +627,7 @@ shown to other users."
 
             ;; Now we are logged in.
             (setq lyskom-pers-no new-me)
+            (setq lyskom-is-administrator nil)
             (lyskom-insert-string 'are-logged-in)
 
             (unless lyskom-is-running-compiled
@@ -613,6 +635,14 @@ shown to other users."
 
             (unless lyskom-dont-read-user-area
               (setq ignored-user-area-vars (lyskom-read-options)))
+
+	    (unless kom-remember-password
+		(setq lyskom-default-password nil))
+
+	    ;Update mode-line string if needed (as early as possible).
+
+	    (lyskom-mode-name-from-host) 
+	    
             (when (or session-priority kom-default-session-priority)
               (setq lyskom-session-priority
                     (or session-priority kom-default-session-priority)))
@@ -623,7 +653,6 @@ shown to other users."
                                        lyskom-current-conf
                                        0)
             (setq lyskom-current-conf 0)
-            ;; (cache-initiate-who-info-buffer (blocking-do 'who-is-on))
             (cache-set-marked-texts (blocking-do 'get-marks))
             ;; What is this variable? It is never used. It is ust to
             ;; fill the cache?
@@ -633,24 +662,40 @@ shown to other users."
           ;; If login succeeded, clear the caches and set the language
 
           (when login-successful
-            (progn (clear-all-caches)
-                   (unless (eq lyskom-language kom-default-language)   
-                     (when (lyskom-set-language kom-default-language 'local)
-                       (unless lyskom-have-one-login
-                         (lyskom-set-language kom-default-language 'global)
-                         (lyskom-maybe-setq-default kom-default-language kom-default-language)
-                         (setq-default lyskom-language kom-default-language))
-                       (lyskom-format-insert-before-prompt
-                        'language-set-to
-                        (lyskom-language-name kom-default-language))))
-                   (setq lyskom-have-one-login t)))
+            (clear-all-caches)
+            (lyskom-init-membership)
+            (when (lyskom-set-language (lyskom-default-language) 'local)
+              (unless lyskom-have-one-login
+                (lyskom-set-language (lyskom-default-language) 'global)
+                (setq-default lyskom-language lyskom-language)
+                (setq-default kom-default-language (list lyskom-language)))
+              (lyskom-format-insert-before-prompt
+               'language-set-to
+               (lyskom-language-name (lyskom-default-language))
+               "Help on changing languages" 
+                `(face underline
+                       mouse-face highlight
+                       lyskom-button t
+                       lyskom-button-text ""
+                       lyskom-button-type func
+                       lyskom-buffer ,(current-buffer)
+                       lyskom-button-arg 
+                       (kom-help ("Changing Languages" . language-help))))
+
+              )
+            (setq lyskom-have-one-login t))
 
           (when ignored-user-area-vars
             (lyskom-format-insert-before-prompt
              'ignored-user-area-var
-             (mapconcat 'symbol-name 
+             (mapconcat (lambda (x) (symbol-name (car x)))
                         ignored-user-area-vars
-                        "\n    ")))
+                        "\n    ")
+             (lyskom-string= lyskom-clientversion lyskom-settings-version)
+             lyskom-settings-version
+             lyskom-clientversion
+             72)
+            (setq lyskom-saved-unknown-variables ignored-user-area-vars))
 
           ;; Show motd and encourage writing a presentation
 
@@ -660,10 +705,7 @@ shown to other users."
                 (progn
                   (lyskom-insert-string 'you-have-motd)
                   (lyskom-view-text (conf-stat->msg-of-day conf-stat))))
-            (if (and conf-stat
-                     (zerop (conf-stat->presentation conf-stat))
-                     (not (zerop (conf-stat->no-of-texts conf-stat))))
-                (lyskom-insert-string 'presentation-encouragement)))
+            (lyskom-nag-about-presentation t))
 
           (setq lyskom-is-new-user nil)
 
@@ -685,25 +727,20 @@ shown to other users."
 (defun lyskom-refetch ()
   "Resets and fetches all reading info.
 This is called at login and after prioritize and set-unread."
-  ;; +++PREFETCH+++
 
   ;; The whole membership!
-  ;; (lyskom-set-membership (blocking-do 'get-membership lyskom-pers-no))
-  ;; (setq lyskom-membership-is-read t)
-  ;; (setq lyskom-unread-confs (blocking-do 'get-unread-confs lyskom-pers-no))
-
-  (setq lyskom-membership nil)
 
+  (lyskom-init-membership)
   (setq lyskom-to-do-list (lyskom-create-read-list))
   (setq lyskom-reading-list (lyskom-create-read-list))
 
 
   (lyskom-reset-prefetch)
   (let ((lyskom-inhibit-prefetch t))
-    (lyskom-prefetch-membership lyskom-pers-no)
     (let ((unreads (blocking-do 'get-unread-confs lyskom-pers-no)))
-      (lyskom-traverse conf-no (nreverse (conf-no-list->conf-nos unreads))
-        (lyskom-prefetch-one-membership conf-no lyskom-pers-no))))
+      (lyskom-traverse conf-no (conf-no-list->conf-nos unreads)
+        (lyskom-prefetch-one-membership conf-no lyskom-pers-no)))
+    (lyskom-prefetch-membership lyskom-pers-no))
   (lyskom-start-prefetch)
 
   (condition-case nil
@@ -715,52 +752,6 @@ This is called at login and after prioritize and set-unread."
     (error nil)
     (quit nil)))
 
-
-(defun lyskom-set-membership (membership)
-  "Sets lyskom-membership to a new value.
-Args: MEMBERSHIP."
-  (setq lyskom-membership (listify-vector membership))
-  (lyskom-sort-membership)
-  (setq lyskom-membership-is-read t))
-
-
-(defun lyskom-print-name (conf-stat)
-  "Print the name of the CONF-STAT, with a trailing \n."
-  (lyskom-insert (concat (conf-stat->name conf-stat) "\n")))
-
-
-(defun lyskom-extract-persons (conf-list)
-  "Extract persons from a conf-list.
-Return a list of pers-nos of all conferences that are persons.
-Args: CONF-LIST."
-  (lyskom-do-extract-persons-or-confs conf-list t))
-
-
-(defun lyskom-extract-confs (conf-list)
-  "Extract conferences from a conf-list.
-Return a list of conf-nos of all conferences that are persons.
-Args: CONF-LIST."
-  (lyskom-do-extract-persons-or-confs conf-list nil))
-
-
-(defun lyskom-do-extract-persons-or-confs (conf-list want-persons)
-  "Extract persons or conferences from CONF-LIST.
-WANT-PERSONS is t for persons, nil for confs."
-  (let* ((result nil)
-	 (i 0)
-	 (nos (conf-list->conf-nos conf-list))
-	 (types (conf-list->conf-types conf-list))
-	 (len (length nos)))
-    (while (< i len)
-      (cond
-       ((eq (conf-type->letterbox (elt types i))
-	    want-persons)
-	(setq result (cons (elt nos i)
-			   result))))
-      (++ i))
-    (nreverse result)))
-
-
 (defun lyskom-create-new-person (name)
   "A new user named NAME (or an old that mis-spelled his name)."
   (lyskom-insert
@@ -799,40 +790,21 @@ WANT-PERSONS is t for persons, nil for confs."
     nil)))
 
 
-;;(defun lyskom-start-anew-create-handler (pers-no name password)
-;;  "A new person has been created. Log in as him."
-;;  (cond
-;;   ((null pers-no)
-;;    (lyskom-insert-string 'could-not-create-you)
-;;    (setq lyskom-executing-command nil)
-;;    (kom-start-anew))
-;;   (t
-;;    (initiate-login 'main 'lyskom-add-for-new-person
-;;		    pers-no password pers-no lyskom-pers-no)
-;;    )))
-
-;;(defun lyskom-add-for-new-person (reply pers-no lyskom-pers-no)
-;;  "Add a news person as member in the default presentation conference."
-;;  (initiate-add-member 'main 'lyskom-start-anew-login-2
-;;		       (server-info->conf-pres-conf lyskom-server-info)
-;;		       pers-no 100 1
-;;		       pers-no lyskom-pers-no))
-
-
 (defun lyskom-read-server-name (prompt)
   "Read the name of a LysKOM server.
-Copmpletion is done on the servers i kom-server-aliases. If an
-alias name is entered, the corresponding address is returned."
+Copmpletion is done on the servers i kom-server-aliases and
+kom-builtin-server-aliases. If an alias name is entered, the
+corresponding address is returned."
   ;; Create a completion table like
   ;; (("kom.lysator.liu.se" . "kom.lysator.liu.se")
   ;;  ("LysKOM" . "kom.lysator.liu.se"))
   (let ((known-servers
 	 (append (mapcar (function (lambda (pair)
 				     (cons (car pair) (car pair))))
-			 kom-server-aliases)
+			 (append kom-server-aliases kom-builtin-server-aliases))
 		 (mapcar (function (lambda (pair)
 				     (cons (cdr pair) (car pair))))
-			 kom-server-aliases)))
+			 (append kom-server-aliases kom-builtin-server-aliases))))
 	(completion-ignore-case t)
 	server)
     (setq server (lyskom-completing-read prompt
@@ -937,8 +909,10 @@ Functions and variables beginning with lyskom- are not intended for the user
 to see, set of call."
   (interactive)
   (lyskom-clear-vars)
+  (lyskom-mode-name-from-host)
   (setq mode-line-buffer-identification
-	(list (concat (lyskom-mode-name-from-host) ": ") 'mode-line-conf-name))
+	; Must start with a string to be handled properly, it seems.
+	(list "" 'mode-line-server-name  ": " 'mode-line-conf-name))
 
   (setq major-mode 'lyskom-mode)
   (setq mode-name "LysKOM")
@@ -954,8 +928,6 @@ to see, set of call."
   "Set up buffer-local vars."
   (lyskom-save-variables (lyskom-proc
                           lyskom-pers-no
-                          lyskom-membership
-                          lyskom-membership-is-read
                           lyskom-last-viewed
                           lyskom-unparsed-buffer
                           lyskom-unparsed-marker
