@@ -1,6 +1,6 @@
 ;;;;; -*-coding: raw-text;-*-
 ;;;;;
-;;;;; $Id: view-text.el,v 44.68 2002/06/12 18:37:44 byers Exp $
+;;;;; $Id: view-text.el,v 44.82 2005/01/11 15:00:14 _cvs_pont_lyskomelisp Exp $
 ;;;;; Copyright (C) 1991-2002  Lysator Academic Computer Association.
 ;;;;;
 ;;;;; This file is part of the LysKOM Emacs LISP client.
@@ -35,7 +35,7 @@
 
 (setq lyskom-clientversion-long 
       (concat lyskom-clientversion-long
-	      "$Id: view-text.el,v 44.68 2002/06/12 18:37:44 byers Exp $\n"))
+	      "$Id: view-text.el,v 44.82 2005/01/11 15:00:14 _cvs_pont_lyskomelisp Exp $\n"))
 
 
 (defvar lyskom-view-text-text)
@@ -67,7 +67,11 @@ Note that this function must not be called asynchronously."
         (start nil)
         (todo nil)
         (lyskom-last-text-format-flags nil))
-    (cond ((eq filter 'skip-text) (lyskom-filter-prompt text-no 'filter-text)
+    (cond ((eq filter 'obliterate)
+	   (setq todo 'next-text)
+	   (lyskom-mark-as-read (blocking-do 'get-text-stat text-no))
+	   'next-text)
+	  ((eq filter 'skip-text) (lyskom-filter-prompt text-no 'filter-text)
 	   (setq todo 'next-text)
 	   (lyskom-mark-as-read (blocking-do 'get-text-stat text-no))
 	   'next-text)
@@ -109,6 +113,7 @@ Note that this function must not be called asynchronously."
                          (mx-author (car (lyskom-get-aux-item (text-stat->aux-items text-stat) 16)))
                          (mx-to (lyskom-get-aux-item (text-stat->aux-items text-stat) 19))
                          (mx-cc (lyskom-get-aux-item (text-stat->aux-items text-stat) 20))
+			 (mx-message-id (lyskom-get-aux-item (text-stat->aux-items text-stat) 22))
                          (mx-filename (lyskom-get-aux-item (text-stat->aux-items text-stat) 10104))
                          (mx-sender (lyskom-get-aux-item (text-stat->aux-items text-stat) 10103))
 			 (mx-headers (lyskom-get-aux-item (text-stat->aux-items text-stat) 24))
@@ -138,7 +143,7 @@ Note that this function must not be called asynchronously."
                     (when (eq filter 'dontshow)
                       (lyskom-get-string 'filtered))
                     (when kom-highlight-first-line
-                      '(face kom-first-line-face)))
+                      `(face ,kom-first-line-face)))
 
 		   (set-marker start nil)
 
@@ -150,6 +155,9 @@ Note that this function must not be called asynchronously."
                    (when mx-filename
                      (lyskom-format-insert 'attachment-filename
                                            (aux-item->data (car mx-filename))))
+                   (when (and mx-message-id kom-show-imported-message-id)
+                     (lyskom-format-insert 'message-id
+                                           (aux-item->data (car mx-message-id))))
 
                    ;; Insert imported at
 
@@ -345,6 +353,20 @@ Note that this function must not be called asynchronously."
 		   (if kom-reading-puts-comments-in-pointers-last
 		       (lyskom-view-text-handle-saved-comments text-stat))
 
+		   ; Show footnotes if we should do that directly
+ 
+		   (if kom-show-footnotes-immediately
+		       (lyskom-traverse misc
+			   (text-stat->misc-info-list text-stat)
+			 (if (eq (misc-info->type misc) 'FOOTN-IN)
+			     ;; Show the footnote
+			     (progn
+			       (lyskom-view-text (misc-info->footn-in misc)
+						 mark-as-read t conf-stat priority)
+			       ; Mark as read (internally) s follow-comments won't add 
+			       ; it to the read list.
+			       (lyskom-is-read (misc-info->footn-in misc))))))
+
                    ;; Prefetch commented texts.
 
 		   (if (or follow-comments
@@ -397,8 +419,28 @@ when put in your `kom-view-text-hook'."
 	  (set-text->text-mass lyskom-view-text-text (substring body 0 sign))
 	  (lyskom-signal-reformatted-text 'reformat-signature))))))
 
+
 ;(add-hook 'kom-view-text-hook 'lyskom-filter-signature-hook)
 
+; review-rot13
+
+(defun lyskom-rot13-string (s)
+  (mapconcat 
+   (lambda (c)
+     (cond
+      ((and (>= c ?a) (<= c ?z)) (string (+ ?a (% (+ 13 (- c ?a)) 26))))
+      ((and (>= c ?A) (<= c ?Z)) (string (+ ?A (% (+ 13 (- c ?A)) 26))))
+      (t (string c))))
+   s ""))
+
+(defun lyskom-filter-rot13 ()
+  "Encrypts everything with rot-13. Least useless when put in your `kom-view-text-hook'."
+  (set-text->text-mass lyskom-view-text-text 
+		       (lyskom-rot13-string 
+			(text->text-mass lyskom-view-text-text)))
+  (lyskom-signal-reformatted-text 'reformat-rot13))
+
+
 (defun lyskom-view-text-convert-ISO-646-SE-to-ISO-8859-1 ()
   "Display r{ksm|rg}s as rksmrgs unless the text is an imported mail
 or we are reviewing without conversion. In other words, the characters
@@ -448,7 +490,6 @@ If MARK-AS-READ is non-nil the texts will be marked as read.
 PRIORITY the priority of the reading.
 If REVIEW-TREE is non-nil then build an entry of type 'REVIEW-TREE in the 
 lyskom-reading-list."
-  ;; . Find footnotes and show them.
   ;; . Fix the reading-list
   ;; . Issue cache-filling initiate-calls for everything left comments.
   
@@ -457,13 +498,6 @@ lyskom-reading-list."
         (mx-attachments-in (lyskom-get-text-attachments text-stat)))
     (lyskom-traverse misc (text-stat->misc-info-list text-stat)
       (cond
-       ((and (eq (misc-info->type misc) 'FOOTN-IN)
-	     (> (misc-info->footn-in misc) (text-stat->text-no text-stat))
-	     kom-show-footnotes-immediately)
-	;; Show the footnote
-	(lyskom-view-text (misc-info->footn-in misc)
-			  mark-as-read t conf-stat priority review-tree)
-	(lyskom-is-read (misc-info->footn-in misc)))
        ((eq (misc-info->type misc) 'FOOTN-IN)
 	(setq flist (cons (misc-info->footn-in misc) flist)))
        ((eq (misc-info->type misc) 'COMM-IN)
@@ -521,6 +555,13 @@ lyskom-reading-list."
      (string-to-number (aux-item->data el)))
    (lyskom-get-aux-item (text-stat->aux-items text-stat) 10101)))
 
+(defun lyskom-get-text-belongs-to (text-stat)
+  "Return a list of all texts TEXT-STAT is an attachment to."
+  (mapcar
+   (lambda (el) (string-to-number (aux-item->data el)))
+   (lyskom-get-aux-item (text-stat->aux-items text-stat) 10100)))
+
+
 (defun lyskom-skip-attachments (text-no mark-as-read)
   "Skip the hierarchy of attachments starting at text-no"
   (let ((queue (list text-no))
@@ -555,35 +596,6 @@ lyskom-reading-list."
           (lyskom-is-read-handler text-stat))))))
 
 
-(defun lyskom-fetch-text-for-cache (text-stat)
-  "Fetches the author and other things of a text and does nothing with it."
-  (cond
-   (text-stat
-    (lyskom-fetch-author-for-cache text-stat)
-    (initiate-get-text 'fill-cache nil (text-stat->text-no text-stat))
-    (lyskom-traverse misc (text-stat->misc-info-list text-stat)
-      (cond
-       ((eq (misc-info->type misc) 'FOOTN-IN)
-	(initiate-get-text-stat 'fill-cache 'lyskom-fetch-author-for-cache
-				(misc-info->footn-in misc)))
-       ((eq (misc-info->type misc) 'FOOTN-TO)
-	(initiate-get-text-stat 'fill-cache 'lyskom-fetch-author-for-cache
-				(misc-info->footn-to misc)))
-       ((eq (misc-info->type misc) 'COMM-IN)
-	(initiate-get-text-stat 'fill-cache 'lyskom-fetch-author-for-cache
-				(misc-info->comm-in misc)))
-       ((eq (misc-info->type misc) 'COMM-TO)
-	(initiate-get-text-stat 'fill-cache 'lyskom-fetch-author-for-cache
-				(misc-info->comm-to misc))))))))
-
-
-
-(defun lyskom-fetch-author-for-cache (text-stat)
-  "Fetches the author of a text and does nothing with it."
-  (if text-stat				;Error check.
-      (initiate-get-conf-stat 'fill-cache nil (text-stat->author text-stat))))
-
-
 
 (defun lyskom-text-read-p (text-stat)
   "Return t if TEXT-STAT has been marked as read in all
@@ -600,13 +612,12 @@ recipients to it that the user is a member in."
 	  ;; Is this function ever called asynchronously? If not, we
 	  ;; can use lyskom-get-membership istead.
 	  (let ((membership (lyskom-try-get-membership
-			     (misc-info->recipient-no misc-info) t))
+			     (misc-info->recipient-no misc-info) nil))
 		(loc-no (misc-info->local-no misc-info)))
 
 	    ;; Make a note that this text really is in a group we are
 	    ;; a member of.
 	    (if (and res membership) (setq res t))
-	    
 	    (if (and membership
 		     (> loc-no (membership->last-text-read membership))
 		     (not (lyskom-vmemq loc-no
@@ -639,7 +650,8 @@ blocking-do."
                                       (lyskom-try-get-membership
                                        (misc-info->recipient-no misc-item) t)
                                     (lyskom-get-membership
-                                     (misc-info->recipient-no misc-item) t)))
+                                     (misc-info->recipient-no misc-item) nil)))
+
                  (when membership
                    (setq is-member t)
                    (when (or (<= (misc-info->local-no misc-item)
@@ -727,7 +739,7 @@ blocking-do."
                    (if kom-show-author-at-end
                        (+ author-name-len 2) 
                      0)
-                   (string-width format-flag-string)))
+                   (lyskom-string-width format-flag-string)))
                (end-dash (if (> end-dash-chars 0)
                              (make-string end-dash-chars ?-)
                            "")))
@@ -830,7 +842,9 @@ blocking-do."
 Print an error message if TEXT-STAT or TEXT is nil.
 Mark the text as read if (and only if) MARK-AS-READ is non-nil.
 Args: TEXT-STAT TEXT MARK-AS-READ TEXT-NO FLAT-REVIEW."
-  (let ((lyskom-current-function 'lyskom-print-text))
+  (let* ((lyskom-current-function 'lyskom-print-text)
+         (cti (lyskom-get-aux-item (text-stat->aux-items text-stat) 1))
+         (content-type (and cti (aux-item->data (car cti)))))
     (cond
      ((or (null text)
           (null text-stat))
@@ -846,20 +860,24 @@ Args: TEXT-STAT TEXT MARK-AS-READ TEXT-NO FLAT-REVIEW."
 	     body)
 
         (cond
-         ((string-match "\n" str)
+         ((and (string-match "\n" str)
+               (not (equal "x-kom/user-area" content-type)))
           (setq lyskom-current-subject (substring str 0 (match-beginning 0)))
           (setq body (substring str (match-end 0)))
           (lyskom-insert-string 'head-Subject)
           (let ((lyskom-current-function-phase 'subject))
             (lyskom-format-insert "%#1r\n" 
-                                  (copy-sequence lyskom-current-subject)))
-            (lyskom-format-insert 
-             "%#2$%#1s\n"
-             (if kom-dashed-lines
-                 (make-string kom-text-header-dash-length ?-)
-               "")
-             (when kom-highlight-dashed-lines
-               '(face kom-dashed-lines-face)))
+                                  (cons text-stat 
+                                        (copy-sequence
+                                         lyskom-current-subject))))
+          (lyskom-format-insert 
+           "%#2$%#1s\n"
+           (if kom-dashed-lines
+               (make-string kom-text-header-dash-length ?-)
+             "")
+           (when kom-highlight-dashed-lines
+             `(face ,(or kom-dashed-lines-face
+                         lyskom-default-dashed-lines-face))))
 
           ;; (setq t1 (point-max))
 
@@ -880,13 +898,14 @@ Args: TEXT-STAT TEXT MARK-AS-READ TEXT-NO FLAT-REVIEW."
             (lyskom-format-insert "%#2$%#1t\n" 
                                   (cons text-stat body)
                                   (and kom-highlight-text-body
-                                       '(face kom-text-body-face)))
+                                       `(face ,(or kom-text-body-face
+                                                   lyskom-default-text-body-face))))
             )
 
 	  ;; Indicate that the text was truncated
 	  (if truncated
               (lyskom-signal-reformatted-text
-               '(reformat-truncated . (face kom-warning-face))))
+               `(reformat-truncated . (face ,kom-warning-face))))
 
 
           ;; (setq t2 (point-max))
@@ -898,11 +917,13 @@ Args: TEXT-STAT TEXT MARK-AS-READ TEXT-NO FLAT-REVIEW."
                (make-string kom-text-header-dash-length ?-)
              "")
            (when kom-highlight-dashed-lines
-             '(face kom-dashed-lines-face)))
+             `(face ,(or kom-dashed-lines-face
+                         lyskom-default-dashed-lines-face))))
           (lyskom-format-insert "%#2$%#1t\n"
                                 (cons text-stat str)
                                 (and kom-highlight-text-body
-                                       '(face kom-text-body-face)))
+                                       `(face ,(or kom-text-body-face
+                                                   lyskom-default-text-body-face))))
           (setq lyskom-current-subject "")))
         (sit-for 0)
         (let* ((lyskom-current-function-phase 'footer)
@@ -917,7 +938,8 @@ Args: TEXT-STAT TEXT MARK-AS-READ TEXT-NO FLAT-REVIEW."
                 (lyskom-format-insert "%#2$%#1s\n"
                                       lyskom-defer-indicator
                                       (when kom-highlight-dashed-lines
-                                        '(face kom-dashed-lines-face)))
+                                        `(face ,(or kom-dashed-lines-face
+                                                    lyskom-default-dashed-lines-face))))
                 (lyskom-defer-insertion
                  (lyskom-create-defer-info
                   'get-conf-stat
@@ -949,7 +971,8 @@ Args: TEXT-STAT TEXT MARK-AS-READ TEXT-NO FLAT-REVIEW."
                 kom-text-footer-format
                 lyskom-last-text-format-flags)
                (when kom-highlight-dashed-lines
-                 '(face kom-dashed-lines-face))))))
+                 `(face ,(or kom-dashed-lines-face
+                             lyskom-default-dashed-lines-face)))))))
         (if mark-as-read
             (lyskom-mark-as-read text-stat))
         (setq lyskom-previous-text lyskom-current-text)
@@ -967,23 +990,19 @@ the client. That is done by lyskom-is-read."
   (let ((misc-info-list (text-stat->misc-info-list text-stat)))
     (lyskom-traverse
      misc-info misc-info-list
-     (if (and (memq (misc-info->type misc-info) '(RECPT BCC-RECPT CC-RECPT))
+     (if (and (memq (misc-info->type misc-info) lyskom-recpt-types-list)
 	      ;; The whole membership list might not be fetched
 	      ;; yet. So we better mark it as read in all conferences.
 	      ;; (lyskom-member-p (misc-info->recipient-no misc-info))
 	      )
 	 (initiate-mark-as-read 'background 
-                                'lyskom-mark-as-read-callback
+                                nil
 				(misc-info->recipient-no misc-info)
 				(list (misc-info->local-no misc-info))
                                 text-stat
                                 (misc-info->recipient-no misc-info)
                                 (list (misc-info->local-no misc-info)))))))
 
-(defun lyskom-mark-as-read-callback (result text-stat recipient local-nos)
-  (lp--maybe-update-unreads recipient))
-
-
 (defun lyskom-print-header-recpt (conf-no misc)
   "Print a line of info about a recipient (or cc-recipient) of a text."
   (lyskom-format-insert "%#1s: %#2M <%#3d>\n"
@@ -1231,32 +1250,34 @@ Args: TEXT-STAT of the text being read."
       )
     author))
 
-(defun lyskom-mx-date-to-time (mx-date)
- "Attempt to convert MX-DATE to a lyskom time structure.
+(lyskom-with-external-functions (calendar-iso-from-absolute
+                                 calendar-absolute-from-gregorian)
+  (defun lyskom-mx-date-to-time (mx-date)
+    "Attempt to convert MX-DATE to a lyskom time structure.
 Returns the time structure if successful, otherwise nil."
-  (if (and mx-date
-           (condition-case nil
-               (progn (require 'calendar)
-                      (require 'cal-iso)
-                      t)
-             (error nil))
-           (string-match "\\([0-9][0-9][0-9][0-9]\\)-\\([0-9][0-9]\\)-\\([0-9][0-9]\\) \\([0-9][0-9]\\):\\([0-9][0-9]\\):\\([0-9][0-9]\\) \\([-+][0-9][0-9]\\)?\\([0-9][0-9]\\)?"
-                         (aux-item->data mx-date)))
-      (let* ((secs (string-to-number (match-string 6 (aux-item->data mx-date))))
-             (mins (string-to-number (match-string 5 (aux-item->data mx-date))))
-             (hour (string-to-number (match-string 4 (aux-item->data mx-date))))
-             (mday (string-to-number (match-string 3 (aux-item->data mx-date))))
-             (mon  (string-to-number (match-string 2 (aux-item->data mx-date))))
-             (year (string-to-number (match-string 1 (aux-item->data mx-date))))
-             (tzhr (match-string 7 (aux-item->data mx-date)))
-             (tzmin (or (match-string 8 (aux-item->data mx-date)) ""))
-             (wday (abs
-                    (elt 
-                     (calendar-iso-from-absolute
-                      (calendar-absolute-from-gregorian
-                       (list mon mday year)))
-                     1))))
-        (lyskom-create-time secs mins hour mday mon year wday 0 nil tzhr tzmin))))
+    (if (and mx-date
+             (condition-case nil
+                 (progn (require 'calendar)
+                        (require 'cal-iso)
+                        t)
+               (error nil))
+             (string-match "\\([0-9][0-9][0-9][0-9]\\)-\\([0-9][0-9]\\)-\\([0-9][0-9]\\) \\([0-9][0-9]\\):\\([0-9][0-9]\\):\\([0-9][0-9]\\) \\([-+][0-9][0-9]\\)?\\([0-9][0-9]\\)?"
+                           (aux-item->data mx-date)))
+        (let* ((secs (string-to-number (match-string 6 (aux-item->data mx-date))))
+               (mins (string-to-number (match-string 5 (aux-item->data mx-date))))
+               (hour (string-to-number (match-string 4 (aux-item->data mx-date))))
+               (mday (string-to-number (match-string 3 (aux-item->data mx-date))))
+               (mon  (string-to-number (match-string 2 (aux-item->data mx-date))))
+               (year (string-to-number (match-string 1 (aux-item->data mx-date))))
+               (tzhr (match-string 7 (aux-item->data mx-date)))
+               (tzmin (or (match-string 8 (aux-item->data mx-date)) ""))
+               (wday (abs
+                      (elt 
+                       (calendar-iso-from-absolute
+                        (calendar-absolute-from-gregorian
+                         (list mon mday year)))
+                       1))))
+          (lyskom-create-time secs mins hour mday mon year wday 0 nil tzhr tzmin)))))
 
 ;;; Local Variables: 
 ;;; eval: (put 'lyskom-traverse 'lisp-indent-hook 2)
